// ABone.cpp: implementation of the ABone class.
//
//////////////////////////////////////////////////////////////////////

#include "ABone.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//ABone** ABone::boneReg=NULL;
//int ABone::boneRegSize=0;

ABone::ABone(ANMBoneAtom* bone)
{
	matDump = false;
	compDump = false;

	_IsDirty=false;
	_parent=NULL;
	_outBuff=NULL;
	isQuat = true;

	dumpBBA = false;

	_boneID=bone->_boneId;
	_parentID=bone->_parentId;

	_rot=bone->_boneTransform->getRotation();
	//matrix4 rot;
	//_rot=rot.makeQuat(_rot);
	//_rot.z=-_rot.z;
	_scRot=bone->_boneTransform->getScRotation();
	//_scRot=rot.makeQuat(_scRot);
	//_scRot.q=-_scRot.q;
	_cen=bone->_boneTransform->getCenter();
	_trans=bone->_boneTransform->getTrans();
	_scale=bone->_boneTransform->getScale();
	//_scale.z=-_scale.z;

	/*_orot=_rot;
	_oscRot=_scRot;
	_ocen=_cen;
	_otrans=_trans;
	_oscale=_scale;*/

	
}

void ABone::CalculateT0()
{
	t0=GetMatrix();
	//t0.DumpToFile("Matrix t0",_boneID,_parentID);
	//t0i=t0.Invert();
	if (_parent)
		t0=t0*_parent->t0;
	//t0.DumpToFile("Matrix t0 multiplied by parent",_boneID,_parentID);
	t0i=t0.Invert();
	//t0i.DumpToFile("Matrix t0 inverted",_boneID,_parentID);

	for (int i=0;i<_children.size();i++)
	{
		ABone* child=_children[i];
		child->CalculateT0();
	}
}

void ABone::SlerpQuats( __vec4f qa1, __vec4f qa2, __vec4f *qr, float u )
{
	// Calculate angle beteen them.
	float angle = qa1.q * qa2.q + qa1.x * qa2.x + qa1.y * qa2.y + qa1.z * qa2.z;
	if ( angle > 1.0 ) angle = 1.0;
	if ( angle < -1.0 ) angle = -1.0;

	if ( angle < 0.0f )
	{
		angle = -angle;
		qa1.x = -qa1.x;
		qa1.y = -qa1.y;
		qa1.z = -qa1.z;
		qa1.q = -qa1.q;
	}

	float scale, invscale;

	if ((angle + 1.0f) > 0.05f) 
    {
        if ((1.0f - angle) >= 0.05f)  // spherical interpolation
        {
                float theta = acos(angle);
                float invsintheta = 1.0f / sin(theta);
                scale = sin(theta * (1.0f-u)) * invsintheta;
                invscale = sin(theta * u) * invsintheta;
        }
        else // linear interploation
        {
                scale = 1.0f - u;
                invscale = u;
        }
    }
    else 
    {
        qa2.x = -qa1.y;
		qa2.y = qa1.x;
		qa2.z = -qa1.q;
		qa2.q = qa1.z;		
        scale = sin(3.14159265358979323846f * (0.5f - u));
        invscale = sin(3.14159265358979323846f * u);
    }

	qr->x = ( qa1.x * scale + qa2.x * invscale );
	qr->y = ( qa1.y * scale + qa2.y * invscale );
	qr->z = ( qa1.z * scale + qa2.z * invscale );
	qr->q = ( qa1.q * scale + qa2.q * invscale );
}

void ABone::InterpolateCoord( SFVec3f *op1, SFVec3f *op2, SFVec3f *res, float u)
{
	if ( u == 1.0f )
	{
		res->x = op2->x;
		res->y = op2->y;
		res->z = op2->z;
		return;
	}
	res->x = op1->x + ( op2->x - op1->x ) * u;
	res->y = op1->y + ( op2->y - op1->y ) * u;
	res->z = op1->z + ( op2->z - op1->z ) * u;
}

void ABone::UpdateData( CBBABone* cBone, unsigned int* keyIndex )
{
	float no=1e-5;
	float step;

	int count=1, keyCount;

	SFVec3f cen, trans, scale;
	SFRotation rot, scRot;

	CBoneData* bData;
	if (cBone->boneDCTData)
	{
		bData = cBone->boneDCTData;
		count = 16;
	}
	else
		bData=&(cBone->boneData);
	
	int i;
	int j;

	char name[100];
	
	for ( i = 0; i < count; i++ )
	{
		if ( keyIndex[i] )
		{
			cen = _cen;
			trans = _trans;
			scale = _scale;
			scRot = _scRot;
			rot = _rot;
			step = 1.0f / ( keyIndex[i] + 1 );
		}

		for ( j = 0; j <= keyIndex[i]; j++ )
		{
			if ( i > 0 || j > 0 )
				AddFrame();

			if ( dumpBBA )
			{
				sprintf( name, "BBAText.%0.2d.txt.bba",  _frames.size() - 1 );
			}
	 
			if  (cBone->isTranslation())
			{
				_IsDirty=true;
				_trans.x = bData[i].TonX*no;
				_trans.y = bData[i].TonY*no;
				_trans.z = bData[i].TonZ*no;
			}
			if (cBone->isRotation())
			{
				_IsDirty=true;
				isQuat = cBone->boneMask.isQuaternion;
				if (isQuat)
				{
					if ( keyIndex[i] == 0 )
					{
						_rot.x = bData[i].RonQx*no;
						_rot.y = bData[i].RonQy*no;
						_rot.z = bData[i].RonQz*no;
						_rot.q = bData[i].RonQw*no;
					}
					else
					{
						SFRotation tmp;
						tmp.x = bData[i].RonQx*no;
						tmp.y = bData[i].RonQy*no;
						tmp.z = bData[i].RonQz*no;
						tmp.q = bData[i].RonQw*no;
						SlerpQuats( rot, tmp, &_rot, (j + 1) * step );
					}
					if ( dumpBBA )
					{
						FILE* f;
						f = fopen( name, "a" );
						fprintf( f, "%d %d %d %d ", (int)(_rot.x*1e5), (int)(_rot.y*1e5), (int)(_rot.z*1e5), (int)(_rot.q*1e5) );
						fclose(f);
					}
				}
				else
				{
					_rote.x = bData[i].RonAx1*no;
					_rote.y = bData[i].RonAx2*no;
					_rote.z = bData[i].RonAx3*no;
				}
			}
			if (cBone->isScale())
			{
				_IsDirty=true;
				_scale.x = bData[i].SonX*no;
				_scale.y = bData[i].SonY*no;
				_scale.z = bData[i].SonZ*no;
				//_scale.z=-_scale.z;
			}
			if (cBone->isScaleOrientation())
			{
				_IsDirty=true;
				_scRot.x = bData[i].SOonAxX*no;
				_scRot.y = bData[i].SOonAxY*no;
				_scRot.z = bData[i].SOonAxZ*no;
				_scRot.q = bData[i].SOonVal*no;
			}
			
			if (cBone->isCenter())
			{
				_IsDirty=true;
				if ( keyIndex[i] == 0 )
				{
					_cen.x = bData[i].ConX*1e-2;
					_cen.y = bData[i].ConY*1e-2;
					_cen.z = bData[i].ConZ*1e-2;
				}
				else
				{
					SFVec3f tmp;
					tmp.x = bData[i].ConX*1e-2;
					tmp.y = bData[i].ConY*1e-2;
					tmp.z = bData[i].ConZ*1e-2;
					InterpolateCoord( &cen, &tmp, &_cen, (j + 1) * step );
				}
				if ( dumpBBA )
				{
					FILE* f;
					f = fopen( name, "a" );
					fprintf( f, "%d %d %d ", (int)(_cen.x*1e2), (int)(_cen.y*1e2), (int)(_cen.z*1e2) );
					fclose(f);
				}
			}
			if (_parent)
				*(_frames[_frames.size()-1]) = GetMatrix() * _parent->GetFrame( _frames.size() - 1 );
			else
				*(_frames[_frames.size()-1]) = GetMatrix();
			
		}
	}
}

matrix4 ABone::GetMatrix()
{
	matrix4 rot,scRot,cen,trans,scale;

	/*SFRotation tmp=rot.unMakeQuat(_rot);
	tmp.x=_rot.z;
	tmp.y=_rot.y;
	tmp.z=_rot.x;
	tmp.q=_rot.q;*/
	if (isQuat)
		rot.Rotation(_rot);//);rot.makeQuat(tmp)
	else
		rot.Rotation(_rote);

	scRot.Rotation(_scRot);
	cen.Translation(_cen);
	trans.Translation(_trans);
	scale.Translation(_scale);

	matrix4 rez;
	//rez=cen.Invert();
	//rez=rez*scRot.Invert();
	//rez=rez*scale;
	//rez=rez*scRot;
	//rez=rez*cen;
	//rez=rez*cen.Invert();	
	rez=rez*rot;
	rez=rez*cen;
	
	//rez=rez*cen;

	return rez;
}

void ABone::AddFrame()
{
	matrix4* mat;
	if (_frames.size()==0)
		mat=new matrix4(t0);//GetMatrix());
	else
		mat=new matrix4(*(_frames[_frames.size()-1]));
	_frames.push_back(mat);
}

matrix4 ABone::GetFrame(int index)
{
	if (index<_frames.size())
		return *(_frames[index]);
	return NULL;
}

void ABone::CalculateMatrix(LPBYTE matBuff, int frameNo, bool invers)
{
	//matrix4 locm((float*)(matBuff+_boneID*16*sizeof(float)));
	matrix4	parm((float*)(matBuff));//+_parentID*16*sizeof(float)));

	matrix4 rez;

	if ( matDump && _boneID == 15 && frameNo == 5)
	{
		t0i.DumpToFile("test", _boneID, _parentID);
		matDump = false;
	}

	//_scale.x = -1.f;
	//_scale.y =  1.f;
	//_scale.z =  1.f;
	//
	//rez.Scale( _scale );

	if ( invers ) //check to multiply by the invers BIFS matrix
		rez=rez*t0i;

	rez=rez*GetFrame(frameNo);
	rez=rez*parm;

	rez.Assign( ( float* )( matBuff + _boneID * 16 * sizeof( float ) ) );
	
	//rez.Assign(locm);
	
}

void ABone::UpdateMatrix(LPBYTE matBuff, int frameNo, bool dirty, bool invers)
{
	if (dirty=(dirty || _IsDirty))
	{
		CalculateMatrix(matBuff,frameNo);
	}
	int i;
	for (i=0;i<_children.size();i++)
	{
		_children[i]->UpdateMatrix(matBuff,frameNo, dirty, invers);
	}	
}

void ABone::ClearData()
{
	for ( int i = 1; i < _frames.size();  )
	{
		delete _frames[0];
		_frames.pop_front();
	}
}

ABone::~ABone()
{
	for ( int i = 0; i < _frames.size(); i++ )
		delete _frames[i];
	_frames.clear();
}
