// BBADecPC.cpp : Defines the entry point for the application.
//
#include "BBAInclude.h"
#include "BBATypeDefs.h"
#include "BBADecoderLib.h"
#include <stdlib.h>
#include <memory.h>
#include "CBBABone.h"
#include  "_ANMAtom.h"
#include "Ostream.h"
#include "ABone.h"
#include "BBADataExchange.h"
#include "Animator.h"
#include "sys/timeb.h"

using namespace BIFS;

void* Animator::ParseANMBuffer(unsigned char* anmBuffer, int anmBufferSize)
{
	
	ANMAnimationAtom*  anmatom=NULL;
	ANMSkeletonAtom*  skeleton=NULL;
	ANMBoneAtom*  bone=NULL;
	ANMTransformAtom*  transform=NULL;
	ostream os;
	os.write((const char *)anmBuffer ,anmBufferSize );
	os.startBuffer ();
/////parser here
	
		unsigned int type;

		type=os.readInt(3);
		if (type =='anm')
		{
			//FILE* f;
			//f=fopen("c:\\anm.txt","w");
	
			anmatom= new ANMAnimationAtom ();
			os.moveForward (13);
			//type=os.readInt(3);
			int i=0;
			while (os.readInt(3)=='skl') 
			{
				//fprintf(f,"\nFrame %d\n",i++);
				skeleton= new ANMSkeletonAtom ();
				anmatom->AddSkeleton (skeleton);
				os.moveForward (8);
				int bonesNo=os.readInt(4);
				for (int j=0;j<bonesNo;j++)
				{
		
					int boneId=os.readInt (4);
					int parentId=os.readInt (4);
					type=os.readInt(3);
					os.moveForward (8);
					if (type=='trn')  
					{
						//read rotation
						float rotX=os.readFloat(4);
						float rotY=os.readFloat(4);
						float rotZ=os.readFloat(4);
						float rotQ=os.readFloat(4);

						//fprintf(f,"Rotation : x=%f, y=%f, z=%f, w=%f\n",rotX,rotY,rotZ,rotQ);

						//read scaleOrientation
						float scRotX=os.readFloat(4);
						float scRotY=os.readFloat(4);
						float scRotZ=os.readFloat(4);
						float scRotQ=os.readFloat(4);
						//read center
						float cenX=os.readFloat(4);
						float cenY=os.readFloat(4);
						float cenZ=os.readFloat(4);

						//fprintf(f,"Center : x=%f, y=%f, z=%f\n",cenX,cenY,cenZ);

						//read translation
						float transX=os.readFloat(4);
						float transY=os.readFloat(4);
						float transZ=os.readFloat(4);
						//read scale
						float scaleX=os.readFloat(4);
						float scaleY=os.readFloat(4);
						float scaleZ=os.readFloat(4);

						transform=new ANMTransformAtom(rotX,rotY,rotZ,rotQ,scRotX,scRotY,scRotZ,scRotQ,cenX,cenY,cenZ,transX,transY,transZ,scaleX,scaleY,scaleZ);
						bone= new ANMBoneAtom(boneId,parentId,transform);
						skeleton->AddBone (bone);
					}
					else
					{
						//fclose(f);
						return NULL;
					}
				}
			}
			//fclose(f);
			//else return NULL;
			return anmatom;
		}
		else return NULL;

}

void Animator::CreateBoneTree()
{
	int i;

	for (i=1;i<=boneRegSize;i++)
	{
		if (boneReg[i]->getParentID()!=0)
		{
			ABone* parent=boneReg[boneReg[i]->getParentID()];
			parent->addChild(boneReg[i]);
			boneReg[i]->setParent(parent);
		}
	}
}



void Animator::FetchBoneData(void* cbone)
{
	CBBABone* cBone=(CBBABone*)cbone;
	boneReg[cBone->boneIdentifier+1]->UpdateData((CBBABone*)cBone, 0 );	
}


int Animator::InsertFrame( unsigned char* pInData, bool addFrame, unsigned int *keyIndex )
{
	DWORD len=0;
	CBBABone* cbone;
	void* pData = NULL;
	
	if( !pInData )
		return 0;

	int sumSize = 0;

	if (addFrame)
	{
		for (int i=0;i<boneRegSize;i++)
		{
			boneReg[i+1]->AddFrame();							
		}		
	}
	
	cbone = (CBBABone *)pInData;

	bool isDCT = false;	
	if ( cbone->boneDCTData )
		isDCT = true;

	FILE* f;
	if ( dumpBBA )
	{
		f = fopen( "BBATextMask.txt.bba", "w" );
		fprintf( f, "0 1 %d ", boneCount);
	}
	
	while (cbone)
	{
		if ( dumpBBA )
		{
			if ( cbone->isCenter()  )
				fprintf( f, "%d 0 1 1 1 1 1 1 0 0 1 1 1 1 ", cbone->boneIdentifier );
			else
				fprintf( f, "%d 0 1 1 1 1 1 1 0 0 0 ", cbone->boneIdentifier );
		}

		//FetchBoneData(cbone);
		boneReg[cbone->boneIdentifier+1]->UpdateData(cbone, keyIndex);
		cbone=cbone->suiv;
	}

	int cnt = 0;

	if ( addFrame )
		if ( isDCT )
		{
			for ( int i = 0; i < SEG_LENGTH; i++ )
				cnt += keyIndex[i] + 1;
		}
		else
		{
			frmTot += keyIndex[0] + 1;
			cnt += keyIndex[0] + 1;
		}

	frameCount += cnt;
	//fprintf( f, "\n" );
	if ( dumpBBA )
	{
		fclose(f);
	}
	return cnt;
}

void Animator::InitialiseDecoder(unsigned char* anmBuffer0, int anmBuffer0Size)
{

	ANMAnimationAtom* anmatom = (ANMAnimationAtom*)ParseANMBuffer(anmBuffer0, anmBuffer0Size);

	boneRegSize=anmatom->getBoneCount();
	boneCount = boneRegSize;
	boneReg=(ABone**)malloc((boneRegSize+1)*sizeof(ABone*));
	
	int i;
	for (i=0;i<boneRegSize;i++)
	{
		ANMBoneAtom* bone=anmatom->getBone(i);
		if (bone->_parentId==0)
		{
			rootID[rootCount]=bone->_boneId; 
			rootCount++;
		}
		boneReg[i+1]=new ABone(bone);
		boneReg[i+1]->dumpBBA = dumpBBA;
	}

	delete anmatom;

	CreateBoneTree();
	for (i=0;i<rootCount;i++)
	{
		boneReg[rootID[i]]->CalculateT0();
	}

	frame = ( float* )malloc( 16 * ( boneRegSize + 1 ) * sizeof(float) );
	memset( frame, 0, 16 * ( boneRegSize + 1 ) * sizeof(float) );

	InsertFrame0();
}

void Animator::InsertFrame0()
{
	for (int i=1;i<=boneRegSize;i++)
		boneReg[i]->AddFrame();

	frameCount++;
}


bool Animator::CheckStartCode()
{	
	int sc;
	BBADecoderLib* myDecoder=(BBADecoderLib*)bbaDecoder;

	sc = myDecoder->getbits(32);
	//start decode
	
	if (sc!=BBA_OBJECT_START_CODE)
	{
		myDecoder->bbaop.stream.p-=4;
		return false;
	}
	
	return true;
}

void Animator::DecodeStream(unsigned char * bbaBuffer, int bbaBufferSize)
{
	LPBYTE  bbabp;
	int bbabpSize,len;
	

	bbabpSize = 150*sizeof(CBBABone);
	bbabp=(LPBYTE) malloc (bbabpSize);
	BBADecoderLib* myDecoder=(BBADecoderLib*)bbaDecoder;
	
	myDecoder->bba_decoder(bbaBuffer,0,bbaBufferSize,bbabp,len);
	InsertFrame(bbabp, false, 0);

	free(bbabp);
}

int Animator::DecodeHeader(unsigned char * bbaBuffer, int bbaBufferSize)
{
	BBADecoderLib dec;
	dec.initStream( bbaBuffer, bbaBufferSize );
	dec.decode_bba_object_plane_header();
	dec.isRunOnce = false;

	if ( framerate == 0 )
		if (dec.bbaop.is_intra)
			if (dec.bbaop.is_frame_rate)
				framerate = dec.bbaop.frame_rate;

	if ( dec.bbaop.is_intra )
		if( dec.bbaop.is_time_code )
			if ( dec.bbaop.time_code_hours >= 24 )
				return dec.bbaop.time_code_seconds;
	return -1;
}

void Animator::InitStream(unsigned char * bbaBuffer, int bbaBufferSize, bool checkAllIntra)
{
	BBADecoderLib* myDecoder=(BBADecoderLib*)bbaDecoder;
	
	myDecoder->initStream(bbaBuffer,bbaBufferSize);
	if (checkAllIntra)
		all_Intra = myDecoder->bba_allIntra();
	else
	{
		all_Intra = false;
		myDecoder->all_Intra = false;
	}

	BBASize = bbaBufferSize;
}


int Animator::InsertData(bool addFrame, int time)
{
	BBADecoderLib* myDecoder=(BBADecoderLib*)bbaDecoder;
	unsigned int* frameIndex;

	if ( myDecoder->bbaop.bba_object_coding_type == PREDICTIVE_CODING_TYPE )
		frameIndex = (unsigned int *)&(myDecoder->bbaop.KeyFrameIndex);
	else
		frameIndex = (unsigned int *)(myDecoder->bbaop.KeyFrameIndexDCT);

	int morphCnt = 0;
	int boneCnt = 0;
	
	if ( myDecoder->bbaop.firstMorph )
	{
		morphCnt = UpdateMorph((LPBYTE)myDecoder->bbaop.firstMorph, frameIndex);
	}

	if ( myDecoder->bbaop.firstBone )
		boneCnt = InsertFrame((LPBYTE)myDecoder->bbaop.firstBone, addFrame, frameIndex );
	else
		if ( morphCnt>0 )
		{
			boneCnt = morphCnt;
			for ( int j = 0; j < morphCnt; j++) 
				for (int i=0;i<boneRegSize;i++)
				{
					boneReg[i+1]->AddFrame();							
				}	
			frameCount += boneCnt;
		}
	return boneCnt;
}

int Animator::DecodeOneFrame(bool addFrame, int time)
{
	BBADecoderLib* myDecoder=(BBADecoderLib*)bbaDecoder;

	//bbabpSize = 150*sizeof(CBBABone);
	//bbabp=(LPBYTE) malloc (bbabpSize);

	_timeb start, stop;
	_ftime( &start );

	if ( myDecoder->decode_bba_object_plane_header() )
		return 0;

	if ( framerate == 0 )
		if (myDecoder->bbaop.is_intra)
			if (myDecoder->bbaop.is_frame_rate)
				framerate = myDecoder->bbaop.frame_rate;
    myDecoder->decode_bba_object_plane_data();

	_ftime( &stop );

	int fr = (stop.time * 1000 + stop.millitm) - (start.time * 1000 + start.millitm);
	decodeTime += fr;

	//myDecoder->WriteDecodedData(bbabp ,len);
	
	InsertData(addFrame, time);

	//Log_Write( MYLOG_TRACE, "%d", myDecoder->bbaop.KeyFrameIndex);
	//free(bbabp);
	return 0;
}

void Animator::BBA_TO_Matrix(unsigned char * bbaBuffer, int bbaBufferSize, unsigned char* anmBuffer0, int anmBuffer0Size)
{
	InitialiseDecoder(anmBuffer0,anmBuffer0Size);

	InitStream(bbaBuffer,bbaBufferSize);
	
	if (!CheckStartCode()) 
		return;

	unsigned int PTS=0;
	
	int fr=0;
	
	BBADecoderLib* myDecoder=(BBADecoderLib*)bbaDecoder;

	/*FILE* f;
	f = fopen("c:\\interp.txt", "w");	
	fclose(f);*/

	do 
	{
		DecodeOneFrame();
		fr++;
	}
	while(myDecoder->nextbits_bytealigned()!=BBA_OBJECT_END_CODE);

	deleteDecoder();
}

void Animator::FreeBBADecoderData()
{
	BBADecoderLib* myDecoder=(BBADecoderLib*)bbaDecoder;
	myDecoder->FreeData();
}

void Animator::deleteDecoder()
{
	if ( bbaDecoder )
	{
		FreeBBADecoderData();
		BBADecoderLib* myDecoder=(BBADecoderLib*)bbaDecoder;
		delete myDecoder;
		bbaDecoder = NULL;
	}
}

void Animator::BBA_TO_ANM(unsigned char * bbaBuffer, int bbaBufferSize, unsigned char* anmBuffer0, int anmBuffer0Size,unsigned char** anmBuffer, int* anmBufferSize)
{

	BBADecoderLib myDecoder;
	ANMAnimationAtom*  anmatom=NULL;
	
	int n=0, n_bap=0, len;
	int bbabpSize,sc;
	unsigned int PTS=0;
	LPBYTE  bbabp;
	bbabpSize = 150*sizeof (CBBABone);
	bbabp=(LPBYTE) malloc (bbabpSize);

	myDecoder.init();

	/* read whole stream from a input buffer in a bbaop.stream.buf */
	myDecoder.initStream(bbaBuffer,bbaBufferSize);

	myDecoder.initbits();
	sc = myDecoder.getbits(32);

	//start decode
	int fr=0;
	if (sc!=BBA_OBJECT_START_CODE)
	{
		//AfxMessageBox("start code not found");
		return;
	}
	do 
	{
		myDecoder.decode_bba_object_plane_header();
		myDecoder.decode_bba_object_plane_data();
		myDecoder.WriteDecodedData(bbabp ,len);
		fr++;

		if (anmatom==NULL) 
			anmatom= (ANMAnimationAtom*)ParseANMBuffer(anmBuffer0, anmBuffer0Size);

		anmatom->InsertFrame (bbabp);  //insert frame by frame in anmatom, 1 frame=1 skeleton atom
	}
	while(myDecoder.nextbits_bytealigned()!=BBA_OBJECT_END_CODE);

	//make output buffer from anmAtom

	BIFS::ostream st;
	anmatom->Write(&st);
	st.GetData((char**)& (*anmBuffer),(*anmBufferSize));

	delete anmatom;
}

void Animator::Init()
{
	frame = NULL;
	rootCount=0;
	frameCount = 0;
	boneCount = 0;
	framerate = 0;
	decodeTime = 0;
	boneReg = NULL;
	boneRegSize = 0;
	dumpBBA = false;
	bbaDecoder = NULL;
	frmTot = 0;
}

void Animator::clearBoneData()
{
	frameCount = 1;
	for ( int i = 0; i < boneRegSize; i++ )
	{
		boneReg[i+1]->ClearData();
	}	
}

Animator::Animator()
{
	Init();
	outType = OUT_MAT;
	bbaDecoder=new BBADecoderLib();
	((BBADecoderLib*)bbaDecoder)->init();
}

Animator::Animator(OUT_TYPE outputType, unsigned char * bbaBuffer, int bbaBufferSize, unsigned char* anmBuffer0, int anmBuffer0Size)
{
	Init();
	outType = outputType;
	bbaDecoder=new BBADecoderLib();
	((BBADecoderLib*)bbaDecoder)->init();

	switch (outType)
	{
	case OUT_MAT:
		{
			BBA_TO_Matrix(bbaBuffer,bbaBufferSize,anmBuffer0,anmBuffer0Size);
			break;
		}
	case OUT_ANM:
		{
			BBA_TO_ANM(bbaBuffer,bbaBufferSize,anmBuffer0,anmBuffer0Size,&anmBuff,&anmBuffSize);
			break;
		}
	}
}

float* Animator::GetFrame(int index, float* trMat, bool invers)
{
	if (outType!=OUT_MAT) 
		return NULL;
		
    memcpy(frame,trMat,16*sizeof(float));
	/*memset(frame,0,16*sizeof(float));
	for (int i=0;i<4;i++)
		frame[i*4+i]=1;*/
	for (int i=0;i<rootCount;i++)
		boneReg[rootID[i]]->UpdateMatrix( (LPBYTE)frame, index, invers );
	return frame;
}


Animator::~Animator()
{
	if ( boneReg )
	{
		for (int i=0;i<boneRegSize;i++)
		{
			delete boneReg[i+1];
		}
		free(boneReg);

		if (outType==OUT_ANM)
		{
			free(anmBuff);
		}
		if (frame)
			free( frame );
	}
	if (bbaDecoder)
		delete (BBADecoderLib*)bbaDecoder;
}

void Animator::FindLeafBones( int* &ids, int &count )
{
	ids = (int*) malloc( boneRegSize * sizeof(int) );
	count = 0;
	for (int i=1;i<=boneRegSize;i++)
	{
		if ( boneReg[i]->GetChildrenCount() == 0 )
		{
			ids[count++] = boneReg[i]->getBoneID();			
		}
	}
}