/********************************************************************************/
/* This software module was developped by										*/

/*	   Marius Preda (MP), marius.preda@int-evry.fr,								*/
/*                        Institut National des Telecommunications, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

#ifndef _CBBA_MORPH_HEADER_
#define _CBBA_MORPH_HEADER_

#include<stdio.h>
#include <memory.h>

class CBBAMorphMask {
public:
	int wcnt;
	bool *isWeightChanged;

	CBBAMorphMask()
	{
		wcnt=0;
	}

	CBBAMorphMask& operator=(const CBBAMorphMask &t)
	{
		if (wcnt && t.wcnt)
			memcpy(isWeightChanged,t.isWeightChanged,wcnt*sizeof(bool));
		return *this;
	}

	inline void Free(){
		wcnt=0;
			free(isWeightChanged);
	}

};

class CBBAMorphData{
public:
	int wcnt;
	int *Weights;

	CBBAMorphData()
	{
		wcnt=0;
	}

	CBBAMorphData& operator=(const CBBAMorphData &t)
	{
		if (wcnt && t.wcnt)
			memcpy(Weights,t.Weights,wcnt*sizeof(bool));
		return *this;
	}

	inline void Free(){
		wcnt=0;
			free(Weights);
	}
};

class CBBAMorph
{
public:

	int morphIdentifier;
	int NumberWeights;
	bool isInCurrentFrame;
	bool canBePredicted;
	CBBAMorphMask morphMask;
	CBBAMorphData morphData;

	int *morphDataExchange;// morphIdentifier, NumberWeights, isDCT, morphMask[0], morphData[0], ...
						   // morphMask[n], morphData[n],
	
	CBBAMorphData *morphDCTData;

	CBBAMorphData morphDataPreviousFrame;
	CBBAMorphData morphDataIMax, morphDataIMin;
	CBBAMorphData morphDataPMax, morphDataPMin;

	CBBAMorphData Irange_max, Irange_min;
  	CBBAMorphData range_max , range_min;

	CBBAMorph *prec;
	CBBAMorph *suiv;

// operations
	inline bool isWeightsChanged(){
		for (int i=0;i<NumberWeights;i++)
			if (morphMask.isWeightChanged[i])
				return true;
		return false;
	}

	CBBAMorph(int bID);
	CBBAMorph();
	~CBBAMorph();

	void AllocMorph();
	void AllocMorph(int nw);
	void AllocMinMaxMorph();
	void AllocPrevMorph();
	void AllocDCTMorph();
	void FreeMorph();
	void FreeMinMaxMorph();
	void FreePrevMorph();
	void FreeDCTMorph();
	int GetSize();
	int GetSizeExchange();
	void LoadDataExchange();

};


CBBAMorph *getnodindexi(int index,CBBAMorph * &first,CBBAMorph* &last);
CBBAMorph *getnodIDi(int ID,CBBAMorph* &first,CBBAMorph* &last);

int innod(CBBAMorph*p,CBBAMorph* &first,CBBAMorph* &last);
void outnod(CBBAMorph * p,CBBAMorph* &first,CBBAMorph* &last);
void freeliste(CBBAMorph* &first,CBBAMorph* &last);
void affiche_liste(CBBAMorph*first);



#endif  // _CBBA_MORPH_HEADER_
