// _ANMAtom.cpp: implementation of the ANMAtom class.
//
//////////////////////////////////////////////////////////////////////
#include "_ANMAtom.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//Chain* ANMAtom::ANMList = NULL;

ANMAtom::ANMAtom()
{
	size	= 0;
	type	= 0;
	atom_id = 0;
}

//////////////////////////////////////////////////////////////////////////////////////

ANMAtom::~ANMAtom()
{
    int i;
    for (i=0;i<_children.size();i++)
	{
		ANMAtom* child=_children[i];
        delete child;
	}
}

//////////////////////////////////////////////////////////////////////////////////////

void ANMAtom::AddChild(ANMAtom *child)
{
    _children.push_back(child);
}

//////////////////////////////////////////////////////////////////////////////////////

void ANMAtom::Write(ostream *os)
{
    WriteHeader(os);
    WriteStuff(os);

    int i;
    for (i=0;i<_children.size();i++)
	{
		ANMAtom* child=_children[i];
        child->Write(os);
	}

    WriteFooter(os);
}

//////////////////////////////////////////////////////////////////////////////////////

void ANMAtom::WriteHeader(ostream *os)
{
    _start = os->tellp();
	os->write(reinterpret_cast<char *>(&type), 3);
    os->write(reinterpret_cast<char *>(&atom_id), 4);
    os->write(reinterpret_cast<char *>(&size), 4);
}

//////////////////////////////////////////////////////////////////////////////////////

void ANMAtom::WriteStuff(ostream *os)
{

}

void ANMAtom::WriteFooter(ostream *os)
{
    _end = os->tellp();
    size = _end - _start;

    os->seekp(_start + 7);
    os->write(reinterpret_cast<char *>(&size), 4);
    os->seekp(_end);
}

//blagica   TransformAtom////////////////////////////////////////////////////////////////////////////////////

ANMTransformAtom::ANMTransformAtom(float rotX,float rotY,float rotZ,float rotQ,float scRotX,float scRotY,float scRotZ,float scRotQ,float cenX,float cenY,float cenZ,float transX,float transY,float transZ,float scaleX,float scaleY,float scaleZ)
{
	type = 'trn';
	atom_id = 0;//addMap(ANMList,NULL);//->NodeID);
	_rot.x=rotX;
	_rot.y=rotY;
	_rot.z=rotZ;
	_rot.q=rotQ;
	_scRot.x=scRotX;
	_scRot.y=scRotY;
	_scRot.z=scRotZ;
	_scRot.q=scRotQ;
	_cen.x=cenX;
	_cen.y=cenY;
	_cen.z=cenZ;
	_trans.x=transX;
	_trans.y=transY;
	_trans.z=transZ;
	_scale.x =scaleX;
	_scale.y =scaleY;
	_scale.z =scaleZ;
}
			

ANMTransformAtom::~ANMTransformAtom()
{
}

void ANMTransformAtom::WriteStuff(ostream *os)
{
 	//write rotation
		os->write(reinterpret_cast<const char *>(&_rot.x),4);
		os->write(reinterpret_cast<const char *>(&_rot.y),4);
		os->write(reinterpret_cast<const char *>(&_rot.z),4);
		os->write(reinterpret_cast<const char *>(&_rot.q),4);
	//write scaleOrientation
		os->write(reinterpret_cast<const char *>(&_scRot.x),4);
		os->write(reinterpret_cast<const char *>(&_scRot.y),4);
		os->write(reinterpret_cast<const char *>(&_scRot.z),4);
		os->write(reinterpret_cast<const char *>(&_scRot.q),4);
	//write center
		os->write(reinterpret_cast<const char *>(&_cen.x),4);
		os->write(reinterpret_cast<const char *>(&_cen.y),4);
		os->write(reinterpret_cast<const char *>(&_cen.z),4);
	//write translation
		os->write(reinterpret_cast<const char *>(&_trans.x),4);
		os->write(reinterpret_cast<const char *>(&_trans.y),4);
		os->write(reinterpret_cast<const char *>(&_trans.z),4);
	//write scale
		os->write(reinterpret_cast<const char *>(&_scale.x),4);
		os->write(reinterpret_cast<const char *>(&_scale.y),4);
		os->write(reinterpret_cast<const char *>(&_scale.z),4);
}


//blagica   boneAtom////////////////////////////////////////////////////////////////////////////////////	

ANMBoneAtom::ANMBoneAtom(int boneID,int parentId,ANMTransformAtom* trans)
{
  _boneId=boneID;
  _parentId=parentId ;
  _boneTransform=trans;  
}

	
ANMBoneAtom::~ANMBoneAtom()
{
	delete _boneTransform;
}
	

void ANMBoneAtom::WriteStuff(ostream *os)
{

}

//blagica   skeletonAtom////////////////////////////////////////////////////////////////////////////////////

ANMSkeletonAtom::~ANMSkeletonAtom()
{
	for (int i=0; i<_bones.size(); i++)
		delete _bones[i];
	_bones.clear();
}
	

void ANMSkeletonAtom::WriteStuff(ostream *os)
{
	int nsize=_bones.size();
	os->write(reinterpret_cast<const char *>(&nsize),4);

	ANMBoneAtom* bone;

	int i;
    for (i=0;i<_bones.size();i++)
	{		
		bone=_bones[i];
		os->write(reinterpret_cast<const char *>(&(bone->_boneId)),4);
		os->write(reinterpret_cast<const char *>(&(bone->_parentId)),4);
		bone->_boneTransform->Write(os); 
	}
}


//blagica   AnimationAtom////////////////////////////////////////////////////////////////////////////////////

ANMAnimationAtom::ANMAnimationAtom()
{
//	ANMAtom::ANMList = NewChain();
	type = 'anm';	
	atom_id = 0;//addMap(ANMList,NULL);
	
	_numFrames=0;
    _localCoordinates=1;	
} 

ANMAnimationAtom::~ANMAnimationAtom()
{
	//deleteMap(ANMAtom::ANMList);	
}
	

void ANMAnimationAtom::WriteStuff(ostream *os)
{
	os->write(reinterpret_cast<const char *>(&(_numFrames)),4);
	os->write(reinterpret_cast<const char *>(&(_localCoordinates)),1);
}

ANMSkeletonAtom::ANMSkeletonAtom()
{
 	type = 'skl';
	atom_id = 0;//addMap(ANMList,NULL);//node->sgprivate);//->NodeID);
}


ANMSkeletonAtom::ANMSkeletonAtom(ANMSkeletonAtom* prevSkeleton)
{
	this->type =prevSkeleton->type;
	this->atom_id = 0;//addMap(ANMList,NULL);
	for (int i=0;i<prevSkeleton->_bones.size();i++)
		this->AddBone(new ANMBoneAtom(prevSkeleton->_bones[i]));
}

ANMBoneAtom::ANMBoneAtom(ANMBoneAtom* bone)
{
	if (bone)
	{
		this->_boneId =bone->_boneId;
		this->_parentId =bone->_parentId;
		this->_boneTransform =new ANMTransformAtom(bone->_boneTransform);
	}
}


ANMTransformAtom::ANMTransformAtom(ANMTransformAtom* transform)
{
	this->type ='trn';
	this->atom_id = 0;//addMap(ANMList,NULL);
	if (transform)
	{
		this->_cen =transform->_cen ;
		this->_rot =transform->_rot ;
		this->_scale =transform->_scale ;
		this->_scRot =transform->_scRot ;
		this->_trans =transform->_trans ;
	}
}


void ANMSkeletonAtom::AddBone (ANMBoneAtom* anmBone)
{
	_bones.push_back(anmBone);
}


void ANMAnimationAtom::AddSkeleton(ANMSkeletonAtom* skeleton)
{
	_frames.push_back(skeleton);
	AddChild(skeleton);
	_numFrames++;
}

void ANMTransformAtom::UpdateAtom(CBBABone* cBone)
{

	float no=1e-5;

	CBoneData* bData;
	if (cBone->boneDCTData)
		bData=cBone->boneDCTData;
	else
		bData=&(cBone->boneData);

	if (cBone->isCenter()) 
	{
		_cen.x = cBone->boneData.ConX*1e-2;
		_cen.y = cBone->boneData.ConY*1e-2;
		_cen.z = cBone->boneData.ConZ*1e-2;
	}
	 
	if  (cBone->isTranslation())
	{
		_trans.x  = cBone->boneData.TonX*no;
		_trans.y = cBone->boneData.TonY*no;
		_trans.z = cBone->boneData.TonZ*no;
	}
	if (cBone->isRotation())
	{
		_rot.x = cBone->boneData.RonQx*no;
		_rot.y = cBone->boneData.RonQy*no; 
		_rot.z = cBone->boneData.RonQz*no;
		_rot.q = cBone->boneData.RonQw*no;
	}
	if (cBone->isScale())
	{
		_scale.x = cBone->boneData.SonX*no;
		_scale.y = cBone->boneData.SonY*no;
		_scale.z = cBone->boneData.SonZ*no;
	}
	if (cBone->isScaleOrientation())
	{
		_scRot.x = cBone->boneData.SOonAxX*no;
		_scRot.y = cBone->boneData.SOonAxY*no;
		_scRot.z = cBone->boneData.SOonAxZ*no;
		_scRot.q = cBone->boneData.SOonVal*no;
	}
}

void ANMSkeletonAtom::UpdateBone(CBBABone* cBone)
{
	for (int i=0;i<_bones.size();i++)
	if ((cBone->boneIdentifier+1)==this->_bones[i]->_boneId)
	{		
		_bones[i]->_boneTransform->UpdateAtom(cBone);
		break;
    }
}

void ANMAnimationAtom::InsertFrame(LPBYTE pInData)
{
	ANMSkeletonAtom* skeleton;
	BYTE tag;
	DWORD len=0;
	CBBABone* cbone;
	void* pData = NULL;

	if( !pInData )
		return;

	int sumSize = 0;

	BBADataExchange dataExchange(pInData, true);
	int ntags = dataExchange.ParseNoTags();
	
	while(pData=dataExchange.FetchNext(tag,len))
	{
		sumSize += len;
		switch(tag)
		{
			case BBADataExchange::TAG_KEY_FRAME:
				{				
					ANMSkeletonAtom *prevSkeleton=_frames[_frames.size()-1];
					skeleton=new ANMSkeletonAtom(prevSkeleton);
 					AddSkeleton(skeleton); 
					break;
				}
			case BBADataExchange::TAG_BONES:
				{
					cbone = (CBBABone *)pData;
					FetchBoneData(cbone,skeleton);
					break;
				}
			default:
				break;
		}
	}
}


void ANMAnimationAtom::FetchBoneData(CBBABone* cbone, ANMSkeletonAtom* skeleton)
{
	//if( !cbone->boneDCTData )
	{
		// one bone
		skeleton->UpdateBone(cbone);
	}
	//else
	//{
	//	// 16 bones at once
	//	for(int t_=0; t_ < 16; t_++)
	//	{
	//		skeleton->UpdateBone(&cbone[t_]);			
	//	}
	//}
}

