// _ANMAtom.h: interface for the ANMAtom class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _ANMATOM_H_
#define _ANMATOM_H_

#ifndef BYTE
typedef unsigned char BYTE;
#endif

#ifndef LPBYTE
typedef BYTE* LPBYTE;
#endif

/// OPCATOM ************************************************************************


#include "BBAInclude.h"

#include "BBADataExchange.h"

//blagica   ANMAtom////////////////////////////////////////////////////////////////////////////////////

#define ostream BIFS::ostream

class ANMAtom
{
public:
    ANMAtom();
    virtual ~ANMAtom();
    void AddChild(ANMAtom *child);
	void Write(ostream *os);
	
	//static Chain* ANMList;

protected:
    unsigned int atom_id;
    unsigned int size;
    unsigned int type;
    void WriteHeader(ostream *os);
    virtual void WriteStuff(ostream *os);
    void WriteFooter(ostream *os);

	long _start, _end;

public:
    //std::vector<ANMAtom *> _children;
#ifdef _IMPORT_MP4SDK 
	std::vector 
#else
	BIFSvector 
#endif 
	<ANMAtom *> _children;
    
};


//blagica   transformAtom////////////////////////////////////////////////////////////////////////////////////
class ANMTransformAtom:public ANMAtom
{
public:
	ANMTransformAtom(ANMTransformAtom* transform=NULL);
	void  UpdateAtom(CBBABone *cBone);
	ANMTransformAtom(float rotX,float rotY,float rotZ,float rotQ,float scRotX,float scRotY,float scRotZ,float scRotQ,float cenX,float cenY,float cenZ,float transX,float transY,float transZ,float scaleX,float scaleY,float scaleZ);

	virtual ~ANMTransformAtom();
	void WriteStuff(ostream *os);

	__vec4f getRotation(){return _rot;}
	__vec4f getScRotation(){return _scRot;}
	__vec3f getCenter(){return _cen;}
	__vec3f getTrans(){return _trans;}
	__vec3f getScale(){return _scale;}


protected:
	__vec4f _rot;
	__vec4f _scRot;
	__vec3f _cen;
	__vec3f _trans;
	__vec3f _scale;
};

//blagica   boneAtom////////////////////////////////////////////////////////////////////////////////////
class ANMBoneAtom:public ANMAtom
{
public:
	ANMBoneAtom(ANMBoneAtom* bone=NULL);
	ANMBoneAtom(int boneID,int parentId,ANMTransformAtom* trans);
	virtual ~ANMBoneAtom();
	void WriteStuff(ostream *os);

	int _boneId,_parentId;
	ANMTransformAtom* _boneTransform;	
};

//blagica   skeletonAtom////////////////////////////////////////////////////////////////////////////////////
class ANMSkeletonAtom:public ANMAtom
{
public:
	
	ANMSkeletonAtom();
	ANMSkeletonAtom(ANMSkeletonAtom* prevSkeleton);
	virtual ~ANMSkeletonAtom();
	void WriteStuff(ostream *os);
	void AddBone (ANMBoneAtom * anmBone);
    void UpdateBone(CBBABone *cBone);

	ANMBoneAtom* getBone(int index){return _bones[index];}
	int getBoneCount(){return _bones.size();}

protected:
	
#ifdef _IMPORT_MP4SDK 
	std::vector 
#else
	BIFSvector 
#endif 
	<ANMBoneAtom*> _bones;	
};

//blagica   AnimationAtom////////////////////////////////////////////////////////////////////////////////////
class ANMAnimationAtom:public ANMAtom
{
public:
	ANMAnimationAtom();
	virtual ~ANMAnimationAtom();
	void WriteStuff(ostream *os);
	void InsertFrame (LPBYTE frameBuffer);
	void FetchBoneData(CBBABone* cbone, ANMSkeletonAtom* skeleton);
	void AddSkeleton(ANMSkeletonAtom * skeleton);

	ANMSkeletonAtom* getFrame(int index){return _frames[index];}
	ANMBoneAtom* getBone(int index){return _frames[0]->getBone(index);}
	int getBoneCount(){return _frames[0]->getBoneCount();}

protected:
	unsigned int _numFrames;
	int _localCoordinates;

#ifdef _IMPORT_MP4SDK 
	std::vector 
#else
	BIFSvector 
#endif 
	<ANMSkeletonAtom*> _frames;	
};

#undef ostream 

#endif // !defined(_ANMATOM_H_)
