// _ANMAtom.cpp: implementation of the ANMAtom class.
//
//////////////////////////////////////////////////////////////////////
#include "_ANMAtom.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Chain* ANMAtom::ANMList=NewChain();

ANMAtom::ANMAtom()
{

}

//////////////////////////////////////////////////////////////////////////////////////

ANMAtom::~ANMAtom()
{
    int i;
    for (i=0;i<_children.size();i++)
	{
		ANMAtom* child=_children[i];
        delete child;
	}
}

//////////////////////////////////////////////////////////////////////////////////////

void ANMAtom::AddChild(ANMAtom *child)
{
    _children.push_back(child);
}

//////////////////////////////////////////////////////////////////////////////////////

void ANMAtom::Write(ostream *os)
{
    WriteHeader(os);
    WriteStuff(os);

    int i;
    for (i=0;i<_children.size();i++)
	{
		ANMAtom* child=_children[i];
        child->Write(os);
	}

    WriteFooter(os);
}

//////////////////////////////////////////////////////////////////////////////////////

void ANMAtom::WriteHeader(ostream *os)
{
    _start = os->tellp();
	os->write(reinterpret_cast<char *>(&type), 3);
    os->write(reinterpret_cast<char *>(&atom_id), 4);
    os->write(reinterpret_cast<char *>(&size), 4);
}

//////////////////////////////////////////////////////////////////////////////////////

void ANMAtom::WriteStuff(ostream *os)
{

}

void ANMAtom::WriteFooter(ostream *os)
{
    _end = os->tellp();
    size = _end - _start;

    os->seekp(_start + 7);
    os->write(reinterpret_cast<char *>(&size), 4);
    os->seekp(_end);
}

//blagica   TransformAtom////////////////////////////////////////////////////////////////////////////////////

ANMTransformAtom::ANMTransformAtom(ANMTransformAtom* transform)
{
	this->type ='trn';
	this->atom_id = addMap(ANMList,NULL);
	if (transform)
	{
		this->_cen =transform->_cen ;
		this->_rot =transform->_rot ;
		this->_scale =transform->_scale ;
		this->_scRot =transform->_scRot ;
		this->_trans =transform->_trans ;
	}
}

ANMTransformAtom::ANMTransformAtom(float rotX,float rotY,float rotZ,float rotQ,float scRotX,float scRotY,float scRotZ,float scRotQ,float cenX,float cenY,float cenZ,float transX,float transY,float transZ,float scaleX,float scaleY,float scaleZ)
{
	type = 'trn';
	atom_id = addMap(ANMList,NULL);//->NodeID);
	_rot.x=rotX;
	_rot.y=rotY;
	_rot.z =rotZ;
	_rot.q=rotQ;
	_scRot.x=scRotX;
	_scRot.y=scRotY;
	_scRot.z=scRotZ;
	_scRot.q=scRotQ;
	_cen.x=cenX;
	_cen.y=cenY;
	_cen.z=cenZ;
	_trans.x=transX;
	_trans.y=transY;
	_trans.z=transZ;
	_scale.x =scaleX;
	_scale.y =scaleY;
	_scale.z =scaleZ;
}
			

ANMTransformAtom::ANMTransformAtom(M_SBBone* pBone)
{
	type = 'trn';

	atom_id = addMap(ANMList,pBone->sgprivate);//->NodeID);


	//matrix4 rot,scRot,cen,trans,scale;
	//rot.Rotation(pBone->rotation);
	//scRot.Rotation(pBone->scaleOrientation);
	//cen.Translation(pBone->center);
	//trans.Translation(pBone->translation);
	//scale.Translation(pBone->scale);

	//matrix4 rez;
	//rez=cen.Invert()*scRot.Invert();
	//rez=rez*scale;
	//rez=rez*scRot;
	//rez=rez*cen;
	//rez=rez*cen.Invert();
	//rez=rez*rot;
	//rez=rez*cen;
	//rez=rez*trans;
	
	//rez.Assign(_matrix);

	_rot=pBone->rotation;
	_scRot=pBone->scaleOrientation;
	_cen=pBone->center;
	_trans =pBone->translation;
	_scale =pBone->scale;
}

void ANMTransformAtom::UpdateAtom(CBone* cBone)
{

	if (cBone->isCenter) 
	{
	_cen.x = cBone->boneData.ConX ;
	_cen.y = cBone->boneData.ConY;
	_cen.z = cBone->boneData.ConZ ;
	}
	 
	if  (cBone->isTranslation)
	{
	_trans.x  = cBone->boneData.TonX ;
	_trans.y = cBone->boneData.TonY;
	_trans.z = cBone->boneData.TonZ ;
	}
	if (cBone->isRotation)
	{
	_rot.x = cBone->boneData.RonQx ;
	_rot.y = cBone->boneData.RonQy ;
	_rot.z = cBone->boneData.RonQz  ;
	_rot.q = cBone->boneData.RonQw ;
	}
	if (cBone->isScale )
	{
	_scale.x = cBone->boneData.SonX ;
	_scale.y = cBone->boneData.SonY;
	_scale.z = cBone->boneData.SonZ ;
	}
	if (cBone->isScaleOrientation )
	{
	_scRot.x = cBone->boneData.SOonAxX;
	_scRot.y = cBone->boneData.SOonAxY;
	_scRot.z = cBone->boneData.SOonAxZ ;
	_scRot.q = cBone->boneData.SOonVal ;
	}
}

ANMTransformAtom::~ANMTransformAtom()
{
}

void ANMTransformAtom::WriteStuff(ostream *os)
{
 	//int i,j;
	//for (i=0;i<4;i++)
	//	for (j=0;j<4;j++)
	//		os->write(reinterpret_cast<const char *>(&(_matrix[i][j])),4);
	//write rotation
		os->write(reinterpret_cast<const char *>(&_rot.x),4);
		os->write(reinterpret_cast<const char *>(&_rot.y),4);
		os->write(reinterpret_cast<const char *>(&_rot.z),4);
		os->write(reinterpret_cast<const char *>(&_rot.q),4);
	//write scaleOrientation
		os->write(reinterpret_cast<const char *>(&_scRot.x),4);
		os->write(reinterpret_cast<const char *>(&_scRot.y),4);
		os->write(reinterpret_cast<const char *>(&_scRot.z),4);
		os->write(reinterpret_cast<const char *>(&_scRot.q),4);
	//write center
		os->write(reinterpret_cast<const char *>(&_cen.x),4);
		os->write(reinterpret_cast<const char *>(&_cen.y),4);
		os->write(reinterpret_cast<const char *>(&_cen.z),4);
	//write translation
		os->write(reinterpret_cast<const char *>(&_trans.x),4);
		os->write(reinterpret_cast<const char *>(&_trans.y),4);
		os->write(reinterpret_cast<const char *>(&_trans.z),4);
	//write scale
		os->write(reinterpret_cast<const char *>(&_scale.x),4);
		os->write(reinterpret_cast<const char *>(&_scale.y),4);
		os->write(reinterpret_cast<const char *>(&_scale.z),4);
}


//blagica   boneAtom////////////////////////////////////////////////////////////////////////////////////	

ANMBoneAtom::ANMBoneAtom(int boneID,int parentId,ANMTransformAtom* trans)
{
  _boneId=boneID;
  _parentId=parentId ;
  _boneTransform=trans;  
}

ANMBoneAtom::ANMBoneAtom(M_SBBone* pBone,int parentId)
{
  _boneId=pBone->boneID;
  _parentId=parentId ;
  _boneTransform=new ANMTransformAtom(pBone);
  
}

ANMBoneAtom::ANMBoneAtom(ANMBoneAtom* bone)
{
	if (bone)
	{
		this->_boneId =bone->_boneId;
		this->_parentId =bone->_parentId;
		this->_boneTransform =new ANMTransformAtom(bone->_boneTransform);
	}
}
	
ANMBoneAtom::~ANMBoneAtom()
{
}
	

void ANMBoneAtom::WriteStuff(ostream *os)
{

}

//blagica   skeletonAtom////////////////////////////////////////////////////////////////////////////////////

ANMSkeletonAtom::~ANMSkeletonAtom()
{

}

ANMSkeletonAtom::ANMSkeletonAtom()
{
 	type = 'skl';
	atom_id = addMap(ANMList,NULL);//node->sgprivate);//->NodeID);
}

ANMBoneAtom * ANMSkeletonAtom::ProcessBones(M_SBBone * bone,int parentID)
{	
	if (!findMap(ANMList,bone->sgprivate))//->NodeID))
	{
		ANMBoneAtom * bn=new ANMBoneAtom(bone,parentID);

		int i;
		for (i=0;i<ChainGetCount(bone->children);i++)
		{
			SFNode* node=(SFNode*)ChainGetEntry(bone->children,i);
			if (node->sgprivate->tag==TAG_MPEG4_SBBone)
			{
				M_SBBone* bon=(M_SBBone*)ChainGetEntry(bone->children,i);
				ANMBoneAtom* anmBone=ProcessBones(bon,bone->boneID);
				if (anmBone)
					_bones.push_back(anmBone);			
				//AddChild(anmBone);
			}
		}
		return bn;
	}
	return NULL;
}

void ANMSkeletonAtom::AddSkinnedModel(M_SBSkinnedModel* pSB)
{
	if (pSB)
	{
		int i;
		for (i=0;i<ChainGetCount(pSB->skeleton);i++)
		{
			M_SBBone* bone=(M_SBBone*)ChainGetEntry(pSB->skeleton,i);
			ANMBoneAtom* anmBone=ProcessBones(bone,0);
			if (anmBone)
				_bones.push_back(anmBone);
			//AddChild(anmBone);
		}
	}
}

void ANMSkeletonAtom::AddBone (ANMBoneAtom* anmBone)
{
	_bones.push_back(anmBone);
}

ANMSkeletonAtom::ANMSkeletonAtom(ANMSkeletonAtom* prevSkeleton)
{
	this->type =prevSkeleton->type;
	this->atom_id = addMap(ANMList,NULL);
	for (int i=0;i<prevSkeleton->_bones.size();i++)
		this->AddBone(new ANMBoneAtom(prevSkeleton->_bones[i]));
}

ANMSkeletonAtom::ANMSkeletonAtom(M_SBSkinnedModel* pSB)
{
 	type = 'skl';
	SFNode* node=(SFNode*)ChainGetEntry(pSB->skeleton,0);
	atom_id = addMap(ANMList,NULL);//node->sgprivate);//->NodeID);

	AddSkinnedModel(pSB);
}
	

void ANMSkeletonAtom::WriteStuff(ostream *os)
{
	int nsize=_bones.size();
	os->write(reinterpret_cast<const char *>(&nsize),4);

	int i;
    for (i=0;i<_bones.size();i++)
	{		
		ANMBoneAtom* bone=_bones[i];
		os->write(reinterpret_cast<const char *>(&(bone->_boneId)),4);
		os->write(reinterpret_cast<const char *>(&(bone->_parentId)),4);
		bone->_boneTransform->Write(os); 
	}
}


//blagica   AnimationAtom////////////////////////////////////////////////////////////////////////////////////

ANMAnimationAtom::ANMAnimationAtom()
{
	type = 'anm';	
	atom_id = addMap(ANMList,NULL);
	
	_numFrames=0;
  _localCoordinates=1;
} 

void ANMAnimationAtom::AddSkeleton(ANMSkeletonAtom* skeleton)
{
	_frames.push_back(skeleton);
	AddChild(skeleton);
	_numFrames++;
}


ANMAnimationAtom::ANMAnimationAtom(M_SBVCAnimation* pAnm)
{
	type = 'anm';	
	atom_id = addMap(ANMList,pAnm->sgprivate);//->NodeID);

	_numFrames=1;
	_localCoordinates=1;
	
	int i=0;
	for (i=0;i<ChainGetCount(pAnm->virtualCharacters);i++)
	{
		M_SBSkinnedModel* skinModel=(M_SBSkinnedModel*)ChainGetEntry(pAnm->virtualCharacters,i);
		if (i==0)
		{
			ANMSkeletonAtom* skeleton=new ANMSkeletonAtom(skinModel);
 			_frames.push_back(skeleton);
			AddChild(skeleton);
		}
		else
			_frames[0]->AddSkinnedModel(skinModel);
	}
}

ANMAnimationAtom::ANMAnimationAtom(M_SBVCAnimationV2* pAnm)
{
	type = 'anm';	
	atom_id = addMap(ANMList,pAnm->sgprivate);//->NodeID);

	_numFrames=1;
	_localCoordinates=1;
	
	int i=0;
	for (i=0;i<ChainGetCount(pAnm->virtualCharacters);i++)
	{
		M_SBSkinnedModel* skinModel=(M_SBSkinnedModel*)ChainGetEntry(pAnm->virtualCharacters,i);
		if (i==0)
		{
			ANMSkeletonAtom* skeleton=new ANMSkeletonAtom(skinModel);
 			_frames.push_back(skeleton);
			AddChild(skeleton);
		}
		else
			_frames[0]->AddSkinnedModel(skinModel);
	}
}


ANMAnimationAtom::~ANMAnimationAtom()
{
}
	

void ANMAnimationAtom::WriteStuff(ostream *os)
{
	os->write(reinterpret_cast<const char *>(&(_numFrames)),4);
	os->write(reinterpret_cast<const char *>(&(_localCoordinates)),1);
}


void ANMSkeletonAtom::UpdateBone(CBone* cBone)
{
	for (int i=0;i<_bones.size();i++)
	if (cBone->boneIdentifier==this->_bones[i]->_boneId)
	{		
		_bones[i]->_boneTransform->UpdateAtom(cBone);
		break;
    }
	

}

void ANMAnimationAtom::InsertFrame(LPBYTE pInData)
{
	ANMSkeletonAtom* skeleton;
	BYTE tag;
	DWORD len=0;
	CBone* cbone;
	void* pData = NULL;

	if( !pInData )
		return;

	int sumSize = 0;

	BBADataExchange dataExchange(pInData, true);
	int ntags = dataExchange.ParseNoTags();
	
	while(pData=dataExchange.FetchNext(tag,len))
	{
		sumSize += len;
		switch(tag)
		{
			case BBADataExchange::TAG_KEY_FRAME:
				{				
					ANMSkeletonAtom *prevSkeleton=_frames[_frames.size()-1];
					skeleton=new ANMSkeletonAtom(prevSkeleton);
 					AddSkeleton(skeleton); 
					break;
				}
			case BBADataExchange::TAG_BONES:
				{
					cbone = (CBone *)pData;
					FetchBoneData(cbone,skeleton);
					break;
				}
			default:
				break;
		}
	}

}


void ANMAnimationAtom::FetchBoneData(CBone* cbone, ANMSkeletonAtom* skeleton)
{
	if( !cbone->boneDCTData )
	{
		// one bone
		skeleton->UpdateBone(cbone);
	}
	else
	{
		// 16 bones at once
		for(int t_=0; t_ < 16; t_++)
		{
			skeleton->UpdateBone(&cbone[t_]);			
		}
	}
}