// _ANMAtom.h: interface for the ANMAtom class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _ANMATOM_H_
#define _ANMATOM_H_

#ifndef BYTE
typedef unsigned char BYTE;
#endif

#ifndef LPBYTE
typedef BYTE* LPBYTE;
#endif

#include "BIFSvector.h"

#include "BBAInclude.h"
#include "BBADataExchange.h"

#include "ostream.h"
using namespace BIFS;


#include <gpac/intern/m4_scenegraph_dev.h>
#include <gpac/intern/m4_bifs_tables.h>
#include <gpac/intern/ztypes.h>

#include "matrix4.h"

#define ostream BIFS::ostream

//blagica   ANMAtom////////////////////////////////////////////////////////////////////////////////////
class ANMAtom
{
public:
    ANMAtom();
    virtual ~ANMAtom();
    void AddChild(ANMAtom *child);
    void Write(ostream *os);

	static Chain* ANMList;

protected:
    unsigned int atom_id;
    unsigned int size;
    unsigned int type;
    void WriteHeader(ostream *os);
    virtual void WriteStuff(ostream *os);
    void WriteFooter(ostream *os);

    //std::vector<ANMAtom *> _children;
#ifdef _IMPORT_MP4SDK 
	std::vector 
#else
	BIFSvector 
#endif 
	<ANMAtom *> _children;
    long _start, _end;
};


//blagica   transformAtom////////////////////////////////////////////////////////////////////////////////////
class ANMTransformAtom:public ANMAtom
{
public:
	ANMTransformAtom(M_SBBone* pBone);
	~ANMTransformAtom();
	void WriteStuff(ostream *os);

	ANMTransformAtom(ANMTransformAtom* transform=NULL);
	void  UpdateAtom(CBone *cBone);
	ANMTransformAtom(float rotX,float rotY,float rotZ,float rotQ,float scRotX,float scRotY,float scRotZ,float scRotQ,float cenX,float cenY,float cenZ,float transX,float transY,float transZ,float scaleX,float scaleY,float scaleZ);

	__vec4f getRotation(){return _rot;}
	__vec4f getScRotation(){return _scRot;}
	__vec3f getCenter(){return _cen;}
	__vec3f getTrans(){return _trans;}
	__vec3f getScale(){return _scale;}

protected:
	__vec4f _rot;
	__vec4f _scRot;
	__vec3f _cen;
	__vec3f _trans;
	__vec3f _scale;
};

//blagica   boneAtom////////////////////////////////////////////////////////////////////////////////////
class ANMBoneAtom:public ANMAtom
{
public:
	ANMBoneAtom(M_SBBone* pBone,int parentId);
        ANMBoneAtom(ANMBoneAtom* bone=NULL);
	ANMBoneAtom(int boneID,int parentId,ANMTransformAtom* trans);
	~ANMBoneAtom();
	void WriteStuff(ostream *os);

	int _boneId,_parentId;
	ANMTransformAtom* _boneTransform;
	
};



//blagica   skeletonAtom////////////////////////////////////////////////////////////////////////////////////
class ANMSkeletonAtom:public ANMAtom
{
public:
	ANMSkeletonAtom(M_SBSkinnedModel* pSB);
	~ANMSkeletonAtom();
	void WriteStuff(ostream *os);
	void AddSkinnedModel(M_SBSkinnedModel* pSB);

	ANMSkeletonAtom();
	ANMSkeletonAtom(ANMSkeletonAtom* prevSkeleton);
	void AddBone (ANMBoneAtom * anmBone);
    void UpdateBone(CBone *cBone);

	ANMBoneAtom* getBone(int index){return _bones[index];}
	int getBoneCount(){return _bones.size();}

protected:
	
#ifdef _IMPORT_MP4SDK 
	std::vector 
#else
	BIFSvector 
#endif 
	<ANMBoneAtom*> _bones;

	ANMBoneAtom * ProcessBones(M_SBBone * bone,int parentID);
};

//blagica   AnimationAtom////////////////////////////////////////////////////////////////////////////////////
class ANMAnimationAtom:public ANMAtom
{
public:
	ANMAnimationAtom();
	ANMAnimationAtom(M_SBVCAnimation* pAnm);
	ANMAnimationAtom(M_SBVCAnimationV2* pAnm);
	~ANMAnimationAtom();
	void WriteStuff(ostream *os);

    void InsertFrame (LPBYTE frameBuffer);
	void FetchBoneData(CBone* cbone, ANMSkeletonAtom* skeleton);
	void AddSkeleton(ANMSkeletonAtom * skeleton);

	ANMSkeletonAtom* getFrame(int index){return _frames[index];}
	ANMBoneAtom* getBone(int index){return _frames[0]->getBone(index);}
	int getBoneCount(){return _frames[0]->getBoneCount();}

protected:
	unsigned int _numFrames;
	int _localCoordinates;

#ifdef _IMPORT_MP4SDK 
	std::vector 
#else
	BIFSvector 
#endif 
	<ANMSkeletonAtom*> _frames;	
};

#undef ostream 

#endif // !defined(_ANMATOM_H_)
