/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_ztype_h)	// Sentry, use file only if it's not already included.
#define _ztype_h

//ML - added the next include line
#include <math.h>
//#include <m4_mpeg4_nodes.h>
//#include "m4_scenegraph_vrml.h"

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Compositor
FILE:       ztype.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   class definitions for types of node fields
-------------------------------------------------------------------------*/

#ifdef _EXPORT_MP4SDK
   #define CLASS_MP4SDK    __declspec(dllexport)
   #define EXPIMP_TEMPLATE
   #define IS_MP4SDK
#else
#ifdef _IMPORT_MP4SDK
   #define CLASS_MP4SDK    __declspec(dllimport)
   #define EXPIMP_TEMPLATE extern
  // #define IS_MP4SDK
#endif
#endif
//added by octavian


/*-----------------------------------------------------------------
Bool
-----------------------------------------------------------------*/

class MP4Rotation;
/*blagica#ifdef IS_MP4SDK
class CLASS_MP4SDK Bool
#else
  class Bool
#endif
{
	bool m_b;

public:
	Bool (bool b = false) : m_b (b) {}
	operator bool () const {return m_b;}
	bool operator = (bool b) {return m_b = b;}
	bool operator &= (bool b) {return m_b &= b;}
	bool operator |= (bool b) {return m_b |= b;}
	bool operator ^= (bool b) {return m_b ^= b;}
	bool operator ++ () {return ++m_b;}
	bool operator == (const Bool &b) { return (m_b == (bool) b);}
};

/*-----------------------------------------------------------------
Time
-----------------------------------------------------------------*/
//typedef double Time;








/*-----------------------------------------------------------------
Vec3f
-----------------------------------------------------------------*/

class MP4Rotation;
#ifdef IS_MP4SDK
class CLASS_MP4SDK Vec3f
#else
class Vec3f
#endif
{
public:
	float x, y, z;

public:
	Vec3f (float xx = 0, float yy = 0, float zz = 0) :
		x (xx), y (yy), z(zz)
	{}
	Vec3f operator = (float f)
	{
		x = y = z = f;
		return *this;
	}
	bool operator == (const Vec3f &vec) const
	{
		return x == vec.x && y == vec.y && z == vec.z;
	}

	//added by Blagica Jovanova
	Vec3f operator = (const SFVec3f &v) {
      x = v.x;
	  y = v.y;
	  z = v.z;
	  return *this;
	}

	// MBS: operator added
	Vec3f operator += (const Vec3f &v) {
	  x += v.x;
	  y += v.y;
	  z += v.z;
	  return *this;
	}
	// VUB: operator added
	Vec3f operator + (const Vec3f &v) {
    Vec3f t;
	  t.x = x + v.x;
	  t.y = y + v.y;
	  t.z = z + v.z;
	  return t;
	}
	Vec3f operator - (const Vec3f &v) {
    Vec3f t;
	  t.x = x - v.x;
	  t.y = y - v.y;
	  t.z = z - v.z;
	  return t;
	}
	Vec3f operator / (const Vec3f &v) {
    Vec3f t;
	  t.x = x / v.x;
	  t.y = y / v.y;
	  t.z = z / v.z;
	  return t;
	}
	Vec3f operator + (float v) {
    Vec3f t;
	  t.x = x + v;
	  t.y = y + v;
	  t.z = z + v;
	  return t;
	}
	Vec3f operator - (float v) {
    Vec3f t;
	  t.x = x - v;
	  t.y = y - v;
	  t.z = z - v;
	  return t;
	}
	Vec3f operator * (float v) {
    Vec3f t;
	  t.x = x * v;
	  t.y = y * v;
	  t.z = z * v;
	  return t;
	}
	Vec3f operator / (float v) {
    Vec3f t;
	  t.x = x / v;
	  t.y = y / v;
	  t.z = z / v;
	  return t;
	}
	// INT new operators added
	void operator*=(const MP4Rotation& q);
	void operator*=(const float m_elements[16]);
	Vec3f Inverse(){
		Vec3f ret(-x,-y,-z);
		return ret;
	}

	
	//added by Octavian Folea
	bool operator < (const Vec3f &vec) const
	{	

		return ( (x + y + z) < (vec.x + vec.y + vec.z) );
	}
	

	//ML - added the rest of the functions in the class
	float GetLength()
	{
		return (float) sqrt(x * x + y * y + z * z);
	}

	void Normalize()
	{
		float mag = GetLength();
		if (mag == 0.0f)
			mag = 1.0f;
		x /= mag;
		y /= mag;
		z /= mag;
	}

	Vec3f& Cross (const Vec3f &vec)
	{
		float new_x, new_y, new_z;
		new_x = y * vec.z - z * vec.y;
		new_y = z * vec.x - x * vec.z;
		new_z = x * vec.y - y * vec.x;
		x = new_x;
		y = new_y;
		z = new_z;
		return *this;
	}

	float operator * (const Vec3f &vec) const
	{
		return (x * vec.x +
			y * vec.y +
			z * vec.z);
	}

	Vec3f& operator -= (const Vec3f &vec)
	{
		x -= vec.x;
		y -= vec.y;
		z -= vec.z;
		return *this;
	}

	operator const float* () const
	{
		return &x;
	}

	Vec3f& operator *= (float scale)
	{
		x *= scale;
		y *= scale;
		z *= scale;
		return *this;
	}
};




/*-----------------------------------------------------------------
Vec2f
-----------------------------------------------------------------*/
#ifdef IS_MP4SDK
class CLASS_MP4SDK Vec2f
#else
class Vec2f
#endif
{
public:
	float x, y;

public:
	Vec2f (float xx = 0, float yy = 0) :
		x (xx), y (yy)
	{}
	Vec2f operator = (float f)
	{
		x = y = f;
		return *this;
	}
	bool operator == (const Vec2f &vec) const
	{
		return x == vec.x && y == vec.y;
	}
	
	//added by Blagica Jovanova
	Vec2f operator = (const SFVec2f &v) {
        
	    x = v.x;
	    y = v.y;
	    return *this;
	}

	Vec2f& operator += (const Vec2f &v) {
	  x += v.x;
	  y += v.y;
	  return *this;
	}
	Vec2f& operator *= (float scale)
	{
		x *= scale;
		y *= scale;
		return *this;
	}
	float GetLength()
	{
		return (float) sqrt(x * x + y * y);
	}

	void Normalize()
	{
		float mag = GetLength();
		if (mag == 0.0f) mag = 1.0f;
		x /= mag;
		y /= mag;
	}
	float operator * (const Vec2f &vec) const
	{
		return (x * vec.x + y * vec.y);
	}
	
	//added by Octavian Folea
	bool operator < (const Vec2f &vec) const
	{
		return ( (x + y ) < (vec.x + vec.y ) );
	}

};

/*-----------------------------------------------------------------
MP4Rotation
-----------------------------------------------------------------*/
class Vec3f;
#ifdef IS_MP4SDK
class CLASS_MP4SDK MP4Rotation
#else
class MP4Rotation
#endif
{
public:
	float x, y, z, theta;

public:
	MP4Rotation (float xx = 0, float yy = 0, float zz = 0, float th = 0) :
		x (xx), y (yy), z (zz), theta (th)
	{}
	MP4Rotation operator = (float f)
	{
		x = y = z = theta = f;
		return *this;
	}
	bool operator == (const MP4Rotation &vec) const
	{
		return
			x == vec.x && y == vec.y && z == vec.z && theta == vec.theta;
	}

		//added by Blagica Jovanova
     	MP4Rotation& operator = (const SFRotation &v) 
	 {
			x = v.x;
	     y = v.y;
		 z = v.z;
         theta=v.q;
	     return *this;
	}

	float operator * (const MP4Rotation &r) const
	{
		return (x * r.x + y * r.y + z * r.z + theta * r.theta);
	}

	MP4Rotation& operator *= (const MP4Rotation &r)
{
		MP4Rotation q = r;

		q.makeQuat();
		makeQuat();
		float qx, qy, qz, qw;
		qx = x;
		qy = y;
		qz = z;
		qw = theta;
		x = qw * q.x +     qx * q.theta + qy * q.z -     qz * q.y;
		y = qw * q.y -     qx * q.z +     qy * q.theta + qz * q.x;
		z = qw * q.z +     qx * q.y -     qy * q.x +     qz * q.theta;
    theta = qw * q.theta - qx * q.x -     qy * q.y -     qz * q.z;
		unmakeQuat();
		return *this;
}

	void Normalize ();
	void makeQuat();
	void unmakeQuat();
	//spherical linear interpolation
	MP4Rotation Slerp(MP4Rotation r, float t);
	
	// operations added by INT
	void operator*=(const Vec3f& v);
	MP4Rotation Inverse(){
		MP4Rotation ret(x,y,z,-theta);
		return ret;
	}
};












/*-----------------------------------------------------------------
Rotation2D
-----------------------------------------------------------------*/
class Rotation2D
{
public:
	float x, y, theta;

public:
	Rotation2D (float xx = 0, float yy = 0, float th = 0) :
		x (xx), y (yy), theta (th)
	{}
	Rotation2D operator = (float f)
	{
		x = y = theta = f;
		return *this;
	}
	bool operator == (const Rotation2D &vec) const
	{
		return
			x == vec.x && y == vec.y && theta == vec.theta;
	}
};

/*-----------------------------------------------------------------
Color
-----------------------------------------------------------------*/
#ifdef IS_MP4SDK
class CLASS_MP4SDK Color3f
#else
class Color3f
#endif
{
public:
	float r, g, b;

public:
	Color3f (float rr = 0, float gg = 0, float bb = 0) :
		r (rr), g (gg), b(bb)
	{}
	Color3f operator = (float f)
	{
		r = g = b = f;
		return *this;
	}
	//added by Blagica Jovanova
	Color3f& operator = (const SFColor &v) {
        
	    r = v.red;
	    g = v.green;
	    b=  v.blue;
	    return *this;
	}
	
	bool operator == (const Color3f &color) const
	{
		return r == color.r && g == color.g && b == color.b;
	}

	//added by Octavian Folea
	bool operator < (const Color3f &color) const
	{
		return ( (r + g + b) < (color.r + color.g + color.b) );
	}

	// SB @@@@ cast Color3f to float* for OpenGL >>>>
	operator const float* () const {return &r;}
	//converts from RGB in[0, 1] to HSV in H[0, 360], SV[0, 1]
	void toHSV();
	//converts from HSV to RGB 
	void fromHSV();
};

/*-----------------------------------------------------------------
IM1Image
-----------------------------------------------------------------/
//MBS 110498: added constructor IM1Image(w,h,components)
//Alvar 981223: Added pixel array allocation and refcount stuff
class IM1Image
{
public:
	int width;
	int height;
	int numComponents;
	BYTE *pixels;
	int *m_pRefCnt;  // reference count for pixel array

	IM1Image (int w, int h, int components);      // Mike B-S ctor
	IM1Image (int = 0);                           // Zvi ctor
	IM1Image (const IM1Image &i);                      // copy ctor
	IM1Image& operator= (const IM1Image &i);           // assignment
	~IM1Image ();
	IM1Image operator = (float)
	{
		return *this;
	}
	//JLF: added dummy operator for comparaison (used in field state checking)
	bool operator == (const IM1Image &) const {return FALSE;}
}; *******/





/*-------------------------------------------------------------
Vec4f
-------------------------------------------------------------*/
class Vec4f
{
public:
	float x, y, z, q;

public:
	Vec4f (float xx = 0, float yy = 0, float zz = 0, float qq = 0) :
		x (xx), y (yy), z(zz), q(qq)
	{}
	Vec4f operator = (float f)
	{
		x = y = z = q = f;
		return *this;
	}
	//added by Blagica Jovanova
	Vec4f operator = (const SFColor &v) {
        x = v.red;
	    y = v.green;
	    z=  v.blue;
		q=0;
	    return *this;
	}
	bool operator == (const Vec4f &vec) const
	{
		return x == vec.x && y == vec.y && z == vec.z && q == vec.q;
	}
	// MBS: operator added
	Vec4f operator += (const Vec4f &v) {
	  x += v.x;
	  y += v.y;
	  z += v.z;
	  q += v.q;
	  return *this;
	}

	//ML - added the rest of the functions in the class
	float GetLength()
	{
		return (float) sqrt(x * x + y * y + z * z + q * q);
	}

	void Normalize()
	{
		float mag = GetLength();
		if (mag == 0.0f)
			mag = 1.0f;
		x /= mag;
		y /= mag;
		z /= mag;
		q /= mag;
	}

	Vec4f& Cross (const Vec4f &vec)
	{
		float new_x, new_y, new_z, new_q;
		new_x = y * vec.z - z * vec.y;
		new_y = z * vec.x - x * vec.z;
		new_z = x * vec.y - y * vec.x;
		new_q = q * vec.q - q * vec.q;
		x = new_x;
		y = new_y;
		z = new_z;
		q = new_q;
		return *this;
	}

	float operator * (const Vec4f &vec) const
	{
		return (
			x * vec.x +
			y * vec.y +
			z * vec.z +
			q * vec.q
		);
	}

	Vec4f& operator -= (const Vec4f &vec)
	{
		x -= vec.x;
		y -= vec.y;
		z -= vec.z;
		q -= vec.q;
		return *this;
	}

	operator const float* () const
	{
		return &x;
	}

	Vec4f& operator *= (float scale)
	{
		x *= scale;
		y *= scale;
		z *= scale;
		q *= scale;
		return *this;
	}
};

#endif                                      // _ztype_h sentry.
