/********************************************************************************/
/* This software module was developped by										*/

/*	   Marius Preda (MP), marius.preda@int-evry.fr,								*/
/*                        Institut National des Telecommunications, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/
//#include<stdafx.h>
#include<stdio.h>
#include<stdlib.h>

#include "CMorph.h"


CMorph::CMorph(){
	morphDCTData = NULL;
	prec=NULL;
	suiv=NULL;
}

CMorph::~CMorph()
{

}

void CMorph::AllocMorph(){

	if (morphMask.wcnt)
		return;

	morphMask.wcnt=NumberWeights;
	morphMask.isWeightChanged=(bool*)malloc(NumberWeights*sizeof(bool));

	morphData.wcnt=NumberWeights;
	morphData.Weights=(int*)malloc(NumberWeights*sizeof(int));

};
void CMorph::AllocDCTMorph(){

	int seg;
	for (seg=0;seg<16;seg++){
		morphDCTData[seg].Weights=(int*)malloc(NumberWeights*sizeof(int));
	}

};
void CMorph::FreeMorph(){
	morphMask.Free();
	morphData.Free();
};

void CMorph::FreeDCTMorph(){
	
	for (int seg=0;seg<16;seg++)
		morphDCTData[seg].Free();

};

void CMorph::AllocMorph(int nw){

	NumberWeights=nw;
	AllocMorph();
};

void CMorph::AllocMinMaxMorph(){

	if (morphDataIMax.wcnt)
		return;
	int i;
	morphDataIMax.wcnt=NumberWeights;
	morphDataIMin.wcnt=NumberWeights;
	morphDataPMax.wcnt=NumberWeights;
	morphDataPMin.wcnt=NumberWeights;
	morphDataIMax.Weights=(int*)malloc(NumberWeights*sizeof(int));
	morphDataIMin.Weights=(int*)malloc(NumberWeights*sizeof(int));
	morphDataPMax.Weights=(int*)malloc(NumberWeights*sizeof(int));
	morphDataPMin.Weights=(int*)malloc(NumberWeights*sizeof(int));

	for (i=0;i<NumberWeights;i++){
		morphDataIMax.Weights[i]=1860;
		morphDataIMin.Weights[i]=-1860;
		morphDataPMax.Weights[i]=600;
		morphDataPMin.Weights[i]=-600;
	}

};

void CMorph::FreeMinMaxMorph(){
	morphDataIMax.Free();
	morphDataIMin.Free();
	morphDataPMax.Free();
	morphDataPMin.Free();
}

void CMorph::AllocPrevMorph(){
	if (morphDataPreviousFrame.wcnt)
		return;
	morphDataPreviousFrame.wcnt=NumberWeights;
	morphDataPreviousFrame.Weights=(int*)malloc(NumberWeights*sizeof(int));
}
void CMorph::FreePrevMorph(){
	morphDataPreviousFrame.Free();
}

int innod(CMorph *p,CMorph* &first,CMorph* &last)
{
	// add or update a muscle content

  CMorph *q;
  q=first;
  
  if(first==NULL){
      first=p;
      first->suiv=NULL;
      first->prec=NULL;
      last=p;
      last->prec=NULL;
      last->suiv=NULL;
      return 1;
    }
  if(p->morphIdentifier < first->morphIdentifier){
      p->suiv=first;
      p->prec=NULL;
      first->prec=p;
      first=p;
      return 1;
    }
  if(p->morphIdentifier > last->morphIdentifier){
      last->suiv=p;
      p->suiv=NULL;
      p->prec=last;
      last=p;
      return 1;
    }
  while (q){
	  if (p->morphIdentifier == q->morphIdentifier){
		q->morphMask=p->morphMask;		
		q->morphDataIMax=p->morphDataIMax;
		q->morphDataIMin=p->morphDataIMin;
		q->morphDataPMax=p->morphDataPMax;
		q->morphDataPMin=p->morphDataPMin;
		q->isInCurrentFrame=true;
		return 0;
	  }
	  else q=q->suiv;
  }
  q=first;
  while (q){
	  while (p->morphIdentifier < q->morphIdentifier)
		q=q->suiv;
	  p->suiv=q;
	  p->prec=q->prec;
	  q->prec->suiv=p;
	  q->prec=p;
	  return 1;
  }
  return 1;
}


CMorph *getnodindexi(int index,CMorph * &first,CMorph * &last)
{
  CMorph *q;
  q=first;
  
  if(first==NULL){
      return NULL;
    }
  
  for (int i=0;i<index;i++)
	q=q->suiv;
  return q;
}

CMorph *getnodIDi(int ID,CMorph * &first,CMorph * &last)
{
  CMorph *q;
  q=first;
  
  if(first==NULL){
      return NULL;
    }
  
  while (q) {
	  if (q->morphIdentifier==ID)
		  return q;
	  q=q->suiv;
  }
  return q;
}


void outnod(CMorph * p,CMorph * &first,CMorph * &last)
{
  
  if(first==NULL) return;

  if(p==first)
    {
      first=first->suiv;
      delete p;
      if(first==NULL) last=NULL;
      else first->prec=NULL;
      return;
     
    }
  if(p==last)
    {
      last=last->prec;
      delete p;
      if(last==NULL) first=NULL;
      else last->suiv=NULL;
      return;
    }
  p->prec->suiv=p->suiv;
  p->suiv->prec=p->prec;
  delete p;
}

void freeliste(CMorph * &first,CMorph* &last)
{
  CMorph *p=NULL;

  if(first==NULL) return;
  
  do
    {
      p=first;
      outnod(p,first,last);
    }while(first != NULL);
}
      
 
void affiche_liste(CMorph *first)
{
 
  CMorph *p=NULL;
    
  if(first==NULL)
    {
      fprintf(stderr,"liste vide\n");
      return;
    }
  p=first;
  do
    {
     
      fprintf(stderr,"%d, ",p->morphIdentifier);
      p=p->suiv;
    }while(p != NULL);
  fprintf(stderr,"\n");
}
