
#include "BIFStoVBLib.h"
#include "BoneShape.h"

BIFStoVBLib::BIFStoVBLib()
{
	VBlist = gf_list_new();	
	origVerCount = 0;
}

BIFStoVBLib::~BIFStoVBLib()
{
	while (gf_list_count(VBlist))
	{
		VertexBuffer* vb = (VertexBuffer*)gf_list_get( VBlist, 0 );
		delete vb;
		gf_list_rem(VBlist,0);
	}
	gf_list_del(VBlist);
}

void BIFStoVBLib::AllignTexCoordIndex( void* TexCoord, VertexBuffer** vert, bool useTextCoord )
{
	m_bUseTexCoord = useTextCoord;
	unsigned int i;
	unsigned int count;

	M_TextureCoordinate* pTexCoord = NULL;
	M_Coordinate* pCoord = NULL;
	if ( m_bUseTexCoord )
	{
		pTexCoord = (M_TextureCoordinate*)TexCoord;
		count = gf_node_get_parent_count((GF_Node*) pTexCoord);
	}
	else
	{
		pCoord = (M_Coordinate*)TexCoord;
		count = gf_node_get_parent_count((GF_Node*)pCoord);
	}


	for (i=0;i < count;i++)
	{
		GF_Node* pNode;

		if ( m_bUseTexCoord )
		{
			pNode=(GF_Node*)gf_node_get_parent((GF_Node*)pTexCoord,i);
		}
		else
		{
			pNode=(GF_Node*)gf_node_get_parent((GF_Node*)pCoord,i);
		}

		M_Normal* pNormal=NULL;
		MFInt32* pCoordIndex=NULL;
		MFInt32* pNormalIndex=NULL;
		MFInt32* pTexIndex=NULL;
		M_Shape* pShape=NULL;
		M_MorphShape* pMorph=NULL;

		switch (pNode->  sgprivate->tag)
		{
		case TAG_MPEG4_IndexedFaceSet:
			{				
				M_IndexedFaceSet* pIfs;
				pIfs         = (M_IndexedFaceSet*)pNode;
				pCoord       = (M_Coordinate*)pIfs->coord;
				pTexCoord    = (M_TextureCoordinate*)pIfs->texCoord;
				pNormal      = (M_Normal*)pIfs->normal;
				pCoordIndex  = &(pIfs->coordIndex);
				pTexIndex    = &(pIfs->texCoordIndex);
				pNormalIndex = &(pIfs->normalIndex);

				pNode = (GF_Node*)gf_node_get_parent((GF_Node*)pIfs,0);
				if ( pNode->sgprivate->tag == TAG_MPEG4_BitWrapper )
				{
					M_BitWrapper* pBit = (M_BitWrapper*)pNode;
					pShape = (M_Shape*)gf_node_get_parent((GF_Node*)pBit,0);
				}
				else
					pShape = (M_Shape*)gf_node_get_parent((GF_Node*)pIfs,0);
				pNode = (GF_Node*)gf_node_get_parent((GF_Node*)pShape,0);
				if (pNode->sgprivate->tag==TAG_MPEG4_MorphShape)
				{
					pMorph=(M_MorphShape*)pNode;
					if ( pShape != (M_Shape*) pMorph->baseShape )
						continue;
					//(*vert)->pMorphShape=(M_MorphShape*)pNode;
					pNode=(GF_Node*)gf_node_get_parent((GF_Node*)pNode,0);
				}
				switch (pNode->sgprivate->tag)
				{
				case TAG_MPEG4_SBSkinnedModel:				
					{
						M_SBSkinnedModel* pSb=(M_SBSkinnedModel*)pNode;
						if (!(*vert))
						{
							//printf("Creating vertex Buffer : %d %d\n",pCoord->point.count,pNormal->vector.count);
							*vert=new VertexBuffer(pCoord->point.count);
							(*vert)->type=VBTYPE_SKIN;
							(*vert)->pCoord=pCoord;
							(*vert)->pNormal=pNormal;
							(*vert)->pMorphShape=pMorph;
							unsigned int j;
							//printf("Reading bones...\n");
							GF_ChildNodeItem* node = pSb->bones;
							while ( node!= NULL )
								//for (j=0;j<gf_list_count(pSb->bones);j++)
							{
								M_SBBone* pBone=(M_SBBone*)node->node;// gf_list_get(pSb->bones,j);
								//printf("Rreading Bone : %d\n",pBone->boneID);
								unsigned int k=0;
								for (k=0;k<pBone->skinCoordIndex.count;k++)
								{
									//if (pBone->boneID==6 && k>430)
									//	printf("%d Coordinate %d bone influence %f\n",k,pBone->skinCoordIndex.vals[k],pBone->skinCoordWeight.vals[k]);
									(*vert)->buffer[pBone->skinCoordIndex.vals[k]].AddInfluense(pBone->boneID+1,pBone->skinCoordWeight.vals[k]);
								}
								node = node->next;
							}
						}
						break;
					}
				case TAG_MPEG4_Group:case TAG_MPEG4_SBSegment:
					pNode=(GF_Node*)gf_node_get_parent((GF_Node*)pNode,0);
				case TAG_MPEG4_SBBone:					
					{
						if (!pNode)
							break;
						if (pNode->sgprivate->tag!=TAG_MPEG4_SBBone)
							break;
						M_SBBone* pBone=(M_SBBone*)pNode;
						unsigned int j;
						//for (j=0;j<gf_list_count(pBone->sgprivate->parents);j++)
						{
							//pNode=(GF_Node*)gf_list_get(pBone->sgprivate->parents,j);
							//if (pNode->sgprivate->tag==TAG_MPEG4_SBSkinnedModel)
							{
								//M_SBSkinnedModel* pSb=(M_SBSkinnedModel*)pNode;
								if (!(*vert))
								{
									*vert=new VertexBuffer(pCoord->point.count);
									(*vert)->type=VBTYPE_RIGD;
									(*vert)->pCoord=pCoord;
									(*vert)->pNormal=pNormal;
									(*vert)->pMorphShape=pMorph;
									unsigned int k;
									for (k=0;k<pCoord->point.count;k++)
										(*vert)->buffer[k].AddInfluense(pBone->boneID+1,1);
								}								
							}
						}
						break;
					}
				}
				break;
			}
		case TAG_MPEG4_SBSkinnedModel:
			{
				continue;
			}
		}

		if (pCoord && pCoordIndex->count)
		{
			origVerCount += pCoord->point.count;
			if (!(*vert))
			{
				(*vert) = new VertexBuffer(pCoord->point.count);
				(*vert)->type=VBTYPE_SKIN;
				(*vert)->pCoord = pCoord;
				(*vert)->pNormal = pNormal;
				(*vert)->pMorphShape = pMorph;
			}
			if (!(*vert)->pTexCoord)
				(*vert)->pTexCoord=pTexCoord;

			(*vert)->AddCoordIndex(pCoordIndex);
			(*vert)->AddTexCoordIndex(pTexIndex);
			(*vert)->AddNormalIndex(pNormalIndex);

			(*vert)->AddShape(pShape);

			unsigned int j;
			for (j=0;j<pCoordIndex->count;j++)
			{
				if (pCoordIndex->vals[j]!=-1)
				{
					if (pTexIndex->count>0)
						if (pNormalIndex->count > 0)
							(*vert)->buffer[pCoordIndex->vals[j]].AddTextureIndex(pTexIndex->vals[j], pNormalIndex->vals[j]);
						else
							(*vert)->buffer[pCoordIndex->vals[j]].AddTextureIndex(pTexIndex->vals[j]);
					else
						if (pNormalIndex->count > 0)
							(*vert)->buffer[pCoordIndex->vals[j]].AddTextureIndex(pCoordIndex->vals[j], pNormalIndex->vals[j]);
						else
							(*vert)->buffer[pCoordIndex->vals[j]].AddTextureIndex(pCoordIndex->vals[j]);
				}
			}	
		}
	}
}

void BIFStoVBLib::CreateBoneSkin(int &boneNum, M_SBBone* pBone, matrix4 par, VertexBuffer* vert, M_Coordinate* pCoord, M_Normal* pNorm)
{
	int i;

	bool hasChild=false;
	bool hasSeg = false;

	int count = 0;
	//count = gf_list_count(pBone->children);

	GF_ChildNodeItem* node;
	node = pBone->children;
	while ( node && !hasChild )
		//for ( int k = 0; k < gf_list_count( pBone->children ) && !hasChild; k++ )
	{		
		GF_Node* pSnode=(GF_Node*)node->node; //gf_list_get( pBone->children, k );
		if ( pSnode->sgprivate->tag == TAG_MPEG4_SBSegment )
		{
			hasSeg = true;
			M_SBSegment* pSeg = ( M_SBSegment* ) pSnode;
			GF_ChildNodeItem* segnode = pSeg->children;
			//for ( int l = 0; l < gf_list_count( pSeg->children ); l++ )
			while ( segnode )
			{
				pSnode = ( GF_Node* )segnode->node;//gf_list_get( pSeg->children, k );
				if ( pSnode->sgprivate->tag == TAG_MPEG4_SBBone )
					hasChild=true;
				segnode = segnode->next;
			}
		}
		else
			if (pSnode->sgprivate->tag==TAG_MPEG4_SBBone)
				hasChild=true;
		node = node->next;
		count++;
	}
	node = pBone->children;

	M_SBSegment* pSeg;

	if ( count && hasSeg )
	{
		GF_Node* pSnode = (GF_Node*)pBone->children->node;// gf_list_get( pBone->children, 0 );
		if ( pSnode->sgprivate->tag == TAG_MPEG4_SBSegment ) 
		{
			pSeg = ( M_SBSegment* ) pSnode;
			//count = gf_list_count( pSeg->children );
			count = 0;
			node = pSeg->children;
			while ( node && ++count )
				node = node->next;
			node = pSeg->children;
		}
	}

	if (!count || !hasChild)
		count=0;

	//for (i=0;i<count;i++)
	while ( node )
	{
		GF_Node* pNode;
		if (hasChild)
			pNode = (GF_Node*)node->node;
		//if ( hasSeg )
		//{
		//	pNode = (GF_Node*)gf_list_get( pSeg->children, i );
		//}
		//else
		//	{
		//		pNode = (GF_Node*)gf_list_get( pBone->children, i );
		//	}
		if ( !hasChild || pNode->sgprivate->tag == TAG_MPEG4_SBBone )
		{

			M_SBBone* pSBone;
			float scale=1;
			SFVec3f a;

			if (hasChild)
			{
				pSBone=(M_SBBone*)pNode;			
				a=pSBone->center;
				boneNum++;
			}				
			else
			{
				a.x=1;
				a.y=1;
				a.z=1;
			}

			SFVec3f b;
			b.x=0;
			b.y=0;
			b.z=0;
			scale = sqrt((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z));
			scale /= boneShapeMax;
			if (!scale)
				scale = 1.f;

			for ( int j = 0; j < boneVertNo; j++ )
			{
				pCoord->point.vals[j + boneNum * boneVertNo].x = boneVertices[j][0];
				pCoord->point.vals[j + boneNum * boneVertNo].y = boneVertices[j][1];
				pCoord->point.vals[j + boneNum * boneVertNo].z = boneVertices[j][2];

				pNorm->vector.vals[j + boneNum * boneVertNo].x = boneNormals[j][0];
				pNorm->vector.vals[j + boneNum * boneVertNo].y = boneNormals[j][1];
				pNorm->vector.vals[j + boneNum * boneVertNo].z = boneNormals[j][2];
			}

			matrix4 rot,rot1,rot2,cen,trn;
			SFRotation objrot1,objrot2;

			SFVec3f p;
			p.x=0;
			p.y=0;
			p.z=1;
			float length = -boneShapeMax * scale;
			float theta=0;

			if (a.z!=0 || a.y!=0)
			{
				float a_norme = sqrt(a.x*a.x + a.y*a.y + a.z*a.z);
				float b_norme = length < 0 ? -length : length;

				/*a.x /= a_norme;
				a.y /= a_norme;
				a.z /= a_norme;

				length /= b_norme;*/

				p.x=0;
				p.y=-length*a.z;
				p.z=length*a.y;

				float p_norme = sqrt(p.x*p.x + p.y*p.y + p.z*p.z);				

				/*p.x /= p_norme;
				p.y /= p_norme;
				p.z /= p_norme;*/

				theta = p_norme / ( a_norme * b_norme );
				if ( theta >1.f )
					theta = 1.f;
				else
					if ( theta < -1.f )
						theta = -1.f;

				theta = asin( theta );

				if (a.x<0)
					theta=3.14159265358979323846 - theta;
			}


			objrot1.x=p.x;
			objrot1.y=p.y;
			objrot1.z=p.z;
			objrot1.q=theta;

			rot1.Rotation(rot.makeQuat(objrot1));
			cen.Translation(a);
			trn=rot1*cen*par;

			//trn.DumpToFile("Transf matr",pSBone->boneID,pBone->boneID);

			for ( int j = 0; j < boneVertNo/*6*/; j++ )
			{
				//Scale the coords
				pCoord->point.vals[j + boneNum * boneVertNo].z *= scale;
				pCoord->point.vals[j + boneNum * boneVertNo].y *= scale;				
				pCoord->point.vals[j + boneNum * boneVertNo].x *= scale;

				matrix4 point, rez;

				// Transform the coords
				point.setData(0,0,pCoord->point.vals[j + boneNum * boneVertNo].x);
				point.setData(0,1,pCoord->point.vals[j + boneNum * boneVertNo].y);
				point.setData(0,2,pCoord->point.vals[j + boneNum * boneVertNo].z);
				point.setData(0,3,1.0);
				rez=point*trn;
				pCoord->point.vals[j + boneNum * boneVertNo].x=rez.Data(0,0);
				pCoord->point.vals[j + boneNum * boneVertNo].y=rez.Data(0,1);
				pCoord->point.vals[j + boneNum * boneVertNo].z=rez.Data(0,2);

				// Transform the normals
				point.Identity();
				point.setData(0,0,pNorm->vector.vals[j + boneNum * boneVertNo].x);
				point.setData(0,1,pNorm->vector.vals[j + boneNum * boneVertNo].y);
				point.setData(0,2,pNorm->vector.vals[j + boneNum * boneVertNo].z);
				point.setData(0,3,1.0);
				rez = point*trn;
				pNorm->vector.vals[j + boneNum * boneVertNo].x=rez.Data(0,0);
				pNorm->vector.vals[j + boneNum * boneVertNo].y=rez.Data(0,1);
				pNorm->vector.vals[j + boneNum * boneVertNo].z=rez.Data(0,2);

				vert->buffer[j + boneNum * boneVertNo].AddTextureIndex(j + boneNum * boneVertNo);
				vert->buffer[j + boneNum * boneVertNo].AddInfluense(pBone->boneID+1,1);
			}

			if (hasChild)
			{
				rot.Rotation( rot.makeQuat(pSBone->rotation) );
				//cen.Translation(pSBone->center);
				trn = rot * cen * par;

				CreateBoneSkin( boneNum, pSBone, trn, vert, pCoord, pNorm );
			}

		}
		node = node->next;
	}

	/*for (i=0;i<gf_list_count(pBone->children);i++)
	{
	GF_Node* pNode=(GF_Node*)gf_list_get(pBone->children,i);
	if (pNode->sgprivate->tag=TAG_MPEG4_SBBone)
	{

	M_SBBone* pSBone=(M_SBBone*)pNode;
	CreateBoneSkin(pScene,pSBone,par,vert);
	}

	}*/

}

void BIFStoVBLib::BoneSkin( __tag_scene_graph* pScene, M_SBSkinnedModel* pSbs, VertexBuffer* &pBuff )
{
	M_SBBone* pBone=(M_SBBone*)pSbs->skeleton->node;//gf_list_get(pSbs->skeleton,0);

	int numBones = gf_node_list_get_count(pSbs->bones);

	int j;

	numBones--;

	if (numBones <= 0 )
		return;

	M_Coordinate* pCoord;
	pCoord = (M_Coordinate*)gf_node_new( pScene, TAG_MPEG4_Coordinate );
	pCoord->point.count = boneVertNo * numBones;
	pCoord->point.vals = (SFVec3f*)malloc( boneVertNo * numBones * sizeof( SFVec3f ) );

	M_Normal* pNorm;
	pNorm = (M_Normal*)gf_node_new( pScene, TAG_MPEG4_Normal );
	pNorm->vector.count = boneVertNo * numBones;
	pNorm->vector.vals = (SFVec3f*)malloc( boneVertNo * numBones * sizeof( SFVec3f ) );

	MFInt32* pCoordIndex;
	pCoordIndex = (MFInt32*)malloc( sizeof(MFInt32)  );
	pCoordIndex->count = boneIndNo * 4 * numBones;
	pCoordIndex->vals  = (int*)malloc( boneIndNo * 4 * numBones * sizeof(int) );

	int l = 0;
	for ( j = 0; j < numBones; j++ )
	{
		for ( int k = 0; k < boneIndNo; k++ )
		{
			pCoordIndex->vals[ l++ ] = boneIndicies[k][0] + j * boneVertNo;
			pCoordIndex->vals[ l++ ] = boneIndicies[k][1] + j * boneVertNo;
			pCoordIndex->vals[ l++ ] = boneIndicies[k][2] + j * boneVertNo;
			pCoordIndex->vals[ l++ ] = -1;
		}
	}

	pBuff = new VertexBuffer( boneVertNo * numBones );
	pBuff->type = VBTYPE_BONE;
	pBuff->pCoord = pCoord;
	pBuff->pNormal = pNorm;
	pBuff->AddCoordIndex( pCoordIndex );


	matrix4 rot,cen,trn;
	rot.Rotation(rot.makeQuat(pBone->rotation));
	cen.Translation(pBone->center);
	trn=rot*cen;

	int boneNum=-1;

	CreateBoneSkin( boneNum, pBone, trn, pBuff, pCoord, pNorm );

	pBuff->CreateVB();

	free( pCoordIndex->vals );
	free( pCoordIndex );
	pCoord->sgprivate->num_instances = 1;
	gf_node_unregister( (GF_Node*)pCoord, NULL );
	pNorm->sgprivate->num_instances = 1;
	gf_node_unregister( (GF_Node*)pNorm, NULL );
}

void BIFStoVBLib::CreateVB( __tag_scene_graph* pScene, bool useTexCoord )
{
	m_bUseTexCoord = useTexCoord;
	unsigned int i;
	VertexBuffer* pBuff=NULL;
	for(i=0;i<gf_list_count(pScene->all_nodes);i++)
	{
		GF_Node* pNode=(GF_Node*)gf_list_get(pScene->all_nodes,i);
		if ( (	m_bUseTexCoord && pNode->sgprivate->tag==TAG_MPEG4_TextureCoordinate) || 
			!m_bUseTexCoord && pNode->sgprivate->tag==TAG_MPEG4_Coordinate)
		{
			//printf("Processing shape %d...\n",i);
			pBuff=NULL;
			AllignTexCoordIndex( pNode, &pBuff, m_bUseTexCoord );
			if (pBuff)
			{
				pBuff->CreateVB();
				gf_list_add(VBlist,pBuff);
			}
			pBuff = NULL;
		}
	}
	pBuff = NULL;
	//if (!gf_list_count(VBlist))
	for(i=0;i<gf_list_count(pScene->all_nodes);i++)
	{
		GF_Node* pNode=(GF_Node*)gf_list_get(pScene->all_nodes,i);
		if (pNode->sgprivate->tag==TAG_MPEG4_SBSkinnedModel)
		{
			M_SBSkinnedModel* pSbs=(M_SBSkinnedModel*)pNode;
			BoneSkin( pScene, pSbs, pBuff );
			if (pBuff)
				gf_list_add(VBlist,pBuff);
			pBuff = NULL;
		}
	}
}

bool BIFStoVBLib::GetVB(unsigned int index,VBstruct** buff,int &size)
{
	if (index<gf_list_count(VBlist))
	{
		VertexBuffer* vb=(VertexBuffer*)gf_list_get(VBlist,index);
		*buff=vb->VB;
		size=vb->VBSize;
		return true;
	}
	return false;
}

bool BIFStoVBLib::GetVB(unsigned int index,VertexBuffer** vb)
{
	if (index<gf_list_count(VBlist))
	{
		*vb=(VertexBuffer*)gf_list_get(VBlist,index);		
		return true;
	}
	return false;
}

VertexBuffer::VertexBuffer(unsigned int count)
{
	type=VBTYPE_NONE;
	pc=0;
	pt=0;
	ps=0;
	pn=0;
	pCoord=NULL;
	pTexCoord=NULL;
	pMorphShape=NULL;
	pNormal=NULL;
	vertCount=count;
	buffer = new Vertex[vertCount];
	VB=NULL;
	memset(pTexCoordIndex,0,50*sizeof(MFInt32*));
	memset(pNormalIndex,0,50*sizeof(MFInt32*));
	memset(pShape,0,50*sizeof(M_Shape*));
}

VertexBuffer::~VertexBuffer()
{
	delete[] buffer;
	if (VB)
		free(VB);
	for (int i=0;i<pc;i++)
		free( indexBuff[i] );
}

void VertexBuffer::AddCoordIndex(MFInt32* pCoord)
{
	pCoordIndex[pc]=pCoord;
	pc++;
}

void VertexBuffer::AddTexCoordIndex(MFInt32* pTexCoord)
{
	pTexCoordIndex[pt]=pTexCoord;
	pt++;
}

void VertexBuffer::AddNormalIndex(MFInt32* pNormalInd)
{
	pNormalIndex[pn] = pNormalInd;
	pn++;
}

void VertexBuffer::AddShape(M_Shape* pShp)
{
	pShape[ps]=pShp;
	ps++;
}

bool VertexBuffer::hasTexCoord()
{
	return (pTexCoord && pTexCoord->point.count > 0);
}

bool VertexBuffer::hasNormal()
{
	return (pNormal && pNormal->vector.count > 0);
}

bool VertexBuffer::hasBoneInfluences()
{
	return hasBoneInfl;
}

void VertexBuffer::CreateVB()
{
	if (!VB)
	{
		hasBoneInfl = false;

		unsigned int i,size=0;
		for (i=0;i<vertCount;i++)
		{
			size+=buffer[i].pt;
		}

		VBSize=size;

		VBstruct* vbs=(VBstruct*)malloc(VBSize*sizeof(VBstruct));
		unsigned int k=0;

		for (i=0;i<vertCount;i++)
		{
			buffer[i].offset=k;
			int j;
			for (j=0;j<buffer[i].pt;j++)
			{				
				vbs[k].x =   pCoord->point.vals[i].x;
				vbs[k].y =   pCoord->point.vals[i].y;
				vbs[k].z =   pCoord->point.vals[i].z;

				if ( pTexCoord && pTexCoord->point.count > 0 )
				{
					vbs[k].u = pTexCoord->point.vals[buffer[i].texture[j]].x;
					vbs[k].v = 1.f - pTexCoord->point.vals[buffer[i].texture[j]].y;
				}
				else
				{
					vbs[k].u = 0.f;
					vbs[k].v = 0.f;
				}

				if ( pNormal && pNormal->vector.count > 0 )
				{
					int nindex = buffer[i].normal[j];
					if (nindex == -1)
						nindex = i;
					vbs[k].nx = pNormal->vector.vals[nindex].x;
					vbs[k].ny = pNormal->vector.vals[nindex].y;
					vbs[k].nz = pNormal->vector.vals[nindex].z;
				}
				else
				{
					vbs[k].nx = 0;
					vbs[k].ny = 0;
					vbs[k].nz = -1;
				}

				hasBoneInfl = hasBoneInfl || ( buffer[i].bone != 0 );
				vbs[k].iw = buffer[i].bone;
				memcpy(vbs[k].w,buffer[i].infl,4*sizeof(float));

				k++;
			}

		}
		VB=vbs;

		for (i=0;i<pc;i++)
		{
			indexBuffCount[i]=pCoordIndex[i]->count;
			indexBuff[i]=(int*)malloc(pCoordIndex[i]->count*sizeof(int));
			unsigned int j;
			for (j=0;j<pCoordIndex[i]->count;j++)
			{
				if (pCoordIndex[i]->vals[j]!=-1)					
				{
					if (pTexCoordIndex[i] && pTexCoordIndex[i]->count>0)
						if (pNormalIndex[i] && pNormalIndex[i]->count>0)
							indexBuff[i][j]=buffer[pCoordIndex[i]->vals[j]].getTextureIndexOffset(pTexCoordIndex[i]->vals[j], pNormalIndex[i]->vals[j]);
						else
							indexBuff[i][j]=buffer[pCoordIndex[i]->vals[j]].getTextureIndexOffset(pTexCoordIndex[i]->vals[j]);
					else
						if (pNormalIndex[i] && pNormalIndex[i]->count>0)
							indexBuff[i][j]=buffer[pCoordIndex[i]->vals[j]].getTextureIndexOffset(pCoordIndex[i]->vals[j], pNormalIndex[i]->vals[j]);
						else
							indexBuff[i][j]=buffer[pCoordIndex[i]->vals[j]].getTextureIndexOffset(pCoordIndex[i]->vals[j]);
				}
				else
					indexBuff[i][j]=-1;
			}				
		}
	}
}


Vertex::Vertex(int vertexIndex)
{
	vertex=vertexIndex;
	memset(texture,0,20*sizeof(int));
	pt=0;
	pb=0;
	bone=0;
	infl[0]=1.0f;
	infl[1]=0;
	infl[2]=0;
	infl[3]=0;
	offset=0;
}

void Vertex::AddTextureIndex( int textureIndex, int normalIndex )
{
	int i;
	for (i=0;i<pt;i++)
		if ( texture[i] == textureIndex && normal[i] == normalIndex)
			return;
	texture[pt]=textureIndex;
	normal[pt] = normalIndex;
	pt++;
}

int Vertex::getTextureIndexOffset( int textureIndex, int normalIndex )
{
	int i;
	for (i=0;i<pt;i++)
		if ( texture[i] == textureIndex && normal[i] == normalIndex )
			return offset+i;
	assert(0);
	return 0;
}

void Vertex::AddInfluense(int boneID,float influence)
{
	//assert(pb!=4);
	if (pb==4)
		return;
	boneID=boneID<<(pb*8);
	if (pb)
		bone |= boneID;
	else
		bone = boneID;
	infl[pb] = influence;
	pb++;		
}