// BIFStoVBLib.h: interface for the BIFStoVBLib class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _BIFSTOVBLIB_H_
#define _BIFSTOVBLIB_H_

//Vertex buffer types

enum vbType{
	VBTYPE_NONE=0,
	VBTYPE_SKIN=1,
	VBTYPE_RIGD=2,
	VBTYPE_BONE=3
};


extern "C" 
	{
		#include "gpac/nodes_mpeg4.h"
		#include "gpac/list.h"
		#include "gpac/internal/bifs_dev.h"
		#include "gpac/internal/scenegraph_dev.h"
		#include "gpac/internal/isomedia_dev.h"
	}
#include "math.h"
#include "matrix4.h"

class Vertex
{
public:

	int offset;

	int vertex;	

	int pt;
	int texture[50];
	int normal[50];
	
	int pb;
	int bone;
	float infl[4];

	Vertex(int vertexIndex=-1);

	void AddTextureIndex( int textureIndex, int normalIndex = -1 );
	int getTextureIndexOffset( int textureIndex, int normalIndex = -1 );
	void AddInfluense( int boneID, float influence );

};

typedef struct 
{
	float x,y,z;
	float w[4];
	unsigned long iw;

	float nx,ny,nz;
	float u,v;
} VBstruct;

class VertexBuffer
{
private:
	bool hasBoneInfl;

public:

	vbType type;

	unsigned int vertCount;
	Vertex* buffer;

	M_Coordinate* pCoord;
	M_TextureCoordinate* pTexCoord;
	M_Normal* pNormal;

	M_MorphShape* pMorphShape;

	unsigned int ps;
	M_Shape* pShape[50];

	unsigned int pc;
	int		indexBuffCount[50];
	int*	indexBuff[50];
	MFInt32* pCoordIndex[50];

	unsigned int pt;
	MFInt32* pTexCoordIndex[50];

	unsigned int pn;
	MFInt32* pNormalIndex[50];

	VBstruct* VB;
	unsigned int VBSize;

	int* IB;
	unsigned int IBSize;

	VertexBuffer(unsigned int count);
	virtual ~VertexBuffer();

	void AddCoordIndex(MFInt32* pCoord);
	void AddTexCoordIndex(MFInt32* pTexCoord);
	void AddNormalIndex(MFInt32* pNormalInd);
	void AddShape(M_Shape* pShp);
	void CreateVB();

	bool hasTexCoord();
	bool hasNormal();
	bool hasBoneInfluences();
};

class BIFStoVBLib  
{
public:
	int origVerCount;
	bool m_bUseTexCoord;
	BIFStoVBLib();
	virtual ~BIFStoVBLib();

	GF_List* VBlist;
	
	void CreateBoneSkin(int &boneNum, M_SBBone* pBone, matrix4 par, VertexBuffer* vert, M_Coordinate* pCoord, M_Normal* pNorm );
	void BoneSkin( __tag_scene_graph* pScene, M_SBSkinnedModel* pSbs, VertexBuffer* &pBuff );

	void AllignTexCoordIndex( void* TexCoord, VertexBuffer** vert, bool useTextCoord );

	void CreateVB(__tag_scene_graph* pScene, bool useTexCoord = true);

	unsigned int GetVBCount(){return gf_list_count(VBlist);}
	bool GetVB(unsigned int index,VBstruct** buff,int &size);
	bool GetVB(unsigned int index,VertexBuffer** vb);
};

#endif // ifndef _BIFSTOVBLIB_H_
