// CoordIKeyValueHeader.h: key value encoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-04-19

#ifndef _COORDIKEYVALUEHEADER_H_
#define _COORDIKEYVALUEHEADER_H_

#include "CIKeyValueMinMax.h"
#include "Entropy.h"
namespace CI {
class CoordIKeyValueHeader
	{
  public:
	// variables: bitstream data of key value
	bool			bTranspose;								// flag of transposition
	int				nKVQBit;								// quantiztion bit size for key value decoding
	int				nCoordQBit;								// coding bit size
	int				nNumberOfCoord;							// the number of vertices
	int				nKVDigit;								// key value digit
	CIKeyValueMinMax	kVMinMax;								// class of reading minimum and maximum for key value
	int				nXQMin_Min;								// minimum value among the minimum values to the quantized x values
	int				nXQMin_Max;								// minimum value among the maximum values to the quantized x values
	int				nYQMin_Min;								// minimum value among the minimum values to the quantized y values
	int				nYQMin_Max;								// minimum value among the maximum values to the quantized y values
	int				nZQMin_Min;								// minimum value among the minimum values to the quantized z values
	int				nZQMin_Max;								// minimum value among the maximum values to the quantized z values
	int				nXQMax_Min;								// maximum value among the minimum values to the quantized x values
	int				nXQMax_Max;								// maximum value among the maximum values to the quantized x values
	int				nYQMax_Min;								// maximum value among the minimum values to the quantized y values
	int				nYQMax_Max;								// maximum value among the maximum values to the quantized y values
	int				nZQMax_Min;								// maximum value among the minimum values to the quantized z values
	int				nZQMax_Max;								// maximum value among the maximum values to the quantized z values

	// variables: nonparsable data
	int				nNumberOfCoord_org;

  public:
	CoordIKeyValueHeader(void)
		{
		bTranspose = false;
		nKVQBit = 0;
		nCoordQBit = 0;
		nNumberOfCoord = 0;
		nKVDigit = 0;
		nXQMin_Min = 0;
		nXQMin_Max = 0;
		nYQMin_Min = 0;
		nYQMin_Max = 0;
		nZQMin_Min = 0;
		nZQMin_Max = 0;
		nXQMax_Min = 0;
		nXQMax_Max = 0;
		nYQMax_Min = 0;
		nYQMax_Max = 0;
		nZQMax_Min = 0;
		nZQMax_Max = 0;

		nNumberOfCoord_org = 0;
		}

	void makeBitstream(CICAC_QF& codec, int& nBitsOfKeyValueHeader)
		{
		// variables: temporal data
		int			nQStep = 0;								// temporal QStep
		int			i = 0;									// index

		ofstream fout("fout-encoding.txt", ios::app);
		fout << "[Start: KeyValueHeader]" << endl;

		// put: flag of transposition
		if(bTranspose == 0)
			{
			codec.putbits(0, 1);
			fout << "bTranspose: " << 0 << endl;
			}
		else
			{
			codec.putbits(1, 1);
			fout << "bTranspose: " << 1 << endl;
			}
		nBitsOfKeyValueHeader += 1;

		// put: nKVQBit
		codec.putbits(nKVQBit, 5);
		fout << "nQStep_KeyValue: " << nKVQBit << endl;
		nBitsOfKeyValueHeader += 5;

		// put: the number of vertices
		nCoordQBit = getNbBitsD(nNumberOfCoord_org);
		codec.putbits(nCoordQBit, 5);
		fout << "nQStep_Coord: " << nCoordQBit << endl;
		nBitsOfKeyValueHeader += 5;
		codec.putbits(nNumberOfCoord_org, nCoordQBit);
		fout << "nNumberOfCoord: " << nNumberOfCoord_org << endl;
		nBitsOfKeyValueHeader += nCoordQBit;

		// put: keyValue digit
		codec.putbits(nKVDigit, 4);
		fout << "nKeyValueDigit: " << nKVDigit << endl;
		nBitsOfKeyValueHeader += 4;

		int nBitsOfStart = (codec.GetBuffernbytes()-1)*8 + (codec.GetBuffernbits()-1);
		// put: keyValue max, min
		if(kVMinMax.nWhichAxis == 0)
			{
			kVMinMax.fMax = kVMinMax.fMax_X;
			kVMinMax.makeBitstream(codec, nKVDigit);
			}
		else if(kVMinMax.nWhichAxis == 1)
			{
			kVMinMax.fMax = kVMinMax.fMax_Y;
			kVMinMax.makeBitstream(codec, nKVDigit);
			}
		else if(kVMinMax.nWhichAxis == 2)
			{
			kVMinMax.fMax = kVMinMax.fMax_Z;
			kVMinMax.makeBitstream(codec, nKVDigit);
			}
		int nBitsOfEnd = (codec.GetBuffernbytes()-1)*8 + (codec.GetBuffernbits()-1);
		nBitsOfKeyValueHeader += nBitsOfEnd - nBitsOfStart;

		// put: min, max of quantized min values for each component
		codec.putbits(nXQMin_Min, nKVQBit);
		fout << "nX_QMin_min: " << nXQMin_Min << endl;
		codec.putbits(nXQMin_Max, nKVQBit);
		fout << "nX_QMin_max: " << nXQMin_Max << endl;

		codec.putbits(nYQMin_Min, nKVQBit);
		fout << "nY_QMin_min: " << nYQMin_Min << endl;
		codec.putbits(nYQMin_Max, nKVQBit);
		fout << "nY_QMin_max: " << nYQMin_Max << endl;

		codec.putbits(nZQMin_Min, nKVQBit);
		fout << "nZ_QMin_min: " << nZQMin_Min << endl;
		codec.putbits(nZQMin_Max, nKVQBit);
		fout << "nZ_QMin_max: " << nZQMin_Max << endl;

		// put: min, max of quantized max values for each component
		codec.putbits(nXQMax_Min, nKVQBit);
		fout << "nX_QMax_min: " << nXQMax_Min << endl;
		codec.putbits(nXQMax_Max, nKVQBit);
		fout << "nX_QMax_max: " << nXQMax_Max << endl;

		codec.putbits(nYQMax_Min, nKVQBit);
		fout << "nY_QMax_min: " << nYQMax_Min << endl;
		codec.putbits(nYQMax_Max, nKVQBit);
		fout << "nY_QMax_max: " << nYQMax_Max << endl;

		codec.putbits(nZQMax_Min, nKVQBit);
		fout << "nZ_QMax_min: " << nZQMax_Min << endl;
		codec.putbits(nZQMax_Max, nKVQBit);
		fout << "nZ_QMax_max: " << nZQMax_Max << endl;

		fout << "[End: KeyValueHeader]" << endl;
		fout << endl;
		fout.close();
		}

	void encodingProcess(void)
		{
		nNumberOfCoord_org = nNumberOfCoord;
		}

  private:
	int getNbBitsD(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			j++;
		
		return j;
		}
	};
}
#endif // _COORDIKEYVALUEHEADER_H_