/******************************************************************************\
* Includes
\******************************************************************************/
#include "MP43DMC.h"

/******************************************************************************\
 MP43DMC.cpp: implementation of the MP43DMC class.
\******************************************************************************/


/******************************************************************************\
* Class MP43DMC
\******************************************************************************/
//constructor
//should be given number of each component,  0 if not exist
MP43DMC::MP43DMC(int coordinate_number, int normal_number, int text_coordinate_number, int color_number, int index_coordinate_number, int index_normal_number, int index_text_coordinate_number, int index_color_number, int DMCID , const char * fileID )
{
	m_coordinatesPos			= 0;
	m_normalsPos				= 0;
	m_texturecoordinatesPos		= 0;
	m_colorsPos					= 0;
	m_coordintesIndexPos		= 0;
	m_normalsIndexPos			= 0;
	m_texturecoordinateIndexPos = 0;
	m_colorsIndexPos			= 0;
	m_coordinatesCount			= coordinate_number;
	m_normalsCount				= normal_number;
	m_texturecoordinatesCount	= text_coordinate_number;
	m_colorsCount				= color_number;
	m_coordintesIndexCount		= index_coordinate_number;
	m_normalsIndexCount			= index_normal_number;
	m_texturecoordinateIndexCount = index_text_coordinate_number;
	m_colorsIndexCount			= index_color_number;
	m_pcoordinates				= (float*) malloc( 3 * coordinate_number * sizeof ( float ));
	m_pnormals					= (float*) malloc( 3 * normal_number * sizeof ( float ));
	m_ptexturecoordinates		= (float*) malloc( 2 * text_coordinate_number * sizeof ( float ));
	m_pcolors					= (float*) malloc( 3 * color_number * sizeof ( float ));
	m_pcoordintesIndex			= (int*) malloc( 4 * index_coordinate_number * sizeof ( int ));
	m_pnormalsIndex				= (int*) malloc( 4 * index_normal_number * sizeof ( int ));
	m_ptexturecoordinateIndex	= (int*) malloc( 4 * index_text_coordinate_number * sizeof ( int ));
	m_pcolorsIndex				= (int*) malloc( 4 * index_color_number * sizeof ( int ));

	m_3DMCID	= DMCID;
	m_pfileID	= strdup(fileID);
}



//deconstructor
MP43DMC::~MP43DMC(void)
{
	if ( m_pcoordinates )
		free ( m_pcoordinates );
	if ( m_pnormals )
		free ( m_pnormals );
	if ( m_ptexturecoordinates )
		free ( m_ptexturecoordinates );
	if ( m_pcolors )
		free ( m_pcolors );
	if ( m_pcoordintesIndex )
		free ( m_pcoordintesIndex );
	if ( m_pnormalsIndex )
		free ( m_pnormalsIndex );
	if ( m_ptexturecoordinateIndex )
		free ( m_ptexturecoordinateIndex );
	if ( m_pcolorsIndex )
		free ( m_pcolorsIndex );
	if ( m_pfileID )
		m_pfileID;
}

bool MP43DMC::AddCoordinate(float x, float y, float z)
{
	m_pcoordinates[m_coordinatesPos++] = x;
	m_pcoordinates[m_coordinatesPos++] = y;
	m_pcoordinates[m_coordinatesPos++] = z;
	return true;
}

bool MP43DMC::AddNormal(float nx, float ny, float nz)
{
	m_pnormals[m_normalsPos++] = nx;
	m_pnormals[m_normalsPos++] = ny;
	m_pnormals[m_normalsPos++] = nz;
	return true;
}

bool MP43DMC::AddTextureCoordinate(float u, float v)
{
	m_ptexturecoordinates[m_texturecoordinatesPos++] = u;
	m_ptexturecoordinates[m_texturecoordinatesPos++] = v;
	return true;
}


bool MP43DMC::AddColor(float r, float g, float b)
{
	m_pcolors[m_colorsPos++] = r;
	m_pcolors[m_colorsPos++] = g;
	m_pcolors[m_colorsPos++] = b;
	return true;

}

bool MP43DMC::AddCoordinateIndex(int first, int second, int third)
{
	m_pcoordintesIndex[m_coordintesIndexPos++] = first;
	m_pcoordintesIndex[m_coordintesIndexPos++] = second;
	m_pcoordintesIndex[m_coordintesIndexPos++] = third;
	m_pcoordintesIndex[m_coordintesIndexPos++] = -1;
	return true;

}

bool MP43DMC::AddNormalIndex(int first, int second, int third)
{
	m_pnormalsIndex[m_normalsIndexPos++] = first;
	m_pnormalsIndex[m_normalsIndexPos++] = second;
	m_pnormalsIndex[m_normalsIndexPos++] = third;
	m_pnormalsIndex[m_normalsIndexPos++] = -1;
	return true;

}

bool MP43DMC::AddTextureCoordinateIndex(int first, int second, int third)
{
	m_ptexturecoordinateIndex[m_texturecoordinateIndexPos++] = first;
	m_ptexturecoordinateIndex[m_texturecoordinateIndexPos++] = second;
	m_ptexturecoordinateIndex[m_texturecoordinateIndexPos++] = third;
	m_ptexturecoordinateIndex[m_texturecoordinateIndexPos++] = -1;
	return true;

}

bool MP43DMC::AddColorIndex(int first, int second, int third)
{
	m_pcolorsIndex[m_colorsIndexPos++] = first;
	m_pcolorsIndex[m_colorsIndexPos++] = second;
	m_pcolorsIndex[m_colorsIndexPos++] = third;
	m_pcolorsIndex[m_colorsIndexPos++] = -1;
	return true;

}