/******************************************************************************\
* Includes
\******************************************************************************/
#include "MP4Animation.h"

/******************************************************************************\
 MP4Animation.cpp: implementation of the:
					MP4BoneData class
					MP4Frames class
					MP4AnimationBuff class
\******************************************************************************/


/******************************************************************************\
* Class MP4BoneData
\******************************************************************************/
MP4BoneData ::MP4BoneData(void)
{
	 m_hasTranslation		= false;
	 m_isQuaternion			= false;
	 m_hasRotation			= false;
	 m_hasCenter			= false;
	 m_hasScaleOrientation	= false;
	 m_hasScale				= false;
}

MP4BoneData ::~MP4BoneData(void)
{
}
bool MP4BoneData::GetTranslation( Vec3f &translation )
{
	if (!m_hasTranslation)
		return false;
	translation.x = m_Translation.x;
	translation.y = m_Translation.y;
	translation.z = m_Translation.z;
	return true;
}

bool MP4BoneData::GetRotation ( Vec4f &rotation )
{
	if (!m_hasRotation )
		return false;
	rotation.x = m_Rotation.x;
	rotation.y = m_Rotation.y;
	rotation.z = m_Rotation.z;
	rotation.q = m_Rotation.q;
	return true;
	
}
bool MP4BoneData::GetCenter( Vec3f &center )
{
	if ( !m_hasCenter )
		return false;
	center.x = m_Center.x;
	center.y = m_Center.y;
	center.z = m_Center.z;
	return true;
}
	
bool MP4BoneData::GetscaleOrienttion( Vec4f &scaleOrientation )
{
	if (!m_hasScaleOrientation )
		return false;
	scaleOrientation.x = m_ScaleOrientation.x;
	scaleOrientation.y = m_ScaleOrientation.y;
	scaleOrientation.z = m_ScaleOrientation.z;
	scaleOrientation.q = m_ScaleOrientation.q;
	return true;
}
	
bool MP4BoneData::GetScale( Vec3f &scale )
{
	if ( !m_hasScale )
		return false;
	scale.x = m_Scale.x;
	scale.y = m_Scale.y;
	scale.z = m_Scale.z;
	return true;
}


bool MP4BoneData::SetTranslation( Vec3f translation )
{
	if (m_hasTranslation)
		return false;
	m_hasTranslation = true;
	m_Translation.x = translation.x;
	m_Translation.y = translation.y;
	m_Translation.z = translation.z;
	return true;
}

bool MP4BoneData::SetRotation ( Vec4f rotation )
{
	if ( m_hasRotation )
		return false;
	m_hasRotation = true;
	m_isQuaternion = true;
	m_Rotation.x = rotation.x;
	m_Rotation.y = rotation.y;
	m_Rotation.z = rotation.z;
	m_Rotation.q = rotation.q;
	return true;
	
}
bool MP4BoneData::SetCenter( Vec3f center )
{
	if ( m_hasCenter )
		return false;
	m_hasCenter= true;
	m_Center.x = center.x;
	m_Center.y = center.y;
	m_Center.z = center.z;
	return true;
}
	
bool MP4BoneData::SetscaleOrienttion( Vec4f scaleOrientation )
{
	if ( m_hasScaleOrientation )
		return false;
	m_hasScaleOrientation= true;
	m_ScaleOrientation.x = scaleOrientation.x;
	m_ScaleOrientation.y = scaleOrientation.y;
	m_ScaleOrientation.z = scaleOrientation.z;
	m_ScaleOrientation.q = scaleOrientation.q;
	return true;
}
	
bool MP4BoneData::SetScale( Vec3f scale )
{
	if ( m_hasScale )
		return false;
	m_hasScale = true;
	m_Scale.x = scale.x;
	m_Scale.y = scale.y;
	m_Scale.z = scale.z ;
	return true;
}

/******************************************************************************\
* Class MP4Frames
\******************************************************************************/

MP4Frames::MP4Frames(void)
{
	m_size = 0;
	m_boneDataCount = 0;
	m_pbonesData = NULL;
}

MP4Frames::~MP4Frames(void)
{
	for (int i = 0; i < m_boneDataCount; i++)
		delete m_pbonesData[i];
	delete[] m_pbonesData;
}

void MP4Frames::Init(unsigned int BoneCount)
{
	m_pbonesData = new MP4BoneData*[BoneCount];
}

MP4BoneData* MP4Frames::GetMP4BoneData( int boneData_no )
{
	if ((boneData_no < 0) || ( boneData_no >= m_boneDataCount ))
		 return NULL;
	return m_pbonesData[boneData_no];
}

float* MP4Frames::GetMP4BoneDataMatrix( int boneData_no )
{
	if ((boneData_no < 0) || ( boneData_no >= m_boneDataCount ))
		 return NULL;
	return ConvertToMatrix ( boneData_no );
}

float* MP4Frames::ConvertToMatrix( int boneData_no )
{
	Vec4f Rot;
	Vec4f ScRot;
	Vec3f Trans;
	Vec3f Scale;
	Vec3f Cen;

	matrix rot,scRot,cen,trans,scale;

	if (m_pbonesData[boneData_no]->GetRotation( Rot ))
		rot.Rotation(Rot);

	if (m_pbonesData[boneData_no]->GetscaleOrienttion( ScRot ))
		scRot.Rotation(ScRot);

	if (m_pbonesData[boneData_no]->GetTranslation( Trans ))
		trans.Translation(Trans);

	if (m_pbonesData[boneData_no]->GetScale( Scale ))
		scale.Scale(Scale);
		
	if (m_pbonesData[boneData_no]->GetCenter( Cen ))
		cen.Translation(Cen);

	matrix rez;
	
	rez = rot;
	rez = rez * trans;

	//rez=rez*rot;
	//rez=rez*cen;

	float* mtx;
	mtx = (float*) malloc(16 * sizeof(float));
	for (int i=0;i<16;i++)
		mtx[i] = rez.Data(i/4, i%4);

	return mtx;
	
	
}


bool MP4Frames::AddMP4BoneData ( MP4BoneData *mp4boneData )
{
	m_pbonesData[ m_boneDataCount++ ] = mp4boneData;
	return true;
}





/******************************************************************************\
* Class MP4AnimationBuff
\******************************************************************************/
MP4AnimationBuff::MP4AnimationBuff( int animID, const char* fileID, int frameCount )
{
	m_NoFrames = 0;
	m_hasHierarchy = false;
	m_pframes = NULL;
	m_animID	= animID;
	m_pfileID	= strdup (fileID);

	m_allocCount  = frameCount;
	m_pframes = new MP4Frames*[frameCount];
}

MP4AnimationBuff::~MP4AnimationBuff(void)
{
	delete[] m_pframes;
	free ( m_pfileID);
}

MP4Frames* MP4AnimationBuff::GetMP4Frame( int frame_no )
{
	 if ((frame_no < 0) || ( frame_no >= m_NoFrames ))
		 return NULL;
	 return m_pframes[frame_no];
}



bool MP4AnimationBuff::AddMP4Frame( MP4Frames* mp4frame )
{
	if( m_NoFrames == m_allocCount)
	{
		m_allocCount += 10;
		MP4Frames** tmp = new MP4Frames*[m_allocCount];
		memcpy(tmp, m_pframes, m_NoFrames * sizeof(MP4Frames*));
		delete[] m_pframes;
		m_pframes = tmp;
	}
	m_pframes[ m_NoFrames++ ] = mp4frame;
	return true;

}



