#ifndef _MP4ANIMATION_H_
#define _MP4ANIMATION_H_

/******************************************************************************\
* Includes
\******************************************************************************/
#include <stdlib.h>
#include <string.h>
#include <vector>
#include "matrix.h"



/******************************************************************************\
* Class MP4BoneData
\******************************************************************************/
class MP4BoneData
{
public:	

	MP4BoneData(void);
	~MP4BoneData(void);

	//methods
	bool GetHasTranslation(){ return m_hasTranslation ;}
	bool GetIsQuaternion(){ return  m_isQuaternion ;}
	bool GetHasRotation(){ return m_hasRotation ;}
	bool GetHasCenter(){ return  m_hasCenter ;}
	bool GetHasScaleOrientation(){ return m_hasScaleOrientation ;}
	bool GetHasScale(){ return m_hasScale ;}
	bool GetTranslation( Vec3f &translation );
	bool GetRotation ( Vec4f &rotation );
	bool GetCenter( Vec3f &center );
	bool GetscaleOrienttion( Vec4f &scaleOrientation );
	bool GetScale( Vec3f &m_scale );
	bool SetTranslation( Vec3f translation );
	bool SetRotation ( Vec4f rotation );
	bool SetCenter( Vec3f center );
	bool SetscaleOrienttion( Vec4f scaleOrientation );
	bool SetScale( Vec3f m_scale );

//private:	
	//variables
	//int	 m_boneDataMask;  / not used
	bool m_hasTranslation;
	bool m_isQuaternion;
	bool m_hasRotation;
	bool m_hasCenter;
	bool m_hasScaleOrientation;
	bool m_hasScale;
//private:
	Vec3f m_Translation;
	Vec4f m_Rotation;
	Vec3f m_Center;
	Vec4f m_ScaleOrientation;
	Vec3f m_Scale;
	
};

/******************************************************************************\
* Class MP4Frames
\******************************************************************************/
class MP4Frames
{
public:

	MP4Frames(void);
	~MP4Frames(void);

	//methods
	void Init(unsigned int BoneCount);
	int GetMP4BoneDataCount() { return  m_boneDataCount; }
	MP4BoneData* GetMP4BoneData( int boneData_no );
	bool AddMP4BoneData ( MP4BoneData *mp4boneData );
	float* GetMP4BoneDataMatrix( int boneData_no );

//private:
	//variables
	int m_size;
	int m_boneDataCount;
	MP4BoneData **m_pbonesData;
	//methods
	float* ConvertToMatrix( int boneData_no );
	
};

/******************************************************************************\
* Class MP4AnimationBuff
\******************************************************************************/
class MP4AnimationBuff
{
public:

	MP4AnimationBuff( int animID = 0, const char* fileID = "",  int frameCount = 10 );
	~MP4AnimationBuff(void);

	//method
	int GetMP4FramesCount () { return m_NoFrames; }
	int GetMP4HierarchySize() { return m_size; }
	bool GetHasHierarhy() {return m_hasHierarchy; }
	MP4Frames* GetMP4Frame( int frame_no );
	bool AddMP4Frame( MP4Frames* mp4frame );
	bool GetAnimationID(){ return m_animID; }
	const char* GetAnimationFileID(){ return m_pfileID; }
	void SetAnimationFileID( const char* fileID ){ if( m_pfileID) free(m_pfileID); m_pfileID = strdup(fileID); }

//private:
	//variables
	int				m_allocCount;
	int				m_size;
	bool			m_hasHierarchy;
	int				m_NoFrames;
	MP4Frames		**m_pframes;

	int				m_animID;
	char*		m_pfileID;

	// CI
	std::vector<double> m_key; 
	std::vector<double> m_keyvalue; 

	int EncodeType;
	//to do::
	//add variables for parametars for BBAEncoder


};

#endif //_MP4ANIMATION_H_


