
#include "MP4toVBLib.h"

extern "C"
{
#include "gpac/isomedia.h"
#include "gpac/internal/isomedia_dev.h"
#include "gpac/nodes_mpeg4.h"
#include "gpac/mpeg4_odf.h"
#include "gpac/internal/scenegraph_dev.h"
#include "gpac/internal/bifs_dev.h"
#include "gpac/scenegraph.h"
	
	
}

#include "BIFStoVBLib.h"
#include "jasper/jasper.h"
#include "ostream.h"

#include <vector>

#define DMCESID	  1
class MP4toVBLib
{
public:
	char	*outBuffer;
	int		outBufferSize;

	char*	outBIFS;
	int		outBIFSSize;

	char	*appBuffer;
	int		appBufferSize;

	int				outTexturesCount;
	char**			outTextures;
	int*			outTexturesSize;
	textureType*	outTexturesType;

	int				out3DMCCount;
	char**			out3DMC;
	int*			out3DMCSize;

	int				outBBACount;
	char**			outBBA;
	int*			outBBASize;

	int				outFAMCCount;
	char**			outFAMC;
	int*			outFAMCSize;

	char*			outRAR;
	int				outRARSize;



	GF_ISOFile		*m_mov;
	GF_SceneGraph	*m_scene;

	MP4toVBLib();
	~MP4toVBLib();
	void freeData();

	int LoadMP4File( const char * FileName, outputType type );
	int GetVBBufferSize();
	int GetBIFSSize();
	int GetBIFS( unsigned char* &buff, int &size, bool alloc );
	int GetVB( unsigned char* &buff, int &size, bool alloc = true );
	int GetAppBufferSize();
	int GetAppBuffer( unsigned char* &buff, int &size, bool alloc = true );
	int GetTextureCount();
	int GetTextureSize( unsigned int texID );
	int GetTexture( unsigned int texID, unsigned char* &buff, int &size, textureType &type, bool alloc = true );
	int DecompressTextures();
	//int DecodeJPEG2000( int texID );
	int GetBBACount();
	int GetBBASize( unsigned int ID );
	int GetBBA( unsigned int ID, unsigned char* &buff, int &size, bool alloc = true );
	int GetBBAbyRef( const char* metaName, unsigned char* &buff, int &size,  bool alloc = true );
	int Get3DMCCount();
	int Get3DMCSize( unsigned int ID );
	int Get3DMC( unsigned int ID, unsigned char* &buff, int &size,  bool alloc );
	int Get3DMCbyMetaName( const char* metaName, unsigned char* &buff, int &size, bool alloc );
	int Get3DMCbyRef( const char* metaName, unsigned char* &buff, int &size,  bool alloc );
	int GetFAMCCount();
	int GetFAMCSize( unsigned int ID );
	int GetFAMC( unsigned int ID, unsigned char* &buff, int &size,  bool alloc );
	int GetFAMCbyRef( const char* metaName, unsigned char* &buff, int &size,  bool alloc );
	int GetTexturebyRef( const char* metaName, unsigned char* &buff, int &size,  bool alloc );
	int GetRAR( unsigned char* &buff, int &size,  bool alloc );
	int GetRARfromMETA(  unsigned char* &buff, int &size,  bool alloc );
	int GetXMLfromMETA( unsigned char* &buff, int &size,  bool alloc );

private:
	int* ODs ;
	int* texIDs;
	int* dmcIDs;
	int* BBAIDs ;
	int* famcIDs;
};

MP4toVBLib::MP4toVBLib()
{
	outBuffer			= NULL;
	outBufferSize		= 0;
	outBIFS				= NULL;
	outBIFSSize			= 0;

	outTexturesCount	= 0;
	outTextures			= NULL;
	outTexturesSize		= NULL;

	appBuffer			= NULL;
	appBufferSize		= 0;

	m_scene				= NULL;
	m_mov				= NULL;

	ODs = NULL;
	texIDs = NULL;
	dmcIDs = NULL;
	BBAIDs = NULL;
	famcIDs = NULL;
}

MP4toVBLib::~MP4toVBLib()
{
	freeData();
}

void MP4toVBLib::freeData()
{
	if ( m_scene )
	{
		gf_sg_del( m_scene );
		m_scene = NULL;
	}
	if ( m_mov )
	{
		gf_isom_close( m_mov );
		m_mov = NULL;
	}	
	
	if ( outBuffer )
	{
		free( outBuffer );
		outBuffer = NULL;
	}
	outBufferSize = 0;
	outBIFSSize = 0;
	if ( outBIFS )
	{
		free( outBIFS );
		outBIFS = NULL;
	}

	if ( appBuffer )
	{
		free( appBuffer );
		appBuffer = NULL;
	}
	appBufferSize = 0;

	if ( outTextures )
	{
		for ( int i = 0; i < outTexturesCount; i++ )
		{
			free( outTextures[i] );
		}
		free( outTextures );
		free( outTexturesSize );
		free( outTexturesType );
	}
	outTexturesCount = 0;
}

int MP4toVBLib::GetVBBufferSize()
{
	return outBufferSize;
}

int MP4toVBLib::GetBIFSSize()
{
	return outBIFSSize;
}

int MP4toVBLib::GetBIFS( unsigned char* &buff, int &size , bool alloc )
{
	size = outBIFSSize;
	if ( alloc )
		buff = (unsigned char*) malloc( size );
	memcpy( buff, outBIFS, size );
	return size;
}

int MP4toVBLib::GetVB( unsigned char* &buff, int &size, bool alloc )
{
	size = outBufferSize;
	if ( alloc )
		buff = (unsigned char*) malloc( size );
	memcpy( buff, outBuffer, size );
	return size;
}

int MP4toVBLib::GetAppBufferSize()
{
	return appBufferSize;
}

int MP4toVBLib::GetAppBuffer( unsigned char* &buff, int &size, bool alloc )
{
	size = appBufferSize;
	if ( alloc )
		buff = (unsigned char*) malloc( size );
	memcpy( buff, appBuffer, size );
	return size;
}

int MP4toVBLib::GetTextureCount()
{
	return outTexturesCount;
}

int MP4toVBLib::GetTextureSize( unsigned int texID )
{
	if ( texID < outTexturesCount )
		return outTexturesSize[texID];
	return -1;
}

int MP4toVBLib::GetTexture( unsigned int texID, unsigned char* &buff, int &size, textureType &type, bool alloc )
{
	if ( texID >= outTexturesCount )
		return -1;
	type = outTexturesType[texID];
	size = outTexturesSize[texID];
	if (alloc)
		buff = (unsigned char*) malloc( size );
	memcpy( buff, outTextures[texID], size );
	return outTexturesSize[texID];
}

int MP4toVBLib::DecompressTextures()
{
	for ( int i = 0; i < outTexturesCount; i++ )
	{
		switch ( outTexturesType[i] )
		{
			case TT_JPEG :
				//DecodeJPEG2000( i );
				break;
			case TT_JPEG2000 :
				//DecodeJPEG2000( i );
				break;
			default :
				break;
		}
	}
	return 0;
}

int MP4toVBLib::Get3DMCCount()
{
	return out3DMCCount;
}

int MP4toVBLib::Get3DMCSize( unsigned int ID )
{
	if ( ID < out3DMCCount )
		return out3DMCSize[ID];
	return -1;
}

int MP4toVBLib::Get3DMC( unsigned int ID, unsigned char* &buff, int &size,  bool alloc )
{
	if ( ID >= out3DMCCount )
		return -1;
	
	size = out3DMCSize[ID];
	if (alloc)
		buff = (unsigned char*) malloc( size );
	memcpy( buff, out3DMC[ID], size );
	return out3DMCSize[ID];
}

int MP4toVBLib::GetBBACount()
{
	return outBBACount;
}

int MP4toVBLib::GetBBASize( unsigned int ID )
{
	if ( ID < outBBACount )
		return outBBASize[ID];
	return -1;
}

int MP4toVBLib::GetBBA( unsigned int ID, unsigned char* &buff, int &size,  bool alloc )
{
	if ( ID >= outBBACount )
		return -1;
	
	size = outBBASize[ID];
	if (alloc)
		buff = (unsigned char*) malloc( size );
	memcpy( buff, outBBA[ID], size );
	return outBBASize[ID];
	
}

int MP4toVBLib::GetBBAbyRef( const char* metaName, unsigned char* &buff, int &size,  bool alloc )
{
	u32 itemID = 0 ;
	u32 protection_idx = 0 ;
	Bool is_self_reference = false;
	const char* item_name = NULL;
	const char* item_mime_type = NULL;
	const char* item_encoding = NULL;
	const char* item_url = NULL;
	const char* item_urn = NULL;

	for (int i=1; i<gf_isom_get_meta_item_count(m_mov, true,0)+1;i++)
	{
		gf_isom_get_meta_item_info(m_mov,true,0, i , &itemID,  &protection_idx, &is_self_reference, &item_name,&item_mime_type,&item_encoding,  &item_url, &item_urn);
			if ( !stricmp(item_name,metaName))
			{				
				
				int ID;
				sscanf(item_url, "#ffp(trackID=%d)", &ID);
				size = outBBASize[BBAIDs[ID]];
				if (alloc)
					buff = (unsigned char*) malloc( size );
				memcpy( buff, outBBA[BBAIDs[ID]], size );
				return size;
			}

	}
	return 0;


}

int MP4toVBLib::GetFAMCCount()
{
	return outFAMCCount;
}

int MP4toVBLib::GetFAMCSize( unsigned int ID )
{
	if ( ID < outFAMCCount )
		return outFAMCSize[ID];
	return -1;
}

int MP4toVBLib::GetFAMC( unsigned int ID, unsigned char* &buff, int &size,  bool alloc )
{
	if ( ID >= outFAMCCount )
		return -1;
	
	size = outFAMCSize[ID];
	if (alloc)
		buff = (unsigned char*) malloc( size );
	memcpy( buff, outFAMC[ID], size );
	return outFAMCSize[ID];
}


int MP4toVBLib::GetFAMCbyRef( const char* metaName, unsigned char* &buff, int &size,  bool alloc )
{
	u32 itemID = 0 ;
	u32 protection_idx = 0 ;
	Bool is_self_reference = false;
	const char* item_name = NULL;
	const char* item_mime_type = NULL;
	const char* item_encoding = NULL;
	const char* item_url = NULL;
	const char* item_urn = NULL;
	
	for (int i=1; i<gf_isom_get_meta_item_count(m_mov, true,0)+1;i++)
	{
	
		gf_isom_get_meta_item_info(m_mov,true,0, i , &itemID,  &protection_idx, &is_self_reference, &item_name,&item_mime_type,&item_encoding,  &item_url, &item_urn);
		if ( !stricmp(item_name,metaName))
		{				
			
			int ID;
			//sscanf(item_mime_type, "scene/3dmc;TrackID=%d", &ID);
			sscanf(item_url, "#ffp(trackID=%d)", &ID);
			if (famcIDs[ID] == -1)
				continue;
			size = outFAMCSize[famcIDs[ID]];
			
			if (alloc)
				buff = (unsigned char*) malloc( size );
			memcpy( buff, outFAMC[famcIDs[ID]], size );
			return size;
		}

	}
	return 0;
}

int MP4toVBLib::Get3DMCbyRef( const char* metaName, unsigned char* &buff, int &size,  bool alloc )
{
	u32 itemID = 0 ;
	u32 protection_idx = 0 ;
	Bool is_self_reference = false;
	const char* item_name = NULL;
	const char* item_mime_type = NULL;
	const char* item_encoding = NULL;
	const char* item_url = NULL;
	const char* item_urn = NULL;
//should be::
	//GF_MetaBox *mb =  gf_isom_get_meta( m_mov, true, 0 );
	//mb->itn.......
	//koga ke se najde metaName == item_name=url, se zema dataref za toj item
	
	

	for (int i=1; i<gf_isom_get_meta_item_count(m_mov, true,0)+1;i++)
	{
	
		gf_isom_get_meta_item_info(m_mov,true,0, i , &itemID,  &protection_idx, &is_self_reference, &item_name,&item_mime_type,&item_encoding,  &item_url, &item_urn);
			if ( !stricmp(item_name,metaName))
			{				
				
				int ID;
				//sscanf(item_mime_type, "scene/3dmc;TrackID=%d", &ID);
				sscanf(item_url, "#ffp(trackID=%d)", &ID);
				size = out3DMCSize[dmcIDs[ID]];
				if (alloc)
					buff = (unsigned char*) malloc( size );
				memcpy( buff, out3DMC[dmcIDs[ID]], size );
				return size;
			}

	}
	return 0;


}



int MP4toVBLib::Get3DMCbyMetaName( const char* metaName, unsigned char* &buff, int &size,  bool alloc )
{
	u32 itemID = 0 ;
	u32 protection_idx = 0 ;
	Bool is_self_reference = false;
	const char* item_name = NULL;
	const char* item_mime_type = NULL;
	const char* item_encoding = NULL;
	const char* item_url = NULL;
	const char* item_urn = NULL;
//should be::
	//GF_MetaBox *mb =  gf_isom_get_meta( m_mov, true, 0 );
	//mb->itn.......
	//koga ke se najde metaName == item_name=url, se zema dataref za toj item
	
	

	for (int i=1; i<gf_isom_get_meta_item_count(m_mov, true,0)+1;i++)
	{
		gf_isom_get_meta_item_info(m_mov,true,0, i , &itemID,  &protection_idx, &is_self_reference, &item_name,&item_mime_type,&item_encoding,  &item_url, &item_urn);
			if ( !stricmp(item_name,metaName))
			{				
				
				int ID;
				sscanf(item_mime_type, "scene/3dmc;TrackID=%d", &ID);
				size = out3DMCSize[ID - DMCESID];
				if (alloc)
					buff = (unsigned char*) malloc( size );
				memcpy( buff, out3DMC[ID - DMCESID], size );
				return size;
			}

	}
	return 0;


}

int MP4toVBLib::GetTexturebyRef( const char* metaName, unsigned char* &buff, int &size,  bool alloc )
{
	u32 itemID = 0 ;
	u32 protection_idx = 0 ;
	Bool is_self_reference = false;
	const char* item_name = NULL;
	const char* item_mime_type = NULL;
	const char* item_encoding = NULL;
	const char* item_url = NULL;
	const char* item_urn = NULL;

	for (int i=1; i<gf_isom_get_meta_item_count(m_mov, true,0)+1;i++)
	{
		gf_isom_get_meta_item_info(m_mov,true,0, i , &itemID,  &protection_idx, &is_self_reference, &item_name,&item_mime_type,&item_encoding,  &item_url, &item_urn);
			if ( !stricmp(item_name,metaName))
			{				
				
				int ID;
				sscanf(item_url, "#ffp(trackID=%d)", &ID);
				size = outTexturesSize[texIDs[ID]];
				if (alloc)
					buff = (unsigned char*) malloc( size );
				memcpy( buff, outTextures[texIDs[ID]], size );
				return size;
			}

	}
	return 0;


}



int MP4toVBLib::GetRAR(  unsigned char* &buff, int &size,  bool alloc )
{
	size = outRARSize;
	if (alloc)
		buff = (unsigned char*) malloc( size );
	memcpy( buff, outRAR, size );
	return outRARSize;
}

int MP4toVBLib::GetXMLfromMETA( unsigned char* &buff, int &size,  bool alloc )
{
	size = outRARSize;
	if (alloc)
		buff = (unsigned char*) malloc( size );
	memcpy( buff, outRAR, size );
	return outRARSize;
}
	
int MP4toVBLib::GetRARfromMETA(  unsigned char* &buff, int &size,  bool alloc )
{
	u32 itemID = 0;
	u32 protection_idx = 0;
	Bool is_self_reference = false;
	const char* item_name = NULL;
	const char* item_mime_type = NULL;
	const char* item_encoding = NULL;
	const char* item_url = NULL;
	const char* item_urn = NULL;
	
	for (int i=1; i<gf_isom_get_meta_item_count(m_mov, true,0)+1;i++)
	{
		//gf_isom_extract_meta_xml(....
		gf_isom_get_meta_item_info(m_mov,true,0, i , &itemID,  &protection_idx, &is_self_reference,  &item_name,&item_mime_type,&item_encoding,  &item_url, &item_urn);
			if (!stricmp(item_name, "rarScene"))
			{
				
				gf_isom_extract_meta_item(m_mov,true,0,i,"rarScene.gz");
				if (alloc)
					buff = (unsigned char*) malloc( size );
				FILE *rf;
				rf = fopen( "rarScene.gz", "rb");
				fseek(rf, 0, SEEK_END);
				size = ftell(rf);
				fseek(rf, 0, SEEK_SET);
				buff = (unsigned char*)malloc(size);
				fread(buff, 1, size, rf);
				fclose( rf );
				return size;
			}

	}
	return 0;

}

u32 M4_GetTrackByOD(int ODs[], u32 odID)
{
	for (int i=1;i<=ODs[0];i++)
		if (ODs[i]==odID) return i;
	return 0;
}

int MP4toVBLib::LoadMP4File(  const char * FileName, outputType type  )
{
	freeData();
	FILE *f;
	
	u64 bytes;

	m_mov = gf_isom_open( FileName, GF_ISOM_OPEN_READ, "" );
	if (! m_mov) return NULL;

	if (m_mov->LastError) {
		gf_isom_delete(m_mov);		
		return -1;
	}

	//get xml first
	Bool b;
	char* tmpfile= "xml.gz";
	gf_isom_extract_meta_xml(m_mov, true, 0, tmpfile , &b);

	FILE* f2=fopen(tmpfile, "rb");
	if ( !f2 )
		return 0;
	fseek(f2,0, SEEK_END);
	outRARSize = ftell(f2);
	fseek(f2,0,0);
	outRAR = (char*) malloc ( outRARSize );
	fread( outRAR, 1, outRARSize, f2 );
	fclose ( f2 );
	remove ( tmpfile );



	//get number of tracks
	int trc = gf_isom_get_track_count(m_mov);

	if (trc<=0)
	{
		//TRACE("No tracks in file");
		gf_isom_close( m_mov );
		m_mov = NULL;
		return -2;
	}

	int ODtrack = 0;
	int BIFStrack = 0;
	std::vector<int> trJpeg, trJpeg2000, trBba, trVideo, trAudio, tr3DMC, trRAR, trBBA, trFAMC;

	for ( int i = 1; i <= trc; i++ )
	{
		GF_ESD* pESD;
		GF_TrackBox *trak;
		trak = gf_isom_get_track( m_mov->moov, i);
		Media_GetESD(trak->Media, 1, &pESD, true);
		u32 streamtype = gf_isom_get_media_type(m_mov, i);
		switch ( streamtype )
		//switch (pESD->decoderConfig->streamType)
		{
			case GF_ISOM_MEDIA_OD:
				//init OD
				ODtrack = i;
				break;
			case GF_ISOM_MEDIA_VISUAL:

				switch (pESD->decoderConfig->objectTypeIndication)
				{
					case 108:
						//init JPEG texture
						trJpeg.push_back(i);
						break;
					case 110:
						//init JPEG2000 texture
						trJpeg2000.push_back(i);
						break;
					case 33:
						//init 3DMC Stream
						trRAR.push_back(i);
						break;
					case 100:
						//init 3DMC Stream
						trFAMC.push_back(i);
						break;
					case 55:
						//init rar buffer
						tr3DMC.push_back(i);
						break;
					
				}
				break;
			case GF_ISOM_MEDIA_SCENE:
				
				switch (pESD->decoderConfig->objectTypeIndication)
				{
					case 1:
					case 2:
						//init BIFS
						BIFStrack = i;
						break;
					// I put this =70
					case 5:
						//init bba buffer
						trBBA.push_back(i);
						break;
				}
				break;
		}
	}
//blagica, it is possible to don't have BIFS
	if (( !BIFStrack ) && (type > -1))
	{
		//sTRACE("No BIFS stream");
		return -3;
	}

	u32 StreamDescriptionIndex=0;
	GF_ISOSample* sample = NULL;
	/*int* ODs = NULL;
	int* texIDs = NULL;
	int* dmcIDs = NULL;
	int* BBAIDs = NULL;*/

	out3DMCCount = tr3DMC.size();
	outFAMCCount = trFAMC.size();
	outBBACount = trBBA.size();
	outTexturesCount	= trJpeg.size() + trJpeg2000.size();
	if ( outTexturesCount || out3DMCCount || outBBACount || outFAMCCount )
	{
		outTexturesSize		= (int*)			malloc( outTexturesCount * sizeof( int ) );
		outTexturesType		= (textureType*)	malloc( outTexturesCount * sizeof( textureType ) );
		outTextures			= (char**)			malloc( outTexturesCount * sizeof( char* ) );

		out3DMCSize		=    (int*)			malloc( out3DMCCount * sizeof( int ) );
		out3DMC			= (char**)			malloc( out3DMCCount * sizeof( char* ) );

		outFAMCSize		=    (int*)			malloc( outFAMCCount * sizeof( int ) );
		outFAMC			= (char**)			malloc( outFAMCCount * sizeof( char* ) );

		outBBASize =		(int*)			malloc ( outBBACount * sizeof( int) );
		outBBA =			(char**)	    malloc ( outBBACount * sizeof( char* ) );
		
		ODs = new int[trc+1];
		texIDs = new int [trc+1]; 
		dmcIDs = new int [trc+1];
		BBAIDs = new int [trc+1];
		famcIDs = new int [trc+1];
		ODs[0]=trc;
		texIDs[0]=trc;
		dmcIDs[0]=trc;
		BBAIDs[0]=trc;
		famcIDs[0]=trc;
		for (int c = 1; c < trc+1; c++)
		{
			ODs   [c] = 0;
			texIDs[c] = -1;
			dmcIDs[c] = -1;
			BBAIDs[c] = -1;
			famcIDs[c]= -1;
		
		}
		

		sample = gf_isom_get_sample(m_mov, ODtrack, 1, &StreamDescriptionIndex);
		
		//fill TRACK <-> OD mapping
		if (sample)
		{
			int texID = 0;
			int dmcID = 0;
			int BBAID = 0;
			int famcID = 0;
			GF_ODCodec* codec = gf_odf_codec_new();			
			GF_Err e = gf_odf_codec_set_au( codec, sample->data, sample->dataLength);
			e = gf_odf_codec_decode(codec);
			GF_ODCom *command = gf_odf_codec_get_com(codec);
			GF_ODUpdate* odUpdate = (GF_ODUpdate*)command;
			for(unsigned int i = 0; i < gf_list_count(odUpdate->objectDescriptors); i++) 
			{
				GF_ObjectDescriptor* od = (GF_ObjectDescriptor*)gf_list_get(odUpdate->objectDescriptors, i);
				GF_ESD* pOdESD = (GF_ESD*)gf_list_get(od->ESDescriptors,0);
				int trNo = gf_isom_get_track_by_id(m_mov,pOdESD->ESID);
				GF_ESD* pESD = gf_isom_get_esd(m_mov, trNo,1);				
				ODs[trNo]=od->objectDescriptorID;

				u32 streamtp = gf_isom_get_media_type(m_mov, trNo);
				switch ( streamtp)
				//switch (pESD->decoderConfig->streamType)
				{
				//case GF_ISOM_PL_VISUAL:
				case GF_ISOM_MEDIA_VISUAL:
						switch (pESD->decoderConfig->objectTypeIndication)
						{
							case 108:
							{
								
								GF_ISOSample* texSmpl = gf_isom_get_sample(m_mov, trNo, 1, &StreamDescriptionIndex);
								outTexturesType[texID]	= TT_JPEG;
								outTexturesSize[texID]	= texSmpl->dataLength;
								outTextures[texID]		= (char*) malloc( outTexturesSize[texID] );
								memcpy( outTextures[texID], texSmpl->data, outTexturesSize[texID] );
								gf_isom_sample_del( &texSmpl );
								texIDs[trNo] = texID++;
								break;
							}
							case 110:
							{
								
								GF_ISOSample* texSmpl = gf_isom_get_sample(m_mov, trNo, 1, &StreamDescriptionIndex);
								outTexturesType[texID]	= TT_JPEG2000;
								outTexturesSize[texID]	= texSmpl->dataLength;
								outTextures[texID]		= (char*) malloc( outTexturesSize[texID] );
								memcpy( outTextures[texID], texSmpl->data, outTexturesSize[texID] );
								gf_isom_sample_del(&texSmpl);
								texIDs[trNo] = texID++;
								break;
							}
							case 33:
								{
								GF_ISOSample* RARSmpl = gf_isom_get_sample(m_mov, trNo, 1, &StreamDescriptionIndex);
								outRARSize	= RARSmpl->dataLength;
								outRAR		= (char*) malloc( outRARSize );
								memcpy( outRAR, RARSmpl->data, outRARSize );
								gf_isom_sample_del( &RARSmpl );
								break;
								}

							case 100:
								{
									/*GF_ISOSample* famcSmpl = gf_isom_get_sample(m_mov, trNo, 1, &StreamDescriptionIndex);
									outFAMCSize[famcID]	= famcSmpl->dataLength;
									outFAMC[famcID]		= (char*) malloc( outFAMCSize[famcID] );
									memcpy( outFAMC[famcID], famcSmpl->data, outFAMCSize[famcID] );
									gf_isom_sample_del( &famcSmpl );
									famcIDs[trNo] = famcID++;
									break;*/
									GF_ISOSample* famcSmpl;
									for (int sn=1; sn<=gf_isom_get_sample_count(m_mov,trNo); sn++)
									{
										famcSmpl = gf_isom_get_sample(m_mov, trNo, sn, &StreamDescriptionIndex);
										int tmp = 0;
										if (sn==1)
										{
											outFAMCSize[famcID]	= famcSmpl->dataLength;
											outFAMC[famcID]		= (char*) malloc( outFAMCSize[famcID] );
											memcpy( outFAMC[famcID], famcSmpl->data, outFAMCSize[famcID] );
										}
										else
										{
											tmp = outFAMCSize[famcID];
											outFAMCSize[famcID]	+= famcSmpl->dataLength;
											outFAMC[famcID]		= (char*) realloc(outFAMC[famcID],outFAMCSize[famcID]);
											memcpy(outFAMC[famcID]+tmp,famcSmpl->data,famcSmpl->dataLength);
										}
									
										gf_isom_sample_del( &famcSmpl );
									}
									
									famcIDs[trNo] = famcID++;
									break;
									}

							case 55:
							{
								GF_ISOSample* dmcSmpl = gf_isom_get_sample(m_mov, trNo, 1, &StreamDescriptionIndex);
								out3DMCSize[dmcID]	= dmcSmpl->dataLength;
								out3DMC[dmcID]		= (char*) malloc( out3DMCSize[dmcID] );
								memcpy( out3DMC[dmcID], dmcSmpl->data, out3DMCSize[dmcID] );
								gf_isom_sample_del( &dmcSmpl );
								dmcIDs[trNo] = dmcID++;
								break;
							
							}
							
						}
						break;
				case GF_ISOM_MEDIA_SCENE:
						switch (pESD->decoderConfig->objectTypeIndication)
						{
							case 5:
							{
								GF_ISOSample* BBASmpl;
								for (int sn=1; sn<=gf_isom_get_sample_count(m_mov,trNo); sn++)
								{
									BBASmpl = gf_isom_get_sample(m_mov, trNo, sn, &StreamDescriptionIndex);
									int tmp = 0;
									if (sn==1)
									{
										outBBASize[BBAID]	= BBASmpl->dataLength;
										outBBA[BBAID]		= (char*) malloc( outBBASize[BBAID] );
										memcpy( outBBA[BBAID], BBASmpl->data, outBBASize[BBAID] );
									}
									else
									{
										tmp = outBBASize[BBAID];
										outBBASize[BBAID]	+= BBASmpl->dataLength;
										outBBA[BBAID]		= (char*) realloc(outBBA[BBAID],outBBASize[BBAID]);
										memcpy(outBBA[BBAID]+tmp,BBASmpl->data,BBASmpl->dataLength);
									}
								
									gf_isom_sample_del( &BBASmpl );
								}
								
								BBAIDs[trNo] = BBAID++;
								break;
							
							}
						}
						break;
				}
				

				gf_odf_desc_del((GF_Descriptor*)pESD);
			}
			gf_odf_com_del(&command);
			gf_odf_codec_del(codec);
			gf_isom_sample_del(&sample);
		}
		else
		{
			gf_isom_close( m_mov );
			m_mov = NULL;
			return -5;
		}
	}
	
	//if (type < 0) 
		//return 0;
	if ( !BIFStrack ) 
		return 0;
	sample = gf_isom_get_sample(m_mov, BIFStrack, 1, &StreamDescriptionIndex);

	GF_ESD* pESD;
	GF_TrackBox *trak;
	trak = gf_isom_get_track(m_mov->moov, BIFStrack);
	Media_GetESD(trak->Media, 1, &pESD, true);
	
	GF_BifsDecoder* codec;

	m_scene = gf_sg_new();
	codec = gf_bifs_decoder_new(m_scene,false);
	GF_Err e;
	e = gf_bifs_decoder_configure_stream (codec,201,(char*)pESD->decoderConfig->decoderSpecificInfo->data, pESD->decoderConfig->decoderSpecificInfo->dataLength, 2);	
	//codec->info = BD_GetStream(codec, 201);
	//codec->info->config.IsCommandStream = 1;
	//codec->info = NULL;

	//BIFS Decoding
	e = gf_bifs_decode_au(codec, 201, (char*)sample->data, sample->dataLength, 0);
	outBIFSSize = sample->dataLength;
	outBIFS = (char*) malloc( sizeof (char) * outBIFSSize);
	memcpy(outBIFS, sample->data, outBIFSSize );


	gf_isom_sample_del(&sample);

	gf_bifs_decoder_del(codec);

	if (e)
	{
		gf_sg_del( m_scene );
		m_scene = NULL;
		gf_isom_close( m_mov );
		m_mov = NULL;
		return -4;
	}

	if ( type == OUT_VB )
	{
		BIFStoVBLib bifsvb;
		bifsvb.CreateVB( m_scene, false );

		BIFS::ostream st;

		st.write( 0, 4 );									// total size of the stream

		unsigned int i;
		int vbCount = 0;
		for (i = 0; i < bifsvb.GetVBCount(); i++)			// Count the number of usefful vertex buffers
		{
			VertexBuffer* vb;
			bifsvb.GetVB(i,&vb);
			if ( vb->type == VBTYPE_SKIN || vb->type == VBTYPE_RIGD )
				vbCount++;
		}


		st.write( vbCount, 1 );								// number of vertex bufer
		
		int ibIndex = 0;
		for (i = 0; i < bifsvb.GetVBCount(); i++)
		{
			VertexBuffer* vb;
			bifsvb.GetVB(i,&vb);

			if ( vb->type != VBTYPE_SKIN && vb->type != VBTYPE_RIGD )
				continue;									// skip the bone skin

			int vPos = st.tellp();
			st.write( 0, 4 );								// total size of vertexbuffer and index buffers

			int vbPos = st.tellp();

			st.write( 0, 4 );								// size of the vertex buffer only

			//building the descriptor
			int desc = 0;
			//if ( vb->hasTexCoord() )
			//	desc += 1;
			//if ( vb->hasNormal() )
			//	desc += 2;
			if ( vb->hasBoneInfluences() )
				desc += 1;
			st.write( desc, 1 );							// description of the vertex buffer structure
			st.write( vb->VBSize, 4 );						// number of vertices in the vertex buffer

			//write the vertex buffer
			unsigned int j;
			for ( j = 0; j < vb->VBSize; j++ )
			{
				st.write( vb->VB[j].x );
				st.write( vb->VB[j].y );
				st.write( vb->VB[j].z );
				if ( vb->hasBoneInfluences() )
				{
					st.write( vb->VB[j].w[0] );
					st.write( vb->VB[j].w[1] );
					st.write( vb->VB[j].w[2] );
					st.write( vb->VB[j].w[3] );
					st.write( vb->VB[j].iw, 4 );			// TODO find how the int is written, and how directx wants it
				}
			}			
			if ( vb->hasNormal() )
			{
				st.write( vb->VBSize, 4 );						// Number of normals in the VB
				for ( j = 0; j < vb->VBSize; j++ )
				{
					st.write( vb->VB[j].nx );
					st.write( vb->VB[j].ny );
					st.write( vb->VB[j].nz );
				}
			}
			else
				st.write( 0, 4 );								// Number of normals in the VB
			if ( vb->hasTexCoord() )
			{
				st.write( vb->VBSize, 4 );						// Number of texCoords in the VB
				for ( j = 0; j < vb->VBSize; j++ )
				{
					st.write( vb->VB[j].u );
					st.write( vb->VB[j].v );
				}
			}
			else
				st.write( 0, 4 );								// Number of texCoords in the VB

			int t = st.tellp();
			int vbSize = t - vbPos;
			st.seekp( vbPos );
			st.write( vbSize, 4 );
			st.seekp( t );

			st.write( vb->pc, 1 );							// number of index buffers

			for ( j = 0; j < vb->pc; j++ )
			{
				int ibPos = st.tellp();

				st.write( 0, 4 );							// size of the index buffer

				//int textureID = -1;
				//if ( vb->pShape[j] )
				//{
				//	if ( vb->pShape[j]->appearance )
				//	{
				//		M_Appearance* pApr = (M_Appearance*) vb->pShape[j]->appearance;
				//		if ( pApr->texture )
				//		{
				//			M_ImageTexture* pTex = (M_ImageTexture*) pApr->texture;
				//			if ( pTex->url.count )
				//			{
				//				int od;
				//				od = pTex->url.vals[0].OD_ID;
				//				int tr = M4_GetTrackByOD( ODs, od );
				//				if ( tr )
				//				{
				//					textureID = texIDs[tr];
				//				}
				//			}
				//		}
				//	}
				//}
				st.write( ibIndex++, 4 );							// id of the appearance for that index buffer

				st.write( vb->indexBuffCount[j] * 3 / 4, 4 );		// number of indices
				unsigned int k;
				for ( k = 0; k < vb->indexBuffCount[j]; )
				{
					st.write( vb->indexBuff[j][k++], 4 );
					st.write( vb->indexBuff[j][k++], 4 );
					st.write( vb->indexBuff[j][k++], 4 );
					k++;
				}


				//st.write( 0, 4 );									// TexCoord Index

				//st.write( 0, 4 );									// Normal Index

				t = st.tellp();
				int ibSize = t - ibPos;
				st.seekp( ibPos );
				st.write( ibSize, 4 );
				st.seekp( t );
			}

			t = st.tellp();
			int vSize = t - vPos;
			st.seekp( vPos );
			st.write( vSize, 4 );
			st.seekp( t );
		}

		int size = st.tellp();
		st.seekp( 0 );
		st.write( size, 4 );
		st.seekp( size );

		st.GetData( &outBuffer, outBufferSize );


		// Writing the appearance buffer

		BIFS::ostream stApp;
		stApp.write( 0, 4 );									// total size of the stream
		stApp.write( 0, 4 );									// Number of appearance atoms

		int appCount = 0;
		for (i = 0; i < bifsvb.GetVBCount(); i++)
		{
			VertexBuffer* vb;
			bifsvb.GetVB(i,&vb);
			
			if ( vb->type != VBTYPE_SKIN && vb->type != VBTYPE_RIGD )
				continue;
		
			unsigned int j;
			for ( j = 0; j < vb->pc; j++ )
			{
				if ( vb->pShape[j] )
				{
					appCount++;
					if ( !vb->pShape[j]->appearance )
					{
						stApp.write( -1, 4 );
						stApp.write( 0, 1 );
					}
					else
					{
						M_Appearance* pApr = (M_Appearance*) vb->pShape[j]->appearance;
						int textureID = -1;
						if ( pApr->texture )
						{
							if ( pApr->texture->sgprivate->tag == TAG_MPEG4_ImageTexture )
							{
								M_ImageTexture* pTex = (M_ImageTexture*) pApr->texture;
								if ( pTex->url.count )
								{
									int od;
									od = pTex->url.vals[0].OD_ID;
									int tr = M4_GetTrackByOD( ODs, od );
									if ( tr )
									{
										textureID = texIDs[tr];
									}
								}
							}
						}
						stApp.write( textureID, 4 );

						if ( !pApr->material )
							stApp.write( 0, 1 );										// No material
						else
						{
							if ( pApr->material->sgprivate->tag != TAG_MPEG4_Material )
								stApp.write( 0, 1 );									// No Material
							else
							{
								stApp.write( 1, 1 );									// Material Exsists
								M_Material* pMat = (M_Material*) pApr->material;
								stApp.write( pMat->diffuseColor.red );
								stApp.write( pMat->diffuseColor.green );
								stApp.write( pMat->diffuseColor.blue );
								stApp.write( pMat->specularColor.red );
								stApp.write( pMat->specularColor.green );
								stApp.write( pMat->specularColor.blue );
								stApp.write( pMat->emissiveColor.red );
								stApp.write( pMat->emissiveColor.green );
								stApp.write( pMat->emissiveColor.blue );
								stApp.write( pMat->transparency );
								stApp.write( pMat->ambientIntensity );
								stApp.write( pMat->shininess );
							}
							
						}

					}
				}
			}
		}
		int appSize = stApp.tellp();
		stApp.seekp( 0 );
		stApp.write( appSize, 4 );
		stApp.write( appCount, 4 );
		stApp.seekp( appSize );

		stApp.GetData( &appBuffer, appBufferSize );
	}
	else
	if ( type == OUT_IFS )
	{

		int appID = 0;

		BIFS::ostream st;
		st.write( 0, 4 );			// Buffer Size
		st.write( 0, 1 );			// Number of VBs

		BIFS::ostream stApp;
		stApp.write( 0, 4 );									// total size of the stream
		stApp.write( 0, 4 );									// Number of appearance atoms

		int vbCount = 0;
		for ( int i = 0; i < gf_list_count( m_scene->all_nodes ); i++ )
		{
			GF_Node* pNode = (GF_Node*)gf_list_get( m_scene->all_nodes, i );
			if ( pNode->sgprivate->tag == TAG_MPEG4_Coordinate )
			{
				M_Coordinate* pCoord = (M_Coordinate*) pNode;
				M_TextureCoordinate* pTexCoord = NULL;
				M_Normal* pNormal = NULL;

				vbCount++;

				M_SBSkinnedModel* pSkin = NULL;
				int indCount = 0;
				for ( int j = 0; j < gf_node_get_parent_count( (GF_Node*) pCoord ); j++ )
				{
					pNode = (GF_Node*)gf_node_get_parent( (GF_Node*) pCoord, j );
					if ( pNode->sgprivate->tag == TAG_MPEG4_SBSkinnedModel )
						pSkin = (M_SBSkinnedModel*)pNode;
					else
					{
						M_IndexedFaceSet* pIfs = (M_IndexedFaceSet*)pNode;
						if ( pIfs->normal )
							pNormal = (M_Normal*)pIfs->normal;
						if ( pIfs->texCoord )
							pTexCoord = (M_TextureCoordinate*)pIfs->texCoord;
						indCount++;
					}
				}

				int vbPos = st.tellp();
				st.write( 0, 4 );											// Total size of one VB
				st.write( 0, 4 );											// Size only of Vertex data
				st.write( 0, 1 );											// Description

				st.write( pCoord->point.count, 4 );
				for ( int j = 0; j < pCoord->point.count; j++ )
				{
					st.write( pCoord->point.vals[j].x );
					st.write( pCoord->point.vals[j].y );
					st.write( pCoord->point.vals[j].z );
				}

				if ( !pNormal )
					st.write( 0, 4 );										// No normals
				else
				{
					st.write( pNormal->vector.count, 4 );					// Number of normals
					for ( int j = 0; j < pNormal->vector.count; j++ )
					{
						st.write( pNormal->vector.vals[j].x );
						st.write( pNormal->vector.vals[j].y );
						st.write( pNormal->vector.vals[j].z );
					}
				}

				// Dump texture coordinates
				if ( !pTexCoord )
					st.write( 0, 4 );										// No texture coordinates
				else
				{
					st.write( pTexCoord->point.count, 4 );					// Number of texture coordinates
					for ( int j = 0; j < pTexCoord->point.count; j++ )
					{
						st.write( pTexCoord->point.vals[j].x );
						st.write( pTexCoord->point.vals[j].y );
					}
				}

				st.write( indCount, 1 );									// Nuber of index buffers

				for ( int j = 0; j < gf_node_get_parent_count( (GF_Node*) pCoord ); j++ )
				{
					pNode = (GF_Node*)gf_node_get_parent( (GF_Node*) pCoord, j );
					if ( pNode->sgprivate->tag != TAG_MPEG4_IndexedFaceSet )
						continue;

					int ifsP;
					ifsP = st.tellp();

					st.write( 0, 4 );										// Size of one index buffer
					st.write( appID, 4 );									// Appearance ID
					appID++;

					M_IndexedFaceSet* pIfs = (M_IndexedFaceSet*)pNode;

					// Dump the coordinates index
					st.write( pIfs->coordIndex.count * 3 / 4, 4 );
					for ( int j = 0; j < pIfs->coordIndex.count; )
					{
						st.write( pIfs->coordIndex.vals[j++], 4 );
						st.write( pIfs->coordIndex.vals[j++], 4 );
						st.write( pIfs->coordIndex.vals[j++], 4 );
						j++;
					}

					// Dump the texture coordinates index
					st.write( pIfs->texCoordIndex.count * 3 / 4, 4 );
					for ( int j = 0; j < pIfs->texCoordIndex.count; )
					{
						st.write( pIfs->texCoordIndex.vals[j++], 4 );
						st.write( pIfs->texCoordIndex.vals[j++], 4 );
						st.write( pIfs->texCoordIndex.vals[j++], 4 );
						j++;
					}

					// Dump the normals index
					st.write( pIfs->normalIndex.count* 3 / 4, 4 );
					for ( int j = 0; j < pIfs->normalIndex.count ; )
					{
						st.write( pIfs->normalIndex.vals[j++], 4 );
						st.write( pIfs->normalIndex.vals[j++], 4 );
						st.write( pIfs->normalIndex.vals[j++], 4 );
						j++;
					}

					int endIfs = st.tellp();
					int size = endIfs - ifsP;
					st.seekp( ifsP );
					st.write( size, 4 );
					st.seekp( endIfs );

					M_Shape* pShp = (M_Shape*)gf_node_get_parent( (GF_Node*) pIfs, 0 );

					if	( !pShp->appearance )
					{
						stApp.write( -1, 4 );
						stApp.write( 0, 1 );
						continue;
					}

					pNode = pShp->appearance;

					if ( pNode->sgprivate->tag != TAG_MPEG4_Appearance )
					{
						stApp.write( -1, 4 );
						stApp.write( 0, 1 );
						continue;
					}

					M_Appearance* pApr = (M_Appearance*)pNode;

					int textureID = -1;
					if ( pApr->texture )
					{
						M_ImageTexture* pTex = (M_ImageTexture*) pApr->texture;
						if ( pTex->url.count )
						{
							int od;
							od = pTex->url.vals[0].OD_ID;
							int tr = M4_GetTrackByOD( ODs, od );
							if ( tr )
							{
								textureID = texIDs[tr];
							}
						}
					}
					stApp.write( textureID, 4 );

					if ( !pApr->material )
						stApp.write( 0, 1 );							// No material
					else
					{
						stApp.write( 1, 1 );							// Material Exsists
						M_Material* pMat = (M_Material*) pApr->material;
						stApp.write( pMat->diffuseColor.red );
						stApp.write( pMat->diffuseColor.green );
						stApp.write( pMat->diffuseColor.blue );
						stApp.write( pMat->specularColor.red );
						stApp.write( pMat->specularColor.green );
						stApp.write( pMat->specularColor.blue );
						stApp.write( pMat->emissiveColor.red );
						stApp.write( pMat->emissiveColor.green );
						stApp.write( pMat->emissiveColor.blue );
						stApp.write( pMat->transparency );
						stApp.write( pMat->ambientIntensity );
						stApp.write( pMat->shininess );
					}
				}
				int vbPos2 = st.tellp();
				st.seekp( vbPos );
				st.write( vbPos2 - vbPos, 4 );
				st.seekp( vbPos2 );
			}
		}
		int vbSize = st.tellp();
		st.seekp( 0 );
		st.write( vbSize, 4 );
		st.write( vbCount, 4 );
		st.seekp( vbSize );

		st.GetData( &outBuffer, outBufferSize );

		int appSize = stApp.tellp();
		stApp.seekp( 0 );
		stApp.write( appSize, 4 );
		stApp.write( vbCount, 4 );
		stApp.seekp( appSize );

		stApp.GetData( &appBuffer, appBufferSize );
	}

	delete[] ODs;
	delete[] texIDs;

	trJpeg.clear();
	trJpeg2000.clear();
	trBba.clear();
	trVideo.clear();
	trAudio.clear();

	return 0;
}

//int MP4toVBLib::DecodeJPEG2000(int texID)
//{
//	jas_stream_t *in, *out;
//
//	if (jas_init()) {
//		return -1;
//	}
//
//	in = jas_stream_memopen( (char*)(outTextures[texID]), outTexturesSize[texID] );
//	
//	jas_image_t *image;
//
//	int infmt =jas_image_getfmt(in);
//	char *inopts = NULL;
//	image = jas_image_decode(in, infmt, inopts);
//	jas_stream_close(in);
//
//	int numcmpts = jas_image_numcmpts(image);
//
//	{
//		jas_image_t *newimage;
//		jas_cmprof_t *outprof;
//		outprof = jas_cmprof_createfromclrspc(JAS_CLRSPC_SRGB);
//		newimage = jas_image_chclrspc(image, outprof, JAS_CMXFORM_INTENT_PER);
//		jas_image_destroy(image);
//		jas_cmprof_destroy(outprof);
//		image = newimage;
//	}
//
//	out = jas_stream_memopen( NULL, 0 );
//	
//	int outfmt = jas_image_strtofmt( "bmp" );
//	char outopts = '\0';
//	jas_image_encode( image, out, outfmt, &outopts );
//	jas_stream_flush(out);
//
//	free( outTextures[texID] );
//	outTexturesSize[texID] = jas_stream_length( out );
//	outTextures[texID] = ( char* ) malloc( outTexturesSize[texID] );
//	jas_stream_rewind( out );
//	jas_stream_read( out, outTextures[texID], outTexturesSize[texID] );
//
//	jas_stream_close(out);
//	
//	jas_image_destroy(image);
//	jas_cleanup();
//	outTexturesType[texID] = TT_BMP;
//	return 0;
//}

MP4toVBLib *vbLib = NULL;

int MP4toVBLib_Init()
{
	delete vbLib;
	vbLib = new MP4toVBLib();
	return 0;
}

int MP4toVBLib_Free()
{
	delete vbLib;
	vbLib = NULL;
	return 0;
}

int MP4toVBLib_LoadMP4File( const char * FileName, outputType type )
{
	if ( !vbLib )
		return -1;
	return vbLib->LoadMP4File( FileName, type );
}

int MP4toVBLib_GetVBBufferSize()
{
	if ( !vbLib )
		return -1;
	return vbLib->GetVBBufferSize();
}

int MP4toVBLib_GetBIFSSize()
{
	if ( !vbLib )
		return -1;
	return vbLib->GetBIFSSize();
}

int MP4toVBLib_GetBIFS( unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetBIFS( buff, size, alloc );
}

int MP4toVBLib_GetVB( unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetVB( buff, size, alloc );
}

int MP4toVBLib_GetAppBufferSize()
{
	if ( !vbLib )
		return -1;
	return vbLib->GetAppBufferSize();
}

int MP4toVBLib_GetAppBuffer( unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetAppBuffer( buff, size, alloc );
}

int MP4toVBLib_GetTextureCount()
{
	if ( !vbLib )
		return -1;
	return vbLib->GetTextureCount();
}

int MP4toVBLib_GetTextureSize( unsigned int texID )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetTextureSize( texID );
}

int MP4toVBLib_GetTexture( unsigned int texID, unsigned char* &buff, int &size, textureType &type, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetTexture( texID, buff, size, type, alloc );
}

int MP4toVBLib_DecompressTextures()
{
	if ( !vbLib )
		return -1;
	return vbLib->DecompressTextures();
}

int MP4toVBLib_GetBBACount()
{
	if ( !vbLib )
		return -1;
	return vbLib->GetBBACount();
}

int MP4toVBLib_GetBBASize( unsigned int ID )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetBBASize( ID );
}

int MP4toVBLib_GetBBA( unsigned int ID, unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetBBA( ID, buff, size,  alloc );
}

int MP4toVBLib_GetBBAbyRef( const char* metaName, unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetBBAbyRef( metaName, buff, size,  alloc );
}

int MP4toVBLib_Get3DMCCount()
{
	if ( !vbLib )
		return -1;
	return vbLib->Get3DMCCount();
}

int MP4toVBLib_Get3DMCSize( unsigned int ID )
{
	if ( !vbLib )
		return -1;
	return vbLib->Get3DMCSize( ID );
}

int MP4toVBLib_Get3DMC( unsigned int ID, unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->Get3DMC( ID, buff, size,  alloc );
}

int MP4toVBLib_Get3DMCbyMetaName( const char* metaName, unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->Get3DMCbyMetaName( metaName, buff, size,  alloc );
}

int MP4toVBLib_Get3DMCbyRef( const char* metaName, unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->Get3DMCbyRef( metaName, buff, size,  alloc );
}


int MP4toVBLib_GetFAMCCount()
{
	if ( !vbLib )
		return -1;
	return vbLib->GetFAMCCount();
}

int MP4toVBLib_GetFAMCSize( unsigned int ID )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetFAMCSize( ID );
}

int MP4toVBLib_GetFAMC( unsigned int ID, unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetFAMC( ID, buff, size,  alloc );
}

int MP4toVBLib_GetFAMCbyRef( const char* metaName, unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	//char * tmp = (char*)malloc(strlen(metaName) + 5 );
	//sprintf( tmp,"%sfamc",metaName );
	int rez = vbLib->GetFAMCbyRef( metaName, buff, size,  alloc );
	//free( tmp );
	return rez;
}




int MP4toVBLib_GetTexturebyRef( const char* metaName, unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetTexturebyRef( metaName, buff, size,  alloc );
}



int MP4toVBLib_GetRAR(  unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetRAR(buff, size,  alloc );
}

int MP4toVBLib_GetRARfromMETA(  unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetRARfromMETA(buff, size,  alloc );
}

int MP4toVBLib_GetXMLfromMETA(  unsigned char* &buff, int &size, bool alloc )
{
	if ( !vbLib )
		return -1;
	return vbLib->GetXMLfromMETA(buff, size,  alloc );
}




