/******************************************************************************\
* Includes
\******************************************************************************/
#include "ColladaParser.h"
#include "Euler.h"
#define _USE_MATH_DEFINES 
#include <math.h> 

/******************************************************************************\
 ColladaParser.cpp: implementation of the ColladaParser class.
\******************************************************************************/


/******************************************************************************\
* Class ColladaParser
\******************************************************************************/

//constructor
ColladaParser::ColladaParser()
{
	m_pColladaFileName		= NULL;
	m_pAnimation			= NULL;
	m_p3dmc					= NULL;
	m_pRestColladaFileName	= NULL;
	m_3dmcCount				= 0;
	m_AnimationCount		= 0;
	m_pImageID				= 0;
	m_pImageName			= 0;
	m_ImageCount			= 0;
	m_BBACount				= 0;
	//ListMesh				= NULL;
	
}

//destructor 
ColladaParser::~ColladaParser()
{
	/*if ( m_pColladaFileName )
		free(m_pColladaFileName);*/
	//remove ( m_pRestColladaFileName );
	if ( m_pRestColladaFileName )
		free(m_pRestColladaFileName);
	for ( unsigned int i = 0; i < m_3dmcCount; i++ )
		delete m_p3dmc[i];
	delete[] m_p3dmc;
	for ( unsigned int i = 0; i < m_AnimationCount; i++ )
		delete m_pAnimation[i];
	delete[] m_pAnimation;
	delete[] m_pImageID;
	delete[] m_pImageName;

}


bool ColladaParser::MakeCopyColladaFile()
{
	
	m_pRestColladaFileName = (char*) malloc ( strlen(m_pColladaFileName)+8);
	sprintf(m_pRestColladaFileName,"%stmp.dae",m_pColladaFileName);
	
	FILE* f = fopen ( m_pColladaFileName, "rb" );
	if ( !f )
		return false;
	fseek(f,0, SEEK_END);
	int size = ftell(f);
	fseek(f,0,0);
	char* buff = (char*) malloc ( size );
	fread( buff, 1, size, f );
	fclose ( f );
	f = fopen ( m_pRestColladaFileName, "wb");
	if (!f)
		return false;
	
	fwrite( buff, 1, size, f );
	fclose ( f );
	
	if ( buff )
		free ( buff );

	return true;
}

bool ColladaParser::ParseColladaFile( bool isFAMC  )
{
	MakeCopyColladaFile();
	FCollada::Initialize();
	printf( "Loading file\n" );
	file = FCollada::LoadDocument( (const fchar*) m_pRestColladaFileName );
	if ( !file )
		return false;
	//RemoveGeometry();
	printf( "Removing animation\n" );
	RemoveAnimation( isFAMC );
	//these are files for testig size of animation in collada
	//no need of them for end-user version
	/*char* OnlyNOANIM = (char*) malloc ( strlen(m_pColladaFileName)+8);
	sprintf(OnlyNOANIM,"%sANM.dae",m_pColladaFileName);
	FCollada::SaveDocument( file, (const fchar*) OnlyNOANIM );
	printf( "Removing geometry\n" );*/
	RemoveGeometry();
	//these are files for testig size of geometry in collada
	//no need of them for end-user version
	/*char* OnlyNOANIMANDGEOM = (char*) malloc ( strlen(m_pColladaFileName)+8);
	sprintf((char*)OnlyNOANIMANDGEOM,"%sGEO.dae",m_pColladaFileName);
	FCollada::SaveDocument( file, (const fchar*) OnlyNOANIMANDGEOM );*/
	
	ParseImages();
	printf( "Saving rest file\n" );
	FCollada::SaveDocument( file, (const fchar*) m_pRestColladaFileName );
	delete file;
	FCollada::Release();
	return true;
}

bool ColladaParser::ParseImages()
{
	FCDImageLibrary *imlib = NULL;
	FCDImage		*im	   = NULL;
	
	imlib = file->GetImageLibrary();
	m_ImageCount = imlib->GetEntityCount();

	m_pImageID = new fstring[ m_ImageCount ];

	m_pImageName = new fstring[ m_ImageCount ];
		
	for ( int i=0; i<m_ImageCount; i++)
	{
		im = imlib->GetEntity( i );
		m_pImageID[i] = im->GetDaeId();
		m_pImageName[i] = im->GetFilename();
	}

	return true;
}

char* ColladaParser::GetImageName( char * IDName )
{	
	for ( int i=0; i<m_ImageCount; i++)
	{
		if (m_pImageID[i] == IDName) 
			return (char*)m_pImageName[i].c_str();
	}
	return NULL;

}


bool ColladaParser::RemoveGeometry( )
{
	FCDGeometryLibrary	*gmlib	= NULL;
	FCDGeometry			*ge		= NULL;
	FCDGeometryMesh		*gm     = NULL;
	
	char		* buff = NULL;
	int			buf_size = 0;

	gmlib = file->GetGeometryLibrary();

	int cnt = 0;
	for ( int i=0; i<gmlib->GetEntityCount(); i++)
	{
		ge = gmlib->GetEntity( i );
		gm =  ge->GetMesh();
		if ( gm )
			cnt++;
	}

	m_p3dmc = new MP43DMC*[cnt];

	for ( int i=0; i<gmlib->GetEntityCount(); i++)
	{
		ge = gmlib->GetEntity( i );
		gm =  ge->GetMesh();
		if ( gm )
		{
			bool loadNormalIndex	= true;
			bool loadTexindex		= true;
			FCDGeometrySource	*gmsC	= NULL;
			FCDGeometrySource	*gmsT	= NULL;
			FCDGeometrySource	*gmsN	= NULL;
			FCDGeometrySource	*gmsRGB	= NULL;
			FCDGeometryPolygons *gmp	= NULL;
			FloatList	verticesC;
			FloatList	verticesT;
			FloatList	verticesN;
			FloatList	verticesRGB;
			FCDGeometryPolygonsInput *indC;
			FCDGeometryPolygonsInput *indT;
			FCDGeometryPolygonsInput *indN;
			FCDGeometryPolygonsInput *indRGB;


			int strideN = 3;
			int strideT = 3;
			int strideC = 3;
			int strideRGB = 3;

			int indexN = 0;
			int indexT = 0;
			int indexC = 0;
			int indexRGB = 0;

			// Load coordinates
		 
			gmsC = gm->FindSourceByType(FUDaeGeometryInput::POSITION);
			if ( gmsC )
			{
				strideC = gmsC->GetStride();
				verticesC = gmsC->GetSourceData().GetDataList();
			}
			
			
			// Load normals
			gmsN = gm->FindSourceByType(FUDaeGeometryInput::NORMAL);
			if ( gmsN )
			{
				strideN = gmsN->GetStride();
				verticesN = gmsN->GetSourceData().GetDataList();
			}
			
			// Load textcoords
			gmsT = gm->FindSourceByType(FUDaeGeometryInput::TEXCOORD);
			if ( gmsT ) 
			{
				strideT = gmsT->GetStride();
				verticesT = gmsT->GetSourceData().GetDataList();
			}

			// Load colors
			gmsRGB = gm->FindSourceByType(FUDaeGeometryInput::COLOR);
			if ( gmsRGB )
			{
				strideRGB = gmsRGB->GetStride();
				verticesRGB = gmsRGB->GetSourceData().GetDataList();
			}
			
			//Indexes
			for (int tmp = 0; tmp < gm->GetPolygonsCount(); tmp++ )
			{
				gmp = gm->GetPolygons ( tmp );
				if (gmp)
				{
					if ( gmsC )
						indC = gmp->FindInput(FUDaeGeometryInput::POSITION);
					if ( gmsN ) 
						indN = gmp->FindInput(FUDaeGeometryInput::NORMAL);
					if ( gmsT ) 
						indT = gmp->FindInput(FUDaeGeometryInput::TEXCOORD);
					if ( gmsRGB ) 
						indRGB = gmp->FindInput(FUDaeGeometryInput::COLOR);

					// Get number of all coordinates index
					if ( gmsC )
						indexC+=indC->GetIndexCount();
					// Get number of all texture coordinates index
					if ( gmsT )
						indexT+=indT->GetIndexCount(); 
					// Get number of all normals index
					if ( gmsN )
						indexN+=indN->GetIndexCount(); 
					// Get number of all colors index
					if ( gmsRGB )
						indexRGB+=indRGB->GetIndexCount(); 
				}
			}

			//Create 3DMC object and fill it
			m_3dmcCount ++;
			m_p3dmc[i] = new MP43DMC(verticesC.size()/strideC, verticesN.size()/strideN,verticesT.size()/strideT,verticesRGB.size()/strideRGB,indexC/3,indexN/3, indexT/3, indexRGB/3, i ,  gm->GetDaeId().c_str() );

			// Add coordinates
			for( int j = 0; j < verticesC.size(); j+=strideC )
			{
				m_p3dmc[i]->AddCoordinate( verticesC[j + 0], verticesC[j + 1], verticesC[j + 2]  );
			}
			// remove coordinates from collada file
			verticesC.clear();
			gmsC->SetData( verticesC, 1, 0 , 0);

			// Add normals
			for( int j = 0; j < verticesN.size(); j+=strideN )
			{
				m_p3dmc[i]->AddNormal( verticesN[j + 0], verticesN[j + 1], verticesN[j + 2] );
			}
			
			// remove normals from collada file
			int t = verticesN.size();
			if ( t )
			{
				
				verticesN.clear();
				gmsN->SetData( verticesN, 1, 0 , 0);
			}

			// Add texture coordinates
			for( int j = 0; j < verticesT.size(); j += strideT )
			{
				float tmpu, tmpv;
				tmpu = verticesT[j+0];
				
				if (tmpu<0)
					tmpu = tmpu + (int)tmpu+1;
				if (tmpu>=0)
					tmpu = tmpu - (int)tmpu + 0.000001;
				
				tmpv = verticesT[j+1];
				if (tmpv<0)
					tmpv = tmpv + (int)tmpv+1;
				if (tmpv>0)
					tmpv = tmpv - (int)tmpv + 0.000001;
				
				m_p3dmc[i]->AddTextureCoordinate( tmpu, tmpv );
			}
			
			// remove texture coordinates from collada file
			t = verticesT.size();
			if ( t )
			{
				verticesT.clear();
				gmsT->SetData( verticesT, 1, 0 , 0);
			}

			//Add colors
			for( int j = 0; j < verticesRGB.size(); j+=strideRGB )
			{
				m_p3dmc[i]->AddColor( verticesRGB[j + 0], verticesRGB[j + 1], verticesRGB[j + 2] );
			}

			// remove texture coordinates from collada file
			t = verticesRGB.size();
			if ( t )
			{
				verticesRGB.clear();
				gmsRGB->SetData( verticesRGB, 1, 0 , 0);
			}
			
			//Indexes
			for (int tmp = 0; tmp < gm->GetPolygonsCount(); tmp++ )
			{
				gmp = gm->GetPolygons ( tmp );
				
				if (gmp)
				{
					if ( gmsC )
						indC = gmp->FindInput(FUDaeGeometryInput::POSITION);
					if ( gmsN ) 
						indN = gmp->FindInput(FUDaeGeometryInput::NORMAL);
					if ( gmsT ) 
						indT = gmp->FindInput(FUDaeGeometryInput::TEXCOORD);
				
					if ( gmsRGB ) 
						indRGB = gmp->FindInput(FUDaeGeometryInput::COLOR);

					// Load the coordinates index
					if ( gmsC )
						for (int t=0;t<indC->GetIndexCount(); t+=3)
							m_p3dmc[i]->AddCoordinateIndex( indC->GetIndices()[t], indC->GetIndices()[t+1],indC->GetIndices()[t+2]);
							
						
					// Load the texture coordinates index
					if ( gmsT )
						for (int t=0;t<indT->GetIndexCount();t+=3 )
						{
							m_p3dmc[i]->AddTextureCoordinateIndex( indT->GetIndices()[t],indT->GetIndices()[t+1],indT->GetIndices()[t+2]);
							
						}

					// Load the normals index
					if ( gmsN )
						for (int t=0;t<indN->GetIndexCount();t+=3 )
						{
							m_p3dmc[i]->AddNormalIndex( indN->GetIndices()[t], indN->GetIndices()[t+1], indN->GetIndices()[t+2] );
						}

					// Load the colors index
					if ( gmsRGB )
						for (int t=0;t<indRGB->GetIndexCount();t+=3 )
						{
							m_p3dmc[i]->AddColorIndex( indRGB->GetIndices()[t], indRGB->GetIndices()[t+1], indRGB->GetIndices()[t+2] );
						}
			
					int textureNo = 0;
					for (int temp=0; temp< gmp->GetInputCount(); temp++)
					{
						if ( gmp->GetInput( temp )->GetSemantic() == FUDaeGeometryInput::TEXCOORD)
							textureNo++ ;
					}
					indC = gmp->FindInput(FUDaeGeometryInput::POSITION);
					indN = gmp->FindInput(FUDaeGeometryInput::NORMAL);
					indT = gmp->FindInput(FUDaeGeometryInput::TEXCOORD);
					indRGB = gmp->FindInput(FUDaeGeometryInput::COLOR);
						
					if ( textureNo == 1 )
					{
						// remove indices from collada file
						int tf = gmp->GetFaceCount();
						for ( int l = 0; l < tf; l++ )
							gmp->RemoveFace(0);

						// add data to determine  each mesh
						gmp->AddFace(3);
						for ( int i = 0; i < 3; i++ )
						{
							indC->GetIndices()[i] = 0;
							if ( indN )
								indN->GetIndices()[i] = 0;
							if (indT)
								indT->GetIndices()[i] = 0;
							if (indRGB)
								indRGB->GetIndices()[i] = 0;
						}
						indC->GetIndices()[0] = tf;
					}
					else
					{
						int tf = indC->GetIndexCount();
						for ( int l = 0; l < tf; l++ )
						{
							indC->GetIndices()[i] = 0;
							if ( indN )
								indN->GetIndices()[i] = 0;
							if (indT)
								indT->GetIndices()[i] = 0;
							if (indRGB)
								indRGB->GetIndices()[i] = 0;
						}
						indC->GetIndices()[0] = gmp->GetFaceCount();
					}
				}
			}
					
			//add extra node
			FCDExtra * fe = NULL;
			FCDEType *ty = NULL;
			FCDETechnique *th = NULL;
			FCDENode *tn= NULL;
			fe = ge->GetExtra();
			ty = fe->AddType("");
			th = ty->AddTechnique("mp4");
			fstring daeid = ge->GetDaeId();
			daeid.append(".3dmc");
			th->AddParameter( "init_from" , daeid );
		}
	}
	return true;
}


int ColladaParser::Get3DMCStreamsCount()
{
	return m_3dmcCount;
}


MP43DMC* ColladaParser::Get3DMCStream( int id )
{
	if ((id<0)||(id>=m_3dmcCount))
		return NULL;
	return m_p3dmc[id];
}




FMQuaternion ColladaParser::GetQuaternion( FMMatrix44& kRot )
{
	FMQuaternion q;
	
	float fTrace = kRot[0][0]+kRot[1][1]+kRot[2][2];
	float fRoot;

	if ( fTrace > 0.0 )
	{
		// |w| > 1/2, may as well choose w > 1/2
		fRoot = sqrt(fTrace + 1.0);  // 2w
		q.w = 0.5*fRoot;
		fRoot = 0.5/fRoot;  // 1/(4w)
		q.x = (kRot[2][1]-kRot[1][2])*fRoot;
		q.y = (kRot[0][2]-kRot[2][0])*fRoot;
		q.z = (kRot[1][0]-kRot[0][1])*fRoot;
	}
	else
	{
		// |w| <= 1/2
		static size_t s_iNext[3] = { 1, 2, 0 };
		size_t i = 0;
		if ( kRot[1][1] > kRot[0][0] )
			i = 1;
		if ( kRot[2][2] > kRot[i][i] )
			i = 2;
		size_t j = s_iNext[i];
		size_t k = s_iNext[j];

		fRoot = sqrt(kRot[i][i]-kRot[j][j]-kRot[k][k] + 1.0);
		float* apkQuat[3] = { &q.x, &q.y, &q.z };
		*apkQuat[i] = 0.5*fRoot;
		fRoot = 0.5/fRoot;
		q.w = (kRot[k][j]-kRot[j][k])*fRoot;
		*apkQuat[j] = (kRot[j][i]+kRot[i][j])*fRoot;
		*apkQuat[k] = (kRot[k][i]+kRot[i][k])*fRoot;
	}
	return q;
}




FCDTransform::Type ColladaParser::GetTransformTypeNameSub(fm::string &tp, fm::string &tq, FCDSceneNode* node) const  
{ 
	printf("%d(%s),", node->GetType(), node->GetDaeId().c_str()); 
	if(node->GetDaeId() == tp) { 
		for(int i=0; i<node->GetTransformCount(); i++) { 
			if(node->GetTransform(i)->GetSubId() == tq)  
				return node->GetTransform(i)->GetType(); 
		} 
		return FCDTransform::Type::TYPE_COUNT; 
	} 

	for(int i=0; i<node->GetChildrenCount(); i++) { 
		FCDTransform::Type ret = GetTransformTypeNameSub(tp, tq, node->GetChild(i)); 
		if(ret != FCDTransform::Type::TYPE_COUNT) return ret; 
	} 
	return FCDTransform::Type::TYPE_COUNT; 
} 

FCDTransform::Type ColladaParser::GetTransformTypeName(fm::string &tp, fm::string &tq) const  
{ 

	for(int i=0; i<file->GetVisualSceneLibrary()->GetEntityCount(); i++) { 
		for(int j=0; j<file->GetVisualSceneLibrary()->GetEntity(i)->GetChildrenCount(); j++) { 
			FCDTransform::Type ret = GetTransformTypeNameSub(tp, tq, file->GetVisualSceneLibrary()->GetEntity(i)->GetChild(j)); 
			if(ret != FCDTransform::Type::TYPE_COUNT) return ret; 
		} 
	} 
	for(int i=0; i<file->GetGeometryLibrary()->GetEntityCount(); i++) { 
		for(int j=0; j<file->GetGeometryLibrary()->GetEntity(i)->GetMesh()->GetSourceCount(); j++) { 

			if(file->GetGeometryLibrary()->GetEntity(i)->GetMesh()->GetSource(j)->GetDaeId() == tp) 
				return FCDTransform::Type::SKEW; 
		} 
	} 
	return FCDTransform::Type::TYPE_COUNT; 
} 

FCDTransform* ColladaParser::GetTransformNameSub(fm::string &tp, fm::string &tq, FCDSceneNode* node) const  
{ 
	//printf("%d(%s),", node->GetType(), node->GetDaeId().c_str()); 
	if(node->GetDaeId() == tp) { 
		for(int i=0; i<node->GetTransformCount(); i++) { 
			if(node->GetTransform(i)->GetSubId() == tq)  
				return node->GetTransform(i); 
		} 
		return NULL; 
	} 

	for(int i=0; i<node->GetChildrenCount(); i++) { 
		FCDTransform* ret = GetTransformNameSub(tp, tq, node->GetChild(i)); 
		if(ret != NULL) return ret; 
	} 
	return NULL; 
} 

FCDTransform* ColladaParser::GetTransformName(fm::string &tp, fm::string &tq) const  
{ 

	for(int i=0; i<file->GetVisualSceneLibrary()->GetEntityCount(); i++) { 
		for(int j=0; j<file->GetVisualSceneLibrary()->GetEntity(i)->GetChildrenCount(); j++) { 
			FCDTransform* ret = GetTransformNameSub(tp, tq, file->GetVisualSceneLibrary()->GetEntity(i)->GetChild(j)); 
			if(ret != NULL) return ret; 
		} 
	} 
	return NULL; 
} 



float ConvFunc( float v )
{
	return 0;
}



bool ColladaParser::RemoveAnimation( bool isFAMC  )

{

	FCDAnimationLibrary *anlib  = NULL;
	FCDAnimation		*an		= NULL;
	FCDAnimation		*anim	= NULL;
	FCDAnimationChannel *anchn	= NULL;
	FCDAnimationCurve	*ancur	= NULL;


	
	anlib = file->GetAnimationLibrary();
	if (! anlib || (anlib->GetEntityCount()==0))
		return false;

	fm::vector<FMMatrix44List> matList;

	bool useBBA = false; 
	bool useOI = false; 
	bool useCI = false; 

	
	int cnt = 0;

	m_pAnimation = new MP4AnimationBuff*[1];

	//for CI definitions
	fm::map<float, fm::map<int, float>> CIdata; 
					
					struct KeyInfo { 
						double key; 
						double x; 
						double y; 
						double z; 
						double keyValue; 
					}; 

	fm::vector<KeyInfo> OIKey; 

	//definitions

	m_pAnimation[0] = new MP4AnimationBuff( 0, anlib->GetEntity(anlib->GetEntityCount()-1)->GetDaeId().c_str() , cnt ); 
	m_AnimationCount++; 


	for ( int i=0; i<anlib->GetEntityCount(); i++)
	{		
		an = anlib->GetEntity ( i );
		{
			anim	= an ; 
			anchn	= anim->GetChannel(0);
		
			//get the target pointer and target qualifier from the extra node
			FCDExtra* extra = anim->GetExtra();
			FCDEType* type = extra->FindType("AnimTargets");
			FCDETechnique* tech = type->FindTechnique("TEMP");
			fm::string pointer = tech->GetChildNode(0)->GetContent();
			fm::string qualifier = tech->GetChildNode(1)->GetContent();

			//change
			//FCDAnimationChannelData& data = FArchiveXML::GetChannelData(*anchn);
			fm::string tp = pointer;//data.targetPointer;
			fm::string tq = qualifier;//data.targetQualifier;

			if ( !tp.size() )
				continue;

			fstring name = tp.substr(0, tp.find('/'));
			//FCDGeometry *gn = file->FindGeometry(name);
			FCDGeometry *gn = NULL;
			FCDGeometryLibrary* gl = file->GetGeometryLibrary();
			FCDGeometrySource *gs = NULL;
			for ( int tmp=0; tmp<gl->GetEntityCount(); tmp++)
			{
				gn = gl->GetEntity( tmp );
				//if ( name == gn->GetDaeId() )
				//	break;
				//gn = NULL;
				FCDGeometryMesh *gm = gn->GetMesh();
				for (int tm=0; tm<gm->GetSourceCount(); tm++ )
				{
					gs = gm->GetSource( tm );
					if ( name == gs->GetDaeId())
					{
						//gs->SetDaeId(gs->GetDaeId() + "_FAMC_");
						break;
					}
					//else
					//	if ( (name + "_FAMC_") == gs->GetDaeId() )
					//		break;
					gs = NULL;
				}
				if ( !gs )
					gn = NULL;
			}


			
			//add extra node
			FCDExtra * fe = NULL;
			FCDEType *ty = NULL;
			FCDETechnique *th = NULL;
			FCDENode *tn= NULL;
			fe = an->GetExtra();
			ty = fe->AddType("");
			th = ty->AddTechnique("famc");
			//change
			//CalculateNodeTargetPointer
			
			fstring daeid = pointer;//data.targetPointer;
			daeid.append(qualifier);//data.targetQualifier);
			th->AddParameter( "anim_target" , daeid );

			fstring tm = pointer;//data.targetPointer;
			tm = "extra";
			tech->GetChildNode(0)->SetContent(TO_FSTRING(tm));//data.targetPointer = tm;
			//anchn->SetTargetPointer(tm);
			tech->GetChildNode(1)->SetContent(TO_FSTRING("(" + TO_FSTRING(i) + ")"));//data.targetQualifier = "(" + TO_FSTRING(i) + ")";
			//anchn->SetTargetQualifier("(" + TO_FSTRING(i) + ")");
			
			//FCDEntity *gs = file->FindEntity( name );
			//gn = (FCDGeometry*) gs->GetParent();
			
			//FCDGeometry *gn = file->FindGeometry("Box01-mesh");
			bool usePI = true;
			if ( !gn )
			{
				//FCDVisualSceneNodeLibrary vl = 
				FCDSceneNode  *joint = file->FindSceneNode(name);
				if ( joint->IsJoint() )
					usePI = false;
				else
				{
					for ( int j = 0; j < joint->GetChildrenCount(); j++ )
					{
						FCDSceneNode* tmp = joint->GetChild(j);
						if ( tmp->IsJoint() )
						{
							usePI = false;
							break;
						}
					}

				}
					
			}

			if ( !gn && !usePI )
			{
				useBBA = true; 

				m_pAnimation[0]->EncodeType = FCDTransform::Type::MATRIX; 

				m_BBACount = 1;

				FMMatrix44List list;
				FMMatrix44 tmp;
				FMMatrix44List listint;
				FMMatrix44 tmpint;
				FMMatrix44List listoutt;
				FMMatrix44 tmpoutt;
				if ( tp.find( "rotation" ))
				{

				}
				int tmpList = 0;
				if ( tp.find( "transform" ) )
				{
					ancur	= anchn->GetCurve( 0 );
					FCDAnimationKey** keys = ancur->GetKeys();
					cnt  = ancur->GetKeyCount();
					for ( int k = 0; k < cnt; k++ )
						list.push_back(tmp);
					if ( !tq.size() )
					{
						for ( int k =0; k<16; k++ )
						{
							ancur	= anchn->GetCurve( k );
							keys	= ancur->GetKeys();

							for (int l=0; l<ancur->GetKeyCount(); l++)
								list[l][k%4][k/4] = keys[l]->output;
								
							ancur->ConvertValues( &ConvFunc, &ConvFunc );
							ancur->ConvertInputs( NULL, &ConvFunc );
						}
					}
					else
					{
						size_t first  = tq.find_first_of( "(" );
						size_t second = tq.find_last_of( ")" );
						char x = tq.at( first + 1 );
						char y = tq.at( second - 1 );

						int ix = x-'0';
						int iy = y-'0';

						ancur	= anchn->GetCurve( 0 );
						keys	= ancur->GetKeys();

						for (int l=0; l<ancur->GetKeyCount(); l++)
							list[l][ix][iy] = keys[l]->output;
							
						ancur->ConvertValues( &ConvFunc, &ConvFunc );
						ancur->ConvertInputs( NULL, &ConvFunc );
					}
				}
				matList.push_back(list);
			}
			else
			{	
				if( isFAMC )
					if (usePI)
						printf("FAMC cannot be used for position interpolator nor orientation interpolator\n");
				if ( isFAMC && !usePI )
				{
					m_AnimationCount = 0;
					
					ancur = anchn->GetCurve( 0 );
					FCDAnimationKey** keys = ancur->GetKeys();
					int count  = ancur->GetKeyCount();
					int k = 0;
					int n = 0;
					
					for (k=0; k<ListMesh.size(); k++)
						//if ( !stricmp(name.c_str(), ListMesh[k][0].mesh->Get3DMCFileID()))
						if ( !stricmp(gn->GetDaeId().c_str(), ListMesh[k][0].mesh->Get3DMCFileID()))
							break;
						//if ( !stricmp("Box01-mesh", ListMesh[k][0].mesh->Get3DMCFileID()))
								//break;

					if ( k == ListMesh.size())
					{
						ListMesh.push_back ( std::vector<MeshFrame>());
						for (int m=0;m<count;m++)
						{
							MeshFrame mf;
							mf.mesh = Meshto3DMC( gn );
							FCDAnimationKey* key = ancur->GetKey(m);
							mf.time = key->input;
							ListMesh[k].push_back ( mf );
							
						}
					}

					//const FCDAnimated* fanim = file->FindAnimatedValue(file->FindAnimatedTarget(tap+taq));
					//else
					/*size_t begin = tq.find( '(' );
					size_t end = tq.find( ')' );
					fstring tmp = tq.substr ( begin +1, end - begin  - 1 );
					int pos = atoi( tmp.c_str() );
					tmp = tq.substr(end+2,1);*/
					for (int tmp=0; tmp<anchn->GetCurveCount();tmp++)
						UpdateMeshes( ListMesh[k], tp , tq , anchn->GetCurve(tmp), tmp );
					ancur->ConvertValues( &ConvFunc, &ConvFunc );
					ancur->ConvertInputs( NULL, &ConvFunc );
					gs->GetAnimatedValues().clear();
				}
				else
				{
					//Interpolators
					fm::string tp1, tp2; 
					size_t splitIndex = tp.find_first_of("/"); 

					if(splitIndex != fm::string::npos) { 
						tp1 = tp.substr(0, splitIndex); 
						tp2 = tp.substr (splitIndex+1); 
					} else { 
						tp1 = tp; 
						tp2.clear(); 
					} 

					FCDTransform::Type temp; 

					switch (temp = GetTransformTypeName(tp1, tp2)) 
					{ 
					case FCDTransform::Type::ROTATION:  
						{
							useOI = true; 

							struct KeyInfo tempOIKeyInfo; 

							float OIx = 0.0,OIy =0.0,OIz =0.0; 
							FCDTRotation *rot = (FCDTRotation *)GetTransformName(tp1, tp2); 
							if(rot) { 
								OIx = rot->GetAxis().x; 
								OIy = rot->GetAxis().y; 
								OIz = rot->GetAxis().z; 
							}
							tempOIKeyInfo.x = 0; //OIx; 
							tempOIKeyInfo.y = 0; //OIy; 
							tempOIKeyInfo.z = 0; //OIz; 

							ancur   = anchn->GetCurve( 0 ); 

							for ( int i = 0; i < ancur->GetKeyCount(); i++ ) { 
								tempOIKeyInfo.key = ancur->GetKey(i)->input; 
								//tempOIKeyInfo.keyValue = ancur->GetKey(i)->output; 

								bool sw =false; 
								for( int j=0; j<OIKey.size(); j++) { 
									if(OIKey[j].key == tempOIKeyInfo.key) { 
										if(OIx != 0.0)  
											OIKey[j].x=ancur->GetKey(i)->output; 
										if(OIy != 0.0)  
											OIKey[j].y=ancur->GetKey(i)->output; 
										if(OIz != 0.0)  
											OIKey[j].z=ancur->GetKey(i)->output; 
										// KeyValue ֟ 
										sw= true; 
									} 

								} 
								if(!sw) { 
									if(OIx != 0.0)  
										tempOIKeyInfo.x=ancur->GetKey(i)->output; 
									if(OIy != 0.0)  
										tempOIKeyInfo.y=ancur->GetKey(i)->output; 
									if(OIz != 0.0)  
										tempOIKeyInfo.z=ancur->GetKey(i)->output; 
									OIKey.push_back(tempOIKeyInfo); 

									for(int j=OIKey.size() - 1; j>0; j--) { 
										if(OIKey[j].key < OIKey[j-1].key) { 
											struct KeyInfo swapTemp = OIKey[j]; 
											OIKey[j] = OIKey[j - 1]; 
											OIKey[j - 1] = swapTemp; 
										} 
									} 
								}
							}
							//blagica
							//values  to 0
							ancur->ConvertValues( &ConvFunc, &ConvFunc );
							ancur->ConvertInputs( NULL, &ConvFunc );	
							//gs->GetAnimatedValues().clear();
							break;
						}
					case FCDTransform::Type::SKEW: // CI
						{
							ancur = anchn->GetCurve( 0 );
							FCDAnimationKey** keys = ancur->GetKeys();
							int count  = ancur->GetKeyCount();
							int k = 0;
							int n = 0;
							
							for (k=0; k<ListMesh.size(); k++)
								//if ( !stricmp(name.c_str(), ListMesh[k][0].mesh->Get3DMCFileID()))
								if ( !stricmp(gn->GetDaeId().c_str(), ListMesh[k][0].mesh->Get3DMCFileID()))
									break;
								//if ( !stricmp("Box01-mesh", ListMesh[k][0].mesh->Get3DMCFileID()))
										//break;

							if ( k == ListMesh.size())
							{
								ListMesh.push_back ( std::vector<MeshFrame>());
								for (int m=0;m<count;m++)
								{
									MeshFrame mf;
									mf.mesh = Meshto3DMC( gn );
									FCDAnimationKey* key = ancur->GetKey(m);
									mf.time = key->input;
									ListMesh[k].push_back ( mf );
									
								}
							}

							size_t begin = tq.find( '(' );
							size_t end = tq.find( ')' );
							fstring tmps = tq.substr ( begin +1, end - begin  - 1 );
							int pos = atoi( tmps.c_str() );
							tmps = tq.substr(end+2,1);
							int coord = tmps.c_str()[0] - 'X';
							if ( coord < 0 )
								coord = tmps.c_str()[0] - 'x';
							if (coord < 0 || coord > 2)
								 coord = -1;
						
							useCI = true; 
							char buf[100]; 
							int d; 
							sscanf(tq.c_str(), "(%d)", &d); 

							if ( coord >=0 )
								d = d * 3 + coord;

							//blagica::why not for all curves???
							ancur   = anchn->GetCurve( 0 ); 

							for ( int i = 0; i < ancur->GetKeyCount(); i++ ) { 
								FCDAnimationKey* key = ancur->GetKey(i);
								if(CIdata.find(key->input) == CIdata.end()) { 
									FCDGeometryLibrary      *gmlib  = file->GetGeometryLibrary(); 

									for(int j=0; j<gmlib->GetEntityCount(); j++) { 
										FCDGeometry             *ge = gmlib->GetEntity( j ); 
										FCDGeometryMesh         *gm =  ge->GetMesh(); 
										FCDGeometrySource       *gmsC   = gm->FindSourceByType(FUDaeGeometryInput::POSITION); 
										FloatList verticesC		= gmsC->GetSourceData().GetDataList(); 
										int strideC = gmsC->GetStride(); 

										bool sw = false; 
										for(int k=0; k<verticesC.size(); k++ ) { 
										 
											CIdata[ancur->GetKey(i)->input][k] = verticesC[k]; 
											sw = true; 
										} 
										if(sw) break; 
									} 
								} 
							
								CIdata[ancur->GetKey(i)->input][d] = ancur->GetKey(i)->output; //blagica::should be d*strideC!!!!!
								//fprintf(f,"%f\n", ancur->GetKey(i)->output); 
							
							}
							//fclose(f);
							//blagica
							//values  to 0
							ancur->ConvertValues( &ConvFunc, &ConvFunc );
							ancur->ConvertInputs( NULL, &ConvFunc );
							//gs->GetAnimatedValues().clear();
							//const FCDAnimated* fanim = file->FindAnimatedValue(file->FindAnimatedTarget(tap+taq));
							//else
							/*size_t begin = tq.find( '(' );
							size_t end = tq.find( ')' );
							fstring tmp = tq.substr ( begin +1, end - begin  - 1 );
							int pos = atoi( tmp.c_str() );
							tmp = tq.substr(end+2,1);*/
							for (int tmp=0; tmp<anchn->GetCurveCount();tmp++)
								UpdateMeshes( ListMesh[k], tp , tq , anchn->GetCurve(tmp), tmp );
							ancur->ConvertValues( &ConvFunc, &ConvFunc );
							ancur->ConvertInputs( NULL, &ConvFunc );
							//gs->GetAnimatedValues().clear();

							break; 
						}
					case FCDTransform::Type::TRANSLATION:  //PI
						{
							ancur	= anchn->GetCurve( 0 );
							for ( int j = 0; j < ancur->GetKeyCount(); j++ ) { 
								printf("%f ", ancur->GetKey(j)->input); 
								m_pAnimation[0]->m_key.push_back(ancur->GetKey(j)->input); 
							} 
							printf("\n");
							for(int i=0; i<ancur->GetKeyCount(); i++) { 
								for ( int j = 0; j < anchn->GetCurveCount(); j++ ) { 
									ancur   = anchn->GetCurve( j ); 
									printf("%f ", ancur->GetKey(i)->output); 
									m_pAnimation[0]->m_keyvalue.push_back(ancur->GetKey(i)->output); 
								} 
							}
							//blagica
							//values  to 0
							ancur->ConvertValues( &ConvFunc, &ConvFunc );
							ancur->ConvertInputs( NULL, &ConvFunc );
							//gs->GetAnimatedValues().clear();
							break; 
						} 
					}
					if ( temp != FCDTransform::Type::MATRIX )
						m_pAnimation[0]->EncodeType = temp;
					//if ( useCI ) 
					//	gs->GetAnimatedValues().clear();
				} 
			}
		}
	}

	
	//m_pAnimation[0] = new MP4AnimationBuff( 0, an->GetDaeId().c_str(), cnt );
	//m_AnimationCount++;
	MP4Frames *frm = NULL;
	for ( int j=0; j< cnt; j++)
	{
		frm = new MP4Frames();
		frm->m_pbonesData = new MP4BoneData*[matList.size()];
		frm->m_boneDataCount = matList.size();
		for (int k=0; k < matList.size() ; k++)
		{
			FMVector3 sc;
			FMVector3 tr;
			FMVector3 rot;
			FMQuaternion rotQv;
			float	t	= 0 ;
			matList[k][j].Decompose( sc, rot, tr,t);
			rotQv = GetQuaternion( matList[k][j] );

			MP4BoneData *bd = new MP4BoneData ();
			
				Vec3f tmp3;
				tmp3.x =  tr.x;
				tmp3.y =  tr.y;
				tmp3.z =  tr.z;
				bd->SetTranslation( tmp3 ); 

				//tmp3.x = sc.x;
				//tmp3.y = sc.y;
				//tmp3.z = sc.z;
				//bd->SetScale( tmp3 ); 
			
				Vec4f tmp4;
				tmp4.x =  rotQv.x;
				tmp4.y =  rotQv.y;
				tmp4.z =  rotQv.z;
				tmp4.q = rotQv.w;
				bd->SetRotation( tmp4 );
			
			frm->m_pbonesData[k] = bd;

		}

		m_pAnimation[0]->AddMP4Frame( frm );
		
	}

	if(useCI) { 
		fm::map<float, fm::map<int, float>>::iterator i; 
		for(i = CIdata.begin(); i != CIdata.end(); i++) { 
			m_pAnimation[0]->m_key.push_back(i->first); 
			//printf("|%d| ", i); 
			fm::map<int, float>::iterator j; 
			for(j = i->second.begin(); j != i->second.end(); j++) { 
				m_pAnimation[0]->m_keyvalue.push_back(j->second); 
			} 
		} 
		printf("\nKEY :"); 
		for(int k = 0; k<m_pAnimation[0]->m_key.size(); k++) { 
			printf("%f ", m_pAnimation[0]->m_key[k]); 
		} 
		printf("\nKEYVALUE :"); 
		for(int k = 0; k<m_pAnimation[0]->m_keyvalue.size(); k++) { 
			printf("%f ", m_pAnimation[0]->m_keyvalue[k]); 
		} 
	} 
	if(useOI) { 
		for(int i=0; i<OIKey.size(); i++) { 
			m_pAnimation[0]->m_key.push_back(OIKey[i].key); 
			// DAE   
			//raidiantoangle(&OIKey[i].y, &OIKey[i].z, &OIKey[i].x); 

			double *keyValue = rotate(OIKey[i].y/180.0 * M_PI, OIKey[i].z/180.0 * M_PI, OIKey[i].x/180.0 * M_PI); // input   
			printf("\nENCODE : "); 
			for(int j= 0; j<4; j++)  
				printf("%f, ", keyValue[j]); 
			m_pAnimation[0]->m_keyvalue.push_back(keyValue[0]); 
			m_pAnimation[0]->m_keyvalue.push_back(keyValue[1]); 
			m_pAnimation[0]->m_keyvalue.push_back(keyValue[2]); 
			m_pAnimation[0]->m_keyvalue.push_back(keyValue[3]); 
		} 
	} 
		
	//if (! frm )
	//	return false;
	//int tmpcnt = cnt%16 ;
	//if  (typeBBA == "DCT")
	//	while (  tmpcnt<16 )
	//		{
	//			
	//			m_pAnimation[0]->AddMP4Frame( frm );
	//			tmpcnt++;
	//		}

	//encode bones

	
	return true;

}


int ColladaParser::GetBBACount()
{
	return m_BBACount;
}

int ColladaParser::GetAnimationcount()
{
	return m_AnimationCount;
}
MP4AnimationBuff* ColladaParser::GetAnimation( int id )
{
	if ((id<0)||(id>=m_AnimationCount))
		return NULL;
	return m_pAnimation[id];

}

bool ColladaParser::LoadColladaFile( const char * fileName, bool isFAMC )
{
	m_pColladaFileName = strdup(fileName);
	ParseColladaFile( isFAMC );
	return true;
}

bool ColladaParser::GetXMLrest ( char* &buff, int &size, bool alloc )
{
	FILE* f = fopen ( m_pRestColladaFileName, "rb" );
	if ( !f )
		return false;
	fseek(f,0, SEEK_END);
	size = ftell(f);
	fseek(f,0,0);
	if ( alloc )
		buff = (char*) malloc ( size );
	fread( buff, 1, size, f );
	fclose ( f );
	return true;
}


MP43DMC* ColladaParser::Meshto3DMC( FCDGeometry *ge )
{
			
	FCDGeometryMesh		*gm     = NULL;
	gm =  ge->GetMesh();
		
	bool loadNormalIndex	= true;
	bool loadTexindex		= true;
	FCDGeometrySource	*gmsC	= NULL;
	FCDGeometrySource	*gmsT	= NULL;
	FCDGeometrySource	*gmsN	= NULL;
	FCDGeometrySource	*gmsRGB	= NULL;
	FCDGeometryPolygons *gmp	= NULL;
	FloatList	verticesC;
	FloatList	verticesT;
	FloatList	verticesN;
	FloatList	verticesRGB;
	FCDGeometryPolygonsInput *indC;
	FCDGeometryPolygonsInput *indT;
	FCDGeometryPolygonsInput *indN;
	FCDGeometryPolygonsInput *indRGB;


	int strideN = 3;
	int strideT = 3;
	int strideC = 3;
	int strideRGB = 3;

	int indexN = 0;
	int indexT = 0;
	int indexC = 0;
	int indexRGB = 0;

	// Load coordinates
 
	gmsC = gm->FindSourceByType(FUDaeGeometryInput::POSITION);
	if ( gmsC )
	{
		strideC = gmsC->GetStride();
		verticesC = gmsC->GetSourceData().GetDataList();
	}
	
	
	// Load normals
	gmsN = gm->FindSourceByType(FUDaeGeometryInput::NORMAL);
	if ( gmsN )
	{
		strideN = gmsN->GetStride();
		verticesN = gmsN->GetSourceData().GetDataList();
	}
	
	// Load textcoords
	gmsT = gm->FindSourceByType(FUDaeGeometryInput::TEXCOORD);
	if ( gmsT ) 
	{
		strideT = gmsT->GetStride();
		verticesT = gmsT->GetSourceData().GetDataList();
	}

	// Load colors
	gmsRGB = gm->FindSourceByType(FUDaeGeometryInput::COLOR);
	if ( gmsRGB )
	{
		strideRGB = gmsRGB->GetStride();
		verticesRGB = gmsRGB->GetSourceData().GetDataList();
	}
	
	//Indexes
	for (int tmp = 0; tmp < gm->GetPolygonsCount(); tmp++ )
	{
		gmp = gm->GetPolygons ( tmp );
		if (gmp)
		{
			if ( gmsC )
				indC = gmp->FindInput(FUDaeGeometryInput::POSITION);
			if ( gmsN ) 
				indN = gmp->FindInput(FUDaeGeometryInput::NORMAL);
			if ( gmsT ) 
				indT = gmp->FindInput(FUDaeGeometryInput::TEXCOORD);
			if ( gmsRGB ) 
				indRGB = gmp->FindInput(FUDaeGeometryInput::COLOR);

			// Get number of all coordinates index
			if ( gmsC )
				indexC+=indC->GetIndexCount();
			// Get number of all texture coordinates index
			if ( gmsT )
				indexT+=indT->GetIndexCount(); 
			// Get number of all normals index
			if ( gmsN )
				indexN+=indN->GetIndexCount(); 
			// Get number of all colors index
			if ( gmsRGB )
				indexRGB+=indRGB->GetIndexCount(); 
		}
	}

	//Create 3DMC object and fill it
	
	MP43DMC* mp43dmc = new MP43DMC(verticesC.size()/strideC, verticesN.size()/strideN,verticesT.size()/strideT,verticesRGB.size()/strideRGB,indexC/3,indexN/3, indexT/3, indexRGB/3, 0 ,  gm->GetDaeId().c_str() );

	// Add coordinates
	for( int j = 0; j < verticesC.size(); j+=strideC )
	{
		mp43dmc->AddCoordinate( verticesC[j + 0], verticesC[j + 1], verticesC[j + 2]  );
	}
	
	// Add normals
	for( int j = 0; j < verticesN.size(); j+=strideN )
	{
		mp43dmc->AddNormal( verticesN[j + 0], verticesN[j + 1], verticesN[j + 2] );
	}
	
	

	// Add texture coordinates
	for( int j = 0; j < verticesT.size(); j += strideT )
	{
		float tmpu, tmpv;
		tmpu = verticesT[j+0];
		
		if (tmpu<0)
			tmpu = tmpu + (int)tmpu+1;
		if (tmpu>=0)
			tmpu = tmpu - (int)tmpu + 0.000001;
		
		tmpv = verticesT[j+1];
		if (tmpv<0)
			tmpv = tmpv + (int)tmpv+1;
		if (tmpv>0)
			tmpv = tmpv - (int)tmpv + 0.000001;
		
		mp43dmc->AddTextureCoordinate( tmpu, tmpv );
	}
	
	
	//Add colors
	for( int j = 0; j < verticesRGB.size(); j+=strideRGB )
	{
		mp43dmc->AddColor( verticesRGB[j + 0], verticesRGB[j + 1], verticesRGB[j + 2] );
	}

	
	//Indexes
	for (int tmp = 0; tmp < gm->GetPolygonsCount(); tmp++ )
	{
		gmp = gm->GetPolygons ( tmp );
		
		if (gmp)
		{
			if ( gmsC )
				indC = gmp->FindInput(FUDaeGeometryInput::POSITION);
			if ( gmsN ) 
				indN = gmp->FindInput(FUDaeGeometryInput::NORMAL);
			if ( gmsT ) 
				indT = gmp->FindInput(FUDaeGeometryInput::TEXCOORD);
		
			if ( gmsRGB ) 
				indRGB = gmp->FindInput(FUDaeGeometryInput::COLOR);

			// Load the coordinates index
			if ( gmsC )
				for (int t=0;t<indC->GetIndexCount(); t+=3)
					mp43dmc->AddCoordinateIndex( indC->GetIndices()[t], indC->GetIndices()[t+1],indC->GetIndices()[t+2]);
					
				
			// Load the texture coordinates index
			if ( gmsT )
				for (int t=0;t<indT->GetIndexCount();t+=3 )
				{
					mp43dmc->AddTextureCoordinateIndex( indT->GetIndices()[t],indT->GetIndices()[t+1],indT->GetIndices()[t+2]);
					
				}

			// Load the normals index
			if ( gmsN )
				for (int t=0;t<indN->GetIndexCount();t+=3 )
				{
					mp43dmc->AddNormalIndex( indN->GetIndices()[t], indN->GetIndices()[t+1], indN->GetIndices()[t+2] );
				}

			// Load the colors index
			if ( gmsRGB )
				for (int t=0;t<indRGB->GetIndexCount();t+=3 )
				{
					mp43dmc->AddColorIndex( indRGB->GetIndices()[t], indRGB->GetIndices()[t+1], indRGB->GetIndices()[t+2] );
				}
		}
	}

	return mp43dmc;
	
}

bool ColladaParser::UpdateMeshes( std::vector<MeshFrame> &meshes,  fstring tp, fstring tq, FCDAnimationCurve* ancur, int crd )
{
	int pos=0;
	int coord=0;
	
	//fstring type = tp.substr(0, tp.find('/'));
	size_t begin = tq.find( '(' );
	size_t end = tq.find( ')' );
	fstring tmp = tq.substr ( begin +1, end - begin  - 1 );
	pos = atoi( tmp.c_str() );
	tmp = tq.substr(end+2,1);
	
		coord = tmp.c_str()[0] - 'X';
		if ( coord < 0 )
			coord = tmp.c_str()[0] - 'x';
	if (coord < 0 || coord > 2)
		 coord = crd;

	int count  = ancur->GetKeyCount();
	
	for (int i=0; i<count;i++)
	{
		FCDAnimationKey* key = ancur->GetKey(i);
		meshes[i].time = key->input;
		if ( tp.find( "positions" )< 1000 ) 
			meshes[i].mesh->m_pcoordinates[ pos * 3 + coord ] += key->output;
		size_t tr = tp.find( "normals" );
		if ( tp.find( "normals" )< 1000 ) 
			meshes[i].mesh->m_pnormals[ pos * 3 + coord ] += key->output;
		if ( tp.find( "colors" )< 1000 ) 
			meshes[i].mesh->m_pcolors[ pos * 3 + coord ] += key->output;
		if ( tp.find( "text" )< 1000) 
			meshes[i].mesh->m_ptexturecoordinates[ pos * 3 + coord ] += key->output;
	}
return true;
}

