#ifndef _COLLADAPARSER_H_
#define _COLLADAPARSER_H_

/******************************************************************************\
* Includes
\******************************************************************************/
#include <vector>

#include "MP43DMC.h"
#include "MP4Animation.h"

#include "ColladaInc.h"
#include "FCollada.h"
#include "FAXColladaParser.h"
#include "FArchiveXML.h"

/******************************************************************************\
* Class ColladaParser
\******************************************************************************/
class ColladaParser
{
public:

	//de/constructor
	ColladaParser(void);
	~ColladaParser(void);

	//methods
	//fileName == name of the input file
	//load, parse and remove mesh and animation from fileName
	bool LoadColladaFile( const char * fileName, bool isFAMC );
	//return the number of meshes in collada == number of 3DMC stream in mp4
	int Get3DMCStreamsCount();
	//return id-th mesh in collada
	MP43DMC* Get3DMCStream( int id );
	//return the number of bba of collada
	int GetBBACount();
	//return the number of animations of collada
	int GetAnimationcount();
	//return id-th animation in collada
	MP4AnimationBuff* GetAnimation( int id );
	//return the rest of collada file in buffer
	// If alloc is true, the buffer is allocated inside the function
	// If alloc is false, the buffer should be allocated before calling the function
	bool GetXMLrest ( char* &buff, int &size, bool alloc );
	//return the count of images
	int GetImagesCount(){ return m_ImageCount ;}
	//return the ID Name ( dae id )  of the i-th image
	char* GetImageID( int i ){return (char*)m_pImageID[i].c_str(); }
	//return the image name of the i-th image
	char* GetImageName( int i ){return (char*)m_pImageName[i].c_str(); }
	//return the image name by ID name (by dae id )
	char* GetImageName( char * IDName );

	std::vector< std::vector< MeshFrame> > ListMesh;




private:

	//variables
	const char*			m_pColladaFileName;
	MP4AnimationBuff**	m_pAnimation;
	MP43DMC**			m_p3dmc;
	char*				m_pRestColladaFileName;
	int					m_3dmcCount;
	int					m_AnimationCount;
	int					m_BBACount;
	FCDocument			*file;
	fstring*			m_pImageID;
	fstring*			m_pImageName;
	int					m_ImageCount;
	/*struct MeshFrame
	{
		MP43DMC* mesh;
		float time;
	};*/
	//
	//MeshFrame** ListMesh;
	
	

	//methods
	bool MakeCopyColladaFile();
	bool ParseColladaFile( bool isFAMC );
	bool RemoveGeometry();
	bool RemoveAnimation( bool isFAMC );
	bool ParseImages();
	FMQuaternion GetQuaternion( FMMatrix44& kRot );
	MP43DMC* Meshto3DMC( FCDGeometry *ge );
	bool UpdateMeshes( std::vector<MeshFrame>& meshes,  fstring tp, fstring  tq, FCDAnimationCurve* ancur, int crd );

	FCDTransform::Type GetTransformTypeNameSub(fm::string &tp, fm::string &tq, FCDSceneNode* node) const; 
	FCDTransform::Type GetTransformTypeName(fm::string &tp, fm::string &tq) const; 
	FCDTransform* GetTransformNameSub(fm::string &tp, fm::string &tq, FCDSceneNode* node) const; 
	FCDTransform* GetTransformName(fm::string &tp, fm::string &tq) const; 



};







#endif //_COLLADAPARSER_H_
