/******************************************************************************\
* Includes
\******************************************************************************/
#include "ColladaParser.h"

/******************************************************************************\
 ColladaParser.cpp: implementation of the ColladaParser class.
\******************************************************************************/


/******************************************************************************\
* Class ColladaParser
\******************************************************************************/

//constructor
ColladaParser::ColladaParser()
{
	m_pColladaFileName		= NULL;
	m_pAnimation			= NULL;
	m_p3dmc					= NULL;
	m_pRestColladaFileName	= NULL;
	m_3dmcCount				= 0;
	m_AnimationCount		= 0;
	m_pImageID				= 0;
	m_pImageName			= 0;
	m_ImageCount			= 0;

	
}

//destructor 
ColladaParser::~ColladaParser()
{
	/*if ( m_pColladaFileName )
		free(m_pColladaFileName);*/
	remove ( m_pRestColladaFileName );
	if ( m_pRestColladaFileName )
		free(m_pRestColladaFileName);
	for ( unsigned int i = 0; i < m_3dmcCount; i++ )
		delete m_p3dmc[i];
	delete[] m_p3dmc;
	for ( unsigned int i = 0; i < m_AnimationCount; i++ )
		delete m_pAnimation[i];
	delete[] m_pAnimation;
	delete[] m_pImageID;
	delete[] m_pImageName;

}


bool ColladaParser::MakeCopyColladaFile()
{
	
	m_pRestColladaFileName = (char*) malloc ( strlen(m_pColladaFileName)+8);
	sprintf(m_pRestColladaFileName,"%stmp.dae",m_pColladaFileName);
	
	FILE* f = fopen ( m_pColladaFileName, "rb" );
	if ( !f )
		return false;
	fseek(f,0, SEEK_END);
	int size = ftell(f);
	fseek(f,0,0);
	char* buff = (char*) malloc ( size );
	fread( buff, 1, size, f );
	fclose ( f );
	f = fopen ( m_pRestColladaFileName, "wb");
	if (!f)
		return false;
	
	fwrite( buff, 1, size, f );
	fclose ( f );
	
	if ( buff )
		free ( buff );

	return true;
}

bool ColladaParser::ParseColladaFile()
{
	MakeCopyColladaFile();
	FCollada::Initialize();
	file = FCollada::LoadDocument( (const fchar*) m_pRestColladaFileName );
	RemoveGeometry();
	RemoveAnimation();
	ParseImages();
	FCollada::SaveDocument( file, (const fchar*) m_pRestColladaFileName );
	delete file;
	FCollada::Release();
	return true;
}

bool ColladaParser::ParseImages()
{
	FCDImageLibrary *imlib = NULL;
	FCDImage		*im	   = NULL;
	
	imlib = file->GetImageLibrary();
	m_ImageCount = imlib->GetEntityCount();

	m_pImageID = new fstring[ m_ImageCount ];

	m_pImageName = new fstring[ m_ImageCount ];
		
	for ( int i=0; i<m_ImageCount; i++)
	{
		im = imlib->GetEntity( i );
		m_pImageID[i] = im->GetDaeId();
		m_pImageName[i] = im->GetFilename();
	}

	return true;
}

char* ColladaParser::GetImageName( char * IDName )
{	
	for ( int i=0; i<m_ImageCount; i++)
	{
		if (m_pImageID[i] == IDName) 
			return (char*)m_pImageName[i].c_str();
	}
	return NULL;

}


bool ColladaParser::RemoveGeometry( )
{
	FCDGeometryLibrary	*gmlib	= NULL;
	FCDGeometry			*ge		= NULL;
	FCDGeometryMesh		*gm     = NULL;
	
	char		* buff = NULL;
	int			buf_size = 0;

	gmlib = file->GetGeometryLibrary();

	int cnt = 0;
	for ( int i=0; i<gmlib->GetEntityCount(); i++)
	{
		ge = gmlib->GetEntity( i );
		gm =  ge->GetMesh();
		if ( gm )
			cnt++;
	}

	m_p3dmc = new MP43DMC*[cnt];

	for ( int i=0; i<gmlib->GetEntityCount(); i++)
	{
		ge = gmlib->GetEntity( i );
		gm =  ge->GetMesh();
		if ( gm )
		{
			bool loadNormalIndex	= true;
			bool loadTexindex		= true;
			FCDGeometrySource	*gmsC	= NULL;
			FCDGeometrySource	*gmsT	= NULL;
			FCDGeometrySource	*gmsN	= NULL;
			FCDGeometrySource	*gmsRGB	= NULL;
			FCDGeometryPolygons *gmp	= NULL;
			FloatList	verticesC;
			FloatList	verticesT;
			FloatList	verticesN;
			FloatList	verticesRGB;
			FCDGeometryPolygonsInput *indC;
			FCDGeometryPolygonsInput *indT;
			FCDGeometryPolygonsInput *indN;
			FCDGeometryPolygonsInput *indRGB;


			int strideN = 3;
			int strideT = 3;
			int strideC = 3;
			int strideRGB = 3;

			int indexN = 0;
			int indexT = 0;
			int indexC = 0;
			int indexRGB = 0;

			// Load coordinates
		 
			gmsC = gm->FindSourceByType(FUDaeGeometryInput::POSITION);
			if ( gmsC )
			{
				strideC = gmsC->GetStride();
				verticesC = gmsC->GetSourceData();
			}
			
			
			// Load normals
			gmsN = gm->FindSourceByType(FUDaeGeometryInput::NORMAL);
			if ( gmsN )
			{
				strideN = gmsN->GetStride();
				verticesN = gmsN->GetSourceData();
			}
			
			// Load textcoords
			gmsT = gm->FindSourceByType(FUDaeGeometryInput::TEXCOORD);
			if ( gmsT ) 
			{
				strideT = gmsT->GetStride();
				verticesT = gmsT->GetSourceData();
			}

			// Load colors
			gmsRGB = gm->FindSourceByType(FUDaeGeometryInput::COLOR);
			if ( gmsRGB )
			{
				strideRGB = gmsRGB->GetStride();
				verticesRGB = gmsRGB->GetSourceData();
			}
			
			//Indexes
			for (int tmp = 0; tmp < gm->GetPolygonsCount(); tmp++ )
			{
				gmp = gm->GetPolygons ( tmp );
				if (gmp)
				{
					if ( gmsC )
						indC = gmp->FindInput(FUDaeGeometryInput::POSITION);
					if ( gmsN ) 
						indN = gmp->FindInput(FUDaeGeometryInput::NORMAL);
					if ( gmsT ) 
						indT = gmp->FindInput(FUDaeGeometryInput::TEXCOORD);
					if ( gmsRGB ) 
						indRGB = gmp->FindInput(FUDaeGeometryInput::COLOR);

					// Get number of all coordinates index
					if ( gmsC )
						indexC+=indC->GetIndexCount();
					// Get number of all texture coordinates index
					if ( gmsT )
						indexT+=indT->GetIndexCount(); 
					// Get number of all normals index
					if ( gmsN )
						indexN+=indN->GetIndexCount(); 
					// Get number of all colors index
					if ( gmsRGB )
						indexRGB+=indRGB->GetIndexCount(); 
				}
			}

			//Create 3DMC object and fill it
			m_3dmcCount ++;
			m_p3dmc[i] = new MP43DMC(verticesC.size()/strideC, verticesN.size()/strideN,verticesT.size()/strideT,verticesRGB.size()/strideRGB,indexC/3,indexN/3, indexT/3, indexRGB/3, i ,  gm->GetDaeId().c_str() );

			// Add coordinates
			for( int j = 0; j < verticesC.size(); j+=strideC )
			{
				m_p3dmc[i]->AddCoordinate( verticesC[j + 0], verticesC[j + 1], verticesC[j + 2]  );
			}
			// remove coordinates from collada file
			verticesC.clear();
			gmsC->SetData( verticesC, 1, 0 , 0);

			// Add normals
			for( int j = 0; j < verticesN.size(); j+=strideN )
			{
				m_p3dmc[i]->AddNormal( verticesN[j + 0], verticesN[j + 1], verticesN[j + 2] );
			}
			
			// remove normals from collada file
			int t = verticesN.size();
			if ( t )
			{
				verticesN.clear();
				gmsN->SetData( verticesN, 1, 0 , 0);
			}

			// Add texture coordinates
			for( int j = 0; j < verticesT.size(); j += strideT )
			{
				float tmpu, tmpv;
				tmpu = verticesT[j+0];
				
				if (tmpu<0)
					tmpu = tmpu + (int)tmpu+1;
				if (tmpu>=0)
					tmpu = tmpu - (int)tmpu + 0.000001;
				
				tmpv = verticesT[j+1];
				if (tmpv<0)
					tmpv = tmpv + (int)tmpv+1;
				if (tmpv>0)
					tmpv = tmpv - (int)tmpv + 0.000001;
				
				m_p3dmc[i]->AddTextureCoordinate( tmpu, tmpv );
			}
			
			// remove texture coordinates from collada file
			t = verticesT.size();
			if ( t )
			{
				verticesT.clear();
				gmsT->SetData( verticesT, 1, 0 , 0);
			}

			//Add colors
			for( int j = 0; j < verticesRGB.size(); j+=strideRGB )
			{
				m_p3dmc[i]->AddColor( verticesRGB[j + 0], verticesRGB[j + 1], verticesRGB[j + 2] );
			}

			// remove texture coordinates from collada file
			t = verticesRGB.size();
			if ( t )
			{
				verticesRGB.clear();
				gmsRGB->SetData( verticesRGB, 1, 0 , 0);
			}
			
			//Indexes
			for (int tmp = 0; tmp < gm->GetPolygonsCount(); tmp++ )
			{
				gmp = gm->GetPolygons ( tmp );
				
				if (gmp)
				{
					if ( gmsC )
						indC = gmp->FindInput(FUDaeGeometryInput::POSITION);
					if ( gmsN ) 
						indN = gmp->FindInput(FUDaeGeometryInput::NORMAL);
					if ( gmsT ) 
						indT = gmp->FindInput(FUDaeGeometryInput::TEXCOORD);
				
					if ( gmsRGB ) 
						indRGB = gmp->FindInput(FUDaeGeometryInput::COLOR);

					// Load the coordinates index
					if ( gmsC )
						for (int t=0;t<indC->GetIndexCount(); t+=3)
							m_p3dmc[i]->AddCoordinateIndex( indC->GetIndices()[t], indC->GetIndices()[t+1],indC->GetIndices()[t+2]);
							
						
					// Load the texture coordinates index
					if ( gmsT )
						for (int t=0;t<indT->GetIndexCount();t+=3 )
						{
							m_p3dmc[i]->AddTextureCoordinateIndex( indT->GetIndices()[t],indT->GetIndices()[t+1],indT->GetIndices()[t+2]);
							
						}

					// Load the normals index
					if ( gmsN )
						for (int t=0;t<indN->GetIndexCount();t+=3 )
						{
							m_p3dmc[i]->AddNormalIndex( indN->GetIndices()[t], indN->GetIndices()[t+1], indN->GetIndices()[t+2] );
						}

					// Load the colors index
					if ( gmsRGB )
						for (int t=0;t<indRGB->GetIndexCount();t+=3 )
						{
							m_p3dmc[i]->AddColorIndex( indRGB->GetIndices()[t], indRGB->GetIndices()[t+1], indRGB->GetIndices()[t+2] );
						}

			
					// remove indices from collada file
					int tf = gmp->GetFaceCount();
						for ( int l = 0; l < tf; l++ )
							gmp->RemoveFace(0);
					// add data to determine  each mesh
					gmp->AddFace(3);
					indC = gmp->FindInput(FUDaeGeometryInput::POSITION);
					indN = gmp->FindInput(FUDaeGeometryInput::NORMAL);
					indT = gmp->FindInput(FUDaeGeometryInput::TEXCOORD);
					
					indRGB = gmp->FindInput(FUDaeGeometryInput::COLOR);
					for ( int i = 0; i < 3; i++ )
					{
						indC->GetIndices()[i] = 0;
						if ( indN )
							indN->GetIndices()[i] = 0;
						if (indT)
							indT->GetIndices()[i] = 0;
						if (indRGB)
							indRGB->GetIndices()[i] = 0;
					}
					indC->GetIndices()[0] = tf;
				}
			}
					
			//add extra node
			FCDExtra * fe = NULL;
			FCDEType *ty = NULL;
			FCDETechnique *th = NULL;
			FCDENode *tn= NULL;
			fe = ge->GetExtra();
			ty = fe->AddType("");
			th = ty->AddTechnique("mp4");
			fstring daeid = ge->GetDaeId();
			daeid.append(".3dmc");
			th->AddParameter( "init_from" , daeid );
		}
	}
	return true;
}


int ColladaParser::Get3DMCStreamsCount()
{
	return m_3dmcCount;
}


MP43DMC* ColladaParser::Get3DMCStream( int id )
{
	if ((id<0)||(id>=m_3dmcCount))
		return NULL;
	return m_p3dmc[id];
}




FMQuaternion ColladaParser::GetQuaternion( FMMatrix44& kRot )
{
	FMQuaternion q;
	
	float fTrace = kRot[0][0]+kRot[1][1]+kRot[2][2];
	float fRoot;

	if ( fTrace > 0.0 )
	{
		// |w| > 1/2, may as well choose w > 1/2
		fRoot = sqrt(fTrace + 1.0);  // 2w
		q.w = 0.5*fRoot;
		fRoot = 0.5/fRoot;  // 1/(4w)
		q.x = (kRot[2][1]-kRot[1][2])*fRoot;
		q.y = (kRot[0][2]-kRot[2][0])*fRoot;
		q.z = (kRot[1][0]-kRot[0][1])*fRoot;
	}
	else
	{
		// |w| <= 1/2
		static size_t s_iNext[3] = { 1, 2, 0 };
		size_t i = 0;
		if ( kRot[1][1] > kRot[0][0] )
			i = 1;
		if ( kRot[2][2] > kRot[i][i] )
			i = 2;
		size_t j = s_iNext[i];
		size_t k = s_iNext[j];

		fRoot = sqrt(kRot[i][i]-kRot[j][j]-kRot[k][k] + 1.0);
		float* apkQuat[3] = { &q.x, &q.y, &q.z };
		*apkQuat[i] = 0.5*fRoot;
		fRoot = 0.5/fRoot;
		q.w = (kRot[k][j]-kRot[j][k])*fRoot;
		*apkQuat[j] = (kRot[j][i]+kRot[i][j])*fRoot;
		*apkQuat[k] = (kRot[k][i]+kRot[i][k])*fRoot;
	}
	return q;
}



float ConvFunc( float v )
{
	return 0;
}

bool ColladaParser::RemoveAnimation( )

{

	FCDAnimationLibrary *anlib  = NULL;
	FCDAnimation		*an		= NULL;
	FCDAnimation		*anim	= NULL;
	FCDAnimationChannel *anchn	= NULL;
	FCDAnimationCurve	*ancur	= NULL;


	
	
	anlib = file->GetAnimationLibrary();
	if (! anlib || (anlib->GetEntityCount()==0))
		return false;

	fm::vector<FMMatrix44List> matList;
	
	int cnt = 0;

	m_pAnimation = new MP4AnimationBuff*[1];

	for ( int i=0; i<anlib->GetEntityCount(); i++)
	{		
		an = anlib->GetEntity ( i );
		{

			anim	= an ; 
			anchn	= anim->GetChannel(0);

			fm::string &tp = anchn->GetTargetPointer();//= data.targetPointer;
			fm::string &tq = anchn->GetTargetQualifier();//data.targetQualifier;
			

			if ( !tp.size() )
				continue;

			FMMatrix44List list;
			FMMatrix44 tmp;
			FMMatrix44List listint;
			FMMatrix44 tmpint;
			FMMatrix44List listoutt;
			FMMatrix44 tmpoutt;
			if ( tp.find( "rotation" ))
			{

			}
			int tmpList = 0;
			if ( tp.find( "transform" ) )
			{


				ancur	= anchn->GetCurve( 0 );
				
				FCDAnimationKey** keys = ancur->GetKeys();
				

				cnt  = ancur->GetKeyCount();
				for ( int k = 0; k < cnt; k++ )
					list.push_back(tmp);

				

				if ( !tq.size() )
				{
					for ( int k =0; k<16; k++ )
					{
						ancur	= anchn->GetCurve( k );
						keys	= ancur->GetKeys();

						for (int l=0; l<ancur->GetKeyCount(); l++)
							list[l][k%4][k/4] = keys[l]->output;
							
						ancur->ConvertValues( &ConvFunc, &ConvFunc );
						ancur->ConvertInputs( NULL, &ConvFunc );
						
					}
				} 
				else
				{
					size_t first  = tq.find_first_of( "(" );
					size_t second = tq.find_last_of( ")" );
					char x = tq.at( first + 1 );
					char y = tq.at( second - 1 );

					int ix = x-'0';
					int iy = y-'0';

					ancur	= anchn->GetCurve( 0 );
					keys	= ancur->GetKeys();

					for (int l=0; l<ancur->GetKeyCount(); l++)
						list[l][iy][ix] = keys[l]->output;
						
					ancur->ConvertValues( &ConvFunc, &ConvFunc );
					ancur->ConvertInputs( NULL, &ConvFunc );

				}
			}

			matList.push_back(list);
			
		}




		
	}
	m_pAnimation[0] = new MP4AnimationBuff( 0, an->GetDaeId().c_str(), cnt );
	m_AnimationCount++;
	MP4Frames *frm = NULL;
	for ( int j=0; j< cnt; j++)
	{
		frm = new MP4Frames();
		frm->m_pbonesData = new MP4BoneData*[matList.size()];
		frm->m_boneDataCount = matList.size();
		for (int k=0; k < matList.size() ; k++)
		{
			FMVector3 sc;
			FMVector3 tr;
			FMVector3 rot;
			FMQuaternion rotQv;
			float	t	= 0 ;
			matList[k][j].Decompose( sc, rot, tr,t);
			rotQv = GetQuaternion( matList[k][j] );

			MP4BoneData *bd = new MP4BoneData ();
			
				Vec3f tmp3;
				tmp3.x =  tr.x;
				tmp3.y =  tr.y;
				tmp3.z =  tr.z;
				bd->SetTranslation( tmp3 ); 

				//tmp3.x = sc.x;
				//tmp3.y = sc.y;
				//tmp3.z = sc.z;
				//bd->SetScale( tmp3 ); 
			
				Vec4f tmp4;
				tmp4.x =  rotQv.x;
				tmp4.y =  rotQv.y;
				tmp4.z =  rotQv.z;
				tmp4.q = rotQv.w;
				bd->SetRotation( tmp4 );
			
			frm->m_pbonesData[k] = bd;

		}

		m_pAnimation[0]->AddMP4Frame( frm );
		
	}
		
	//if (! frm )
	//	return false;
	//int tmpcnt = cnt%16 ;
	//if  (typeBBA == "DCT")
	//	while (  tmpcnt<16 )
	//		{
	//			
	//			m_pAnimation[0]->AddMP4Frame( frm );
	//			tmpcnt++;
	//		}

	//encode bones

	
	return true;

}




int ColladaParser::GetAnimationcount()
{
	return m_AnimationCount;
}
MP4AnimationBuff* ColladaParser::GetAnimation( int id )
{
	if ((id<0)||(id>=m_AnimationCount))
		return NULL;
	return m_pAnimation[id];

}

bool ColladaParser::LoadColladaFile( const char * fileName )
{
	m_pColladaFileName = strdup(fileName);
	ParseColladaFile();
	return true;
}

bool ColladaParser::GetXMLrest ( char* &buff, int &size, bool alloc )
{
	FILE* f = fopen ( m_pRestColladaFileName, "rb" );
	if ( !f )
		return false;
	fseek(f,0, SEEK_END);
	size = ftell(f);
	fseek(f,0,0);
	if ( alloc )
		buff = (char*) malloc ( size );
	fread( buff, 1, size, f );
	fclose ( f );
	return true;
}