#include "Animation_E.h"

Animation_E::Animation_E(void)
{
}

Animation_E::~Animation_E(void)
{
}

bool Animation_E::LoadInterpolators_3DMC( std::vector<MeshFrame> &dmcMesh  )
{

	FreeMem();

	std::vector<float> coord;
	std::vector<float> color;
	std::vector<float> normal;
	std::vector<float> attributes;
	std::vector<float> coordKey;
	std::vector<float> colorKey;
	std::vector<float> normalKey;
	std::vector<float> attributesKey;

	int i=1;
	for (i=1; i<dmcMesh.size();i++)
	{
		coordKey.push_back(dmcMesh[i].time - dmcMesh[i-1].time );
		normalKey.push_back(dmcMesh[i].time - dmcMesh[i-1].time );
		colorKey.push_back(dmcMesh[i].time - dmcMesh[i-1].time );
		
		MP43DMC* mesh = dmcMesh[i].mesh;
		
		//get coordinates
		for ( int j=0; j<mesh->GetCoordinateCount(); j++)
			coord.push_back( mesh->GetCoordinate()[j]);
		
		//get normals
		//for ( int j=0; j<mesh->GetNormalCount(); j++)
		//	normal.push_back( mesh->GetNormal()[j]);
		
		//get colors
		//for ( int j=0; j<mesh->GetColorCount(); j++)
		//	color.push_back( mesh->GetColor()[j]);
		
	}

		// we get the animation duration
		duration_ = (double) dmcMesh[i-1].time + dmcMesh[i-1].time - dmcMesh[i-2].time  ;
		
	nKeyCoord_ = (int) coordKey.size();
	nKeyColor_ = (int) colorKey.size();
	nKeyNormal_ = (int) normalKey.size();
	nKeyAttributes_ = (int) attributes.size();

	if (nKeyCoord_ != 0) nCoord_ = (int) coord.size()/(3 * nKeyCoord_);
	if (nKeyNormal_ != 0) nNormal_ = (int) normal.size()/(3 * nKeyNormal_);
	if (nKeyColor_ != 0) nColor_ = (int) color.size()/(3 * nKeyColor_);
	if (nKeyAttributes_ != 0) nAttributes_ =	(int) attributes.size()/nKeyAttributes_;

	// we fill coord
	if ( nKeyCoord_ > 0) {
		
		coord_ = new float * [nKeyCoord_];
		coordKey_ = new float [nKeyCoord_];
		int k=0;
		int i=0;
		for (i = 0; i < nKeyCoord_; i++) {
			coord_[i] = new float [3*nCoord_];
			coordKey_[i] = coordKey[i];
			for (int p = 0; p < nCoord_; p++) {
				for(int h = 0; h < 3; h++) {
					coord_[i][p*3+h] = coord[k];
					k++;
				}
			}
		}
	
	}
	// we fill normal
	if ( nKeyNormal_ > 0) {
		
		normal_ = new float * [nKeyNormal_];
		normalKey_ = new float [nKeyNormal_];
		int k=0;
		int i=0;
		for (i = 0; i < nKeyNormal_; i++) {
			normal_[i] = new float [3*nNormal_];
			normalKey_[i] = normalKey[i];
			for (int p = 0; p < nNormal_; p++) {
				for(int h = 0; h < 3; h++) {
					normal_[i][p*3+h] = normal[k];
					k++;
				}
			}
		}
		
	}
	if ( nKeyColor_ > 0) {
		
		color_ = new float * [nKeyColor_];
		colorKey_ = new float [nKeyColor_];
		int k=0;
		int i=0;
		for (i = 0; i < nKeyColor_; i++) {
			color_[i] = new float [3*nColor_];
			colorKey_[i] = colorKey[i];
			for (int p = 0; p < nColor_; p++) {
				for(int h = 0; h < 3; h++) {
					color_[i][p*3+h] = color[k];
					k++;
				}
			}
		}
		
	}
	if ( nKeyAttributes_ > 0) {
		
		attributes_ = new float * [nKeyAttributes_];
		attributesKey_ = new float [nKeyAttributes_];
		int k=0;
		int i=0;
		for (i = 0; i < nKeyAttributes_; i++) {
			attributes_[i] = new float [3*nAttributes_];
			attributesKey_[i] = attributesKey[i];
			for (int p = 0; p < nAttributes_; p++) {
				for(int h = 0; h < 3; h++) {
					attributes_[i][p*3+h] = attributes[k];
					k++;
				}
			}
		}
		
	}
	return true;
}

bool Animation_E::SaveInterpolators_3DMC( std::vector<MeshFrame> &dmcMesh )
{
	//FreeMem();

	MeshFrame msh;
	for ( int i = 0; i < nKeyCoord_; i++ )
	{
		msh.time = dmcMesh[i].time + coordKey_[i];
		dmcMesh.push_back(msh);
	}

	for ( int i = 0; i < nKeyCoord_; i++ )
	{
		dmcMesh[i+1].mesh = new MP43DMC(nCoord_, nNormal_, 0, nColor_, 0, 0, 0, 0, dmcMesh[0].mesh->Get3DMCID(), dmcMesh[0].mesh->Get3DMCFileID() );
		for ( int j = 0; j < nCoord_; j++ )
			dmcMesh[i+1].mesh->AddCoordinate( coord_[i][j*3+0], coord_[i][j*3+1], coord_[i][j*3+2] );
		for ( int j = 0; j < nNormal_; j++ )
			dmcMesh[i+1].mesh->AddNormal( normal_[i][j*3+0], normal_[i][j*3+1], normal_[i][j*3+2] );
		for ( int j = 0; j < nColor_; j++ )
			dmcMesh[i+1].mesh->AddColor( color_[i][j*3+0], color_[i][j*3+1], color_[i][j*3+2] );
	}

	return true;
}