#include "FAMCDecoder_E.h"
#include "IndexedFaceSet_E.h"
#include "Animation_E.h"
#include "LDSegmentDecoder.h"

//#define PROFILING_FAMC

FAMCDecoder_E::FAMCDecoder_E(void)
{
}

FAMCDecoder_E::~FAMCDecoder_E(void)
{
}

bool FAMCDecoder_E::DecodeStream( char * file, std::vector<MeshFrame> &meshVec, int numberOfSpatialLayers/*=-1 */ )
{
	Animation_E myAnimDec;
	char fin[1024];
	sprintf(fin, "%s.mp4", file);

	//std::vector<MeshFrame> meshVec;
	//MeshFrame mesh;
	//mesh.mesh = dmc;
	//mesh.time=0;
	//meshVec.push_back( mesh );

	// we save the first frame
	//char fFirstFrame[1024];
	//sprintf(fFirstFrame, "%sFF.wrl", file);
	IndexedFaceSet_E firstFrame;
	firstFrame.Load_IFS_3DMC( meshVec );
	//IndexedFaceSet firstFrame;
	//firstFrame.LoadIFSVRML2( fFirstFrame);

	FAMCDecoder decoder;
	FILE * in = fopen(fin, "rb");
	bool stillAU = true;
	std::vector<float> keys;
	std::vector<float> coord;
	std::vector<float> normal;
	std::vector<float> color;

	int s = 0;
	while (stillAU) {
		Animation animSeg;
		stillAU = decoder.DecodeAU(in, animSeg, firstFrame);
		if (stillAU) {	
			animSeg.GetAnimationData(keys, coord, normal, color, 1);
		
#ifdef PROFILING_FAMC
			IndexedFaceSet_E ifs;
			ifs.Load_IFS_3DMC( meshVec );
			ifs.Load_IFS_Vectors( coord, normal, color );
			char fileName[255];
			sprintf(fileName, "Frame_out_%02d.wrl", s);
			ifs.SaveIFSVRML2(fileName);
			s++;
#endif
		}
	}
	fclose(in);

	// update keys
	for (int f = 0; f < (int) keys.size(); f++) {
		keys[f] = (float) f /(keys.size()-1);
	}

	int numberOfDecomposedLayers = decoder.GetNumberOfDecomposedLayers();

	myAnimDec.CreateAnimation(keys, coord, normal, color);
	if (numberOfSpatialLayers>0 && numberOfSpatialLayers<numberOfDecomposedLayers) {
		printf("<========== LOD start ==========>\n");
		// get layered decomposition
		LayeredDecomposition ld;
		LDSegmentDecoder::createLayeredDecompositionFromConn(firstFrame, numberOfDecomposedLayers, ld);

		printf("writing decimated connectivity: layer %d of %d\n", numberOfSpatialLayers, numberOfDecomposedLayers);
		// we copy the decimated IFS
		int coordIndex[3] = {-1, -1, -1};
		coordIndex[0]= -1;
		coordIndex[1]= -1;
		coordIndex[2]= -1;
		int T = firstFrame.GetNCoordIndex(); // number of triangles
		for(int t = 0; t < T; t++) {
			firstFrame.SetCoordIndex(t, coordIndex);
		}
		int Td = (int) ld.layerIFS[numberOfSpatialLayers-1].size();
		for(int t = 0; t < Td; t++) {
			for (int k = 0; k < 3; k++) {
				coordIndex[k] = ld.layerIFS[numberOfSpatialLayers-1][t][k];
			}
			firstFrame.SetCoordIndex(t, coordIndex);
		}
		printf("<========== LOD end ============>\n");
		
	}

	myAnimDec.CreateAnimation(keys, coord, normal, color);

	//char fAnimOut[1024];
	//sprintf(fAnimOut, "%s_dec.wrl", file);
	//myAnimDec.SaveInterpolatorVRML2(fAnimOut, firstFrame);
	myAnimDec.SaveInterpolators_3DMC(meshVec);

	// added to output the animation as a sequence of IFSs
//	char fileOutM[1024];
//	for (int frame = 0; frame < myAnimDec.GetNKeyCoord(); frame++) {
//		sprintf(fileOutM, "%s_%i_dec.wrl", file, frame);
//		myAnimDec.SaveFrame(fileOutM, firstFrame, frame);
//	}
	return true;
}


