/*!
 ************************************************************************
 *  \file
 *     Graph.cpp
 *  \brief
 *     Graph class.
 *  \author
 *     Copyright (C) 2007  ARTEMIS Department INT/GET, Paris, France.
 *
 *     Khaled MAMOU               <khaled.mamou@int-evry.fr>
 *
 *	   Institut National des Telecommunications		tel.: +33 (0)1 60 76 40 94
 *     9, Rue Charles Fourier,						fax.: +33 (0)1 60 76 43 81
 *     91011 Evry Cedex France
 *
 ************************************************************************
 */

#include "Graph.h"
#include <math.h>


extern CConsole logger;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Graph::Graph()
{
	verticesNames_ = 0;
	edgesNames_ = 0;
	nbrCCs_ = 0;
}

Graph::~Graph()
{

}

/*************/
int Graph::AddVertex()
{
	int name = verticesNames_;
	vertices_[name].name_ = name;
	verticesNames_++;
	return name;
}

int Graph::AddVertex(gVertex &vertex)
{
	vertices_[vertex.name_] = vertex;
	return vertex.name_;
}

/*************/
int Graph::AddEdge(int v1, int v2)
{
	if (IsEdge(v1, v2) == 0) {
		//if ( v1 >= vertices_.size() || v2 >= vertices_.size() )
		//	return -1;
		int name = edgesNames_;
		edges_[name].name_ = name;
		edges_[name].v1_ = v1;
		edges_[name].v2_ = v2;
		edgesNames_++;
		vertices_[v1].AddEdge(name);
		vertices_[v2].AddEdge(name);
		return name;	
	}	
	else {
		return -1;
	}
}
/*************/
int Graph::DelEdge(int name)
{
	
	gEdgeMap::iterator it = edges_.find(name);
	if (  it != edges_.end()) {
		int v1 = (it->second).v1_;
		int v2 = (it->second).v2_;
		edges_.erase(it);
		vertices_[v1].DelEdge(name);
		vertices_[v2].DelEdge(name);
		return 0;
	}
	return -1;
}


/********************/

void Graph::Info()
{
	logger.write_2_log("\nVerticesNbr  = %i, EdgesNbr = %i \n", vertices_.size(), edges_.size());

	logger.write_2_log("Vertices\n");
	for (gVertexMap::iterator pos = vertices_.begin(); pos != vertices_.end(); ++pos) {
        (pos->second).Info();
		logger.write_2_log("\n");
    }

	logger.write_2_log("Edges\n");
	for (gEdgeMap::iterator pose = edges_.begin(); pose != edges_.end(); ++pose) {
        (pose->second).Info();
		logger.write_2_log("\n");
    }

}

void Graph::EdgeCollapse(int v1, int v2){

	if (IsEdge(v1, v2) == 1) {
		vertices_[v1].ancestors_.push_back(v2);
		for (std::vector<int>::iterator pos1 = vertices_[v2].ancestors_.begin(); pos1 != vertices_[v2].ancestors_.end(); ++pos1) {
			vertices_[v1].ancestors_.push_back(*pos1);
	    }

		std::vector<int> incident;
		for(std::vector<int>::iterator ed = vertices_[v1].edges_.begin(); ed != vertices_[v1].edges_.end(); ed++) {
			edges_[*ed].tag_ = 1;
		}

		std::vector<int> delEdge;
		std::vector<int> addEdge;

		int addedOnes = 0;
		for(std::vector<int>::iterator ed = vertices_[v2].edges_.begin(); ed != vertices_[v2].edges_.end(); ed++) {
			edges_[*ed].tag_ = 1;
			delEdge.push_back(*ed);

			int ev1 = edges_[*ed].v1_;
			int ev2 = edges_[*ed].v2_;


			if ( ev1 == v2) {
				ev1 = v1;
			}
			if ( ev2 == v2) {
				ev2 = v1;
			}

			if ( (ev1 == ev2) || (IsEdge(ev1, ev2) == 1) ){					
			}
			else {
				addEdge.push_back(ev1);
				addEdge.push_back(ev2);
				addedOnes++;
			}	
		}

		for (std::vector<int>::iterator it = addEdge.begin(); it != addEdge.end(); it++) {
			int a = *it;
			it++;
			int b = *it;
			int ed = AddEdge(a, b);
			edges_[ed].tag_ = 1;
		}
		

		int delsize = (int) delEdge.size() ;
		for (std::vector<int>::iterator it = delEdge.begin(); it != delEdge.end(); it++) {

			if (  delEdge.size() - addedOnes != 3) {
				int o = 5;
			}
			DelEdge(*it);
		}
		vertices_.erase(vertices_.find(v2));
	}
	else {
		logger.write_2_log("edge (%i, %i) not found", v1, v2);
	}
}

int Graph::IsEdge(int v1, int v2) {
	if ((vertices_.find(v1) == vertices_.end()) ||
		(vertices_.find(v2) == vertices_.end())) {
		return 0;
	}
	
	for(std::vector<int>::iterator ed = vertices_[v1].edges_.begin(); ed != vertices_[v1].edges_.end(); ed++) {
		if ( ((edges_[*ed].v1_ == v1) && (edges_[*ed].v2_ == v2)) ||
			((edges_[*ed].v1_ == v2) && (edges_[*ed].v2_ == v1)) ) {
			return 1;
		}
	}

	return 0;
}

int Graph::ExtractCCs(){
	// all CCs to -1
	for (gVertexMap::iterator pos = vertices_.begin(); pos != vertices_.end(); ++pos) {
        (pos->second).cc_ = -1;
    }

	// we get the CCs
	nbrCCs_ = 0;
	for (gVertexMap::iterator pos = vertices_.begin(); pos != vertices_.end(); ++pos) {
		if ((pos->second).cc_ == -1) {
			(pos->second).cc_ = nbrCCs_;
			std::vector<int> temp;
			temp.push_back((pos->second).name_);

			while ( temp.size() != 0 ) {
				int vertex = temp[temp.size()-1];
				temp.pop_back();

				for(std::vector<int>::iterator ed = vertices_[vertex].edges_.begin(); ed != vertices_[vertex].edges_.end(); ed++) {
					int vertex2 = vertex;
					if (edges_[*ed].v1_ == vertex) {
						vertex2 = edges_[*ed].v2_;
					}
					else {
						vertex2 = edges_[*ed].v1_;
					}
					if ( vertices_[vertex2].cc_ == -1 ) {
						vertices_[vertex2].cc_ = nbrCCs_;
						temp.push_back(vertex2);
					}
				}
			}
			nbrCCs_++;
		}
    }

	return nbrCCs_;
}

void Graph::ConnectCCs(){
	ExtractCCs();
	std::map<int, int> mapp;
	for (gVertexMap::iterator pos = vertices_.begin(); pos != vertices_.end(); ++pos) {
		int cluster = (pos->second).cc_;
		int vertex = (pos->second).name_;

		if (mapp.find(cluster) == mapp.end()) {
			mapp[cluster] = vertex;
//			logger.write_2_log("v%i cc%i\n", vertex, cluster);
		}
	}

	for (int i = 0; i < nbrCCs_; i++) {
		for (int j = 0; j < nbrCCs_; j++) {
			if (i > j) {
				int name = AddEdge(mapp[i], mapp[j]);
//				logger.write_2_log("%i (%i, %i) %i\n", name, mapp[i], mapp[j], edges_.size());
			}
		}
	}

}