/*!
 ************************************************************************
 *  \file
 *     Graph.h
 *  \brief
 *     Graph class.
 *  \author
 *     Copyright (C) 2007  ARTEMIS Department INT/GET, Paris, France.
 *
 *     Khaled MAMOU               <khaled.mamou@int-evry.fr>
 *
 *	   Institut National des Telecommunications		tel.: +33 (0)1 60 76 40 94
 *     9, Rue Charles Fourier,						fax.: +33 (0)1 60 76 43 81
 *     91011 Evry Cedex France
 *
 ************************************************************************
 */
#pragma once
#include <map>
#include <vector>
#include "gEdge.h"
#include "gVertex.h"
#include "Console.h"


typedef std::map<int, gEdge> gEdgeMap;
typedef std::map<int, gVertex> gVertexMap;
typedef std::map<int, int> gIntMap;

class Graph  
{
public:
	int verticesNames_;
	int edgesNames_;
	int nbrCCs_;
	gEdgeMap edges_;
	gVertexMap vertices_;

	void EdgeCollapse(int v1, int v2);
	int AddVertex();
	int AddVertex(gVertex &vertex);
	int AddEdge(int v1, int v2);
	int DelEdge(int name);	
	int IsEdge(int v1, int v2);
	int ExtractCCs();
	void ConnectCCs();

	void Info();

	Graph();
	virtual ~Graph();

};