/******************************************************************************\
* Includes
\******************************************************************************/
#include "DecodeMP4Streams.h"

#include "BBADecoderLib.h"
#include "mpg_ifsq_E_ifs.h"
#include "zlib.h"
#include "jasper/jasper.h"
#include "FAMCDecoder_E.h"
#include "MP43DMC.h"

/******************************************************************************\
 DecodeMP4Streams.cpp: implementation of the DecodeMP4Streams class
\******************************************************************************/



/******************************************************************************\
* Class DecodeMP4Streams
\******************************************************************************/


DecodeMP4Streams::DecodeMP4Streams(void)
{
	m_panimation = NULL;
	m_p3DMC		 = NULL;
	m_ptextBuff  = NULL;
	m_textBuffSize = 0;
}


DecodeMP4Streams::~DecodeMP4Streams(void)
{
	delete m_panimation;
	delete m_p3DMC;
	if (m_ptextBuff)
		free (m_ptextBuff);
}


void DecodeMP4Streams::Decode3DMCStream( char* DMCbuff, int DMCbuffSize )
{
	MPG_IfsQ_E_Ifs ifs;
	MPG_InputStream istrm;
	istrm.open( DMCbuffSize, (unsigned char*)DMCbuff );
	ifs.expand(istrm,false);
	istrm.close();
	ifs.apply3DMCExtension();
	m_p3DMC = new MP43DMC(ifs.getCoord().size()/3,ifs.getNormal().size()/3,ifs.getTexCoord().size()/2,ifs.getColor().size()/3,ifs.getCoordIndex().size()/4,ifs.getNormalIndex().size()/4,ifs.getTexCoordIndex().size()/4,ifs.getColorIndex().size()/4);

	for (int i=0;i<ifs.getCoord().size();i+=3)
		m_p3DMC->AddCoordinate(ifs.getCoord()[i+0],ifs.getCoord()[i+1],ifs.getCoord()[i+2]);
	for (int i=0;i<ifs.getNormal().size();i+=3)
		m_p3DMC->AddNormal(ifs.getNormal()[i+0],ifs.getNormal()[i+1],ifs.getNormal()[i+2]);
	for (int i=0;i<ifs.getTexCoord().size();i+=2)
		m_p3DMC->AddTextureCoordinate(ifs.getTexCoord()[i+0],ifs.getTexCoord()[i+1]);
	for (int i=0;i<ifs.getColor().size();i+=3)
		m_p3DMC->AddColor(ifs.getColor()[i+0],ifs.getColor()[i+1],ifs.getColor()[i+2]);
	for (int i=0;i<ifs.getCoordIndex().size();i+=4)
		m_p3DMC->AddCoordinateIndex(ifs.getCoordIndex()[i+0],ifs.getCoordIndex()[i+1],ifs.getCoordIndex()[i+2]);
	for (int i=0;i<ifs.getNormalIndex().size();i+=4)
		m_p3DMC->AddNormalIndex(ifs.getNormalIndex()[i+0],ifs.getNormalIndex()[i+1],ifs.getNormalIndex()[i+2]);
	for (int i=0;i<ifs.getTexCoordIndex().size();i+=4)
		m_p3DMC->AddTextureCoordinateIndex(ifs.getTexCoordIndex()[i+0],ifs.getTexCoordIndex()[i+1],ifs.getTexCoordIndex()[i+2]);
	for (int i=0;i<ifs.getColorIndex().size();i+=4)
		m_p3DMC->AddColorIndex(ifs.getColorIndex()[i+0],ifs.getColorIndex()[i+1],ifs.getColorIndex()[i+2]);
}
	
void DecodeMP4Streams::DecodeBBAStream( unsigned char* Animbuff, int AnimbuffSize )
{
	m_panimation = new MP4AnimationBuff();
	MP4Frames *frame = NULL;
	MP4BoneData *bd  = NULL;

	//decode a stream
	FILE  *fbba;
	int sc=0;
	int fr=0;
	
	BBADecoderLib bbadec;
	bbadec.init();
	bbadec.initStream( Animbuff, AnimbuffSize);
	bbadec.all_Intra = bbadec.bba_allIntra();
	bbadec.initbits();
	sc = bbadec.getbits(32);
	if (sc!=BBA_OBJECT_START_CODE)
		exit (1);
	do {
		if (bbadec.decode_bba_object_plane_header())
			break;
		bbadec.decode_bba_object_plane_data();
		int cnt = 16;
		if ( bbadec.coding_type == PREDICTIVE_CODING_TYPE )
			cnt = 1;
		for ( int i = 0; i < cnt ; i ++ )
		{
			frame = new MP4Frames();
			frame->Init(bbadec.bbaop.NumberOfBones);
			CBBABone *q;
			q = bbadec.bbaop.firstBone;
			while (q){
				bd = new MP4BoneData();
				Vec3f tmp3;
				Vec4f tmp4;
				if (q->boneMask.isConXChanged || q->boneMask.isConYChanged || q->boneMask.isConZChanged )
				{
					if ( cnt == 1 )
					{
						tmp3.x = q->boneData.ConX*0.01f;
						tmp3.y = q->boneData.ConY*0.01f;
						tmp3.z = q->boneData.ConZ*0.01f;
					}
					else
					{
						tmp3.x = q->boneDCTData[i].ConX*0.01f;
						tmp3.y = q->boneDCTData[i].ConY*0.01f;
						tmp3.z = q->boneDCTData[i].ConZ*0.01f;
					}
					bd->SetTranslation( tmp3 );
				}
				if (q->boneMask.isRonQwChanged || q->boneMask.isRonQxChanged || q->boneMask.isRonQyChanged || q->boneMask.isRonQzChanged )
				{
					if ( cnt == 1 )
					{
						tmp4.x = q->boneData.RonQx*0.00001f;
						tmp4.y = q->boneData.RonQy*0.00001f;
						tmp4.z = q->boneData.RonQz*0.00001f;
						tmp4.q = q->boneData.RonQw*0.00001f;
					}
					else
					{
						tmp4.x = q->boneDCTData[i].RonQx*0.00001f;
						tmp4.y = q->boneDCTData[i].RonQy*0.00001f;
						tmp4.z = q->boneDCTData[i].RonQz*0.00001f;
						tmp4.q = q->boneDCTData[i].RonQw*0.00001f;
					}
					bd->SetRotation( tmp4 );
				}
				if (q->boneMask.isSonXChanged || q->boneMask.isSonYChanged || q->boneMask.isSonZChanged  )
				{
					if ( cnt == 1 )
					{
						tmp3.x = q->boneData.SonX * 0.0001;
						tmp3.y = q->boneData.SonY * 0.0001;
						tmp3.z = q->boneData.SonZ * 0.0001;
					}
					else
					{
						tmp3.x = q->boneDCTData[i].SonX * 0.0001;
						tmp3.y = q->boneDCTData[i].SonY * 0.0001;
						tmp3.z = q->boneDCTData[i].SonZ * 0.0001;
					}
					bd->SetScale( tmp3 );
				}
				if (q->boneMask.isSOonAxXChanged || q->boneMask.isSOonAxYChanged || q->boneMask.isSOonAxZChanged || q->boneMask.isSOonValChanged )
				{
					if ( cnt == 1 )
					{
						tmp4.x = q->boneData.SOonAxX;
						tmp4.y = q->boneData.SOonAxY;
						tmp4.z = q->boneData.SOonAxZ;
						tmp4.q = q->boneData.SOonVal;
					}
					else
					{
						tmp4.x = q->boneDCTData[i].SOonAxX;
						tmp4.y = q->boneDCTData[i].SOonAxY;
						tmp4.z = q->boneDCTData[i].SOonAxZ;
						tmp4.q = q->boneDCTData[i].SOonVal;
					}
					bd->SetscaleOrienttion( tmp4 );
				}
				if (q->boneMask.isTonXChanged || q->boneMask.isTonYChanged || q->boneMask.isTonZChanged  )
				{
					if ( cnt == 1 )
					{
						tmp3.x = q->boneData.TonX*0.01;
						tmp3.y = q->boneData.TonY*0.01;
						tmp3.z = q->boneData.TonZ*0.01;
					}
					else
					{
						tmp3.x = q->boneDCTData[i].TonX*0.01;
						tmp3.y = q->boneDCTData[i].TonY*0.01;
						tmp3.z = q->boneDCTData[i].TonZ*0.01;
					}
					bd->SetCenter( tmp3 );
				}
				frame->AddMP4BoneData( bd );
				q=q->suiv;
			}
			
			fr++;
			m_panimation->AddMP4Frame( frame );
		}
	}
	while(bbadec.nextbits_bytealigned()!=BBA_OBJECT_END_CODE);
	bbadec.FreeData();
	

}

void DecodeMP4Streams::DecodeFAMCStream( char* FAMCbuff, int FAMCbuffSize, std::vector<MeshFrame> &meshVec )
{
		FILE* f = fopen( "temp.mp4", "wb" );
		fwrite( FAMCbuff, 1 , FAMCbuffSize, f );
		fclose( f );

		int numberOfSpatialLayers = -1;

		FAMCDecoder_E decoder;
		decoder.DecodeStream( "temp", meshVec, -1 );

		remove( "temp.mp4" );
}

void DecodeMP4Streams::DecodeXMLStream( unsigned char* buff, int buffSize )
{
	FILE *gz;
	gz = fopen("rarScene.gz","wb" );
	fwrite(  buff, buffSize,1,gz );
	fclose( gz );

}
void DecodeMP4Streams::GetDecodedXML(unsigned char* &buff, int &buffSize, bool alloc)
{
	int tmpsize = 10000;
	int i = 0;
	gzFile gz;
	gz = gzopen( "rarScene.gz", "rb");
	buff = ( unsigned char* ) malloc( tmpsize );
	int t = gzread( gz, buff, tmpsize );
	while (t==tmpsize  )
	{
		i++;
		buff = (unsigned char*) realloc( buff, (i+1)*tmpsize );
		t=gzread( gz, (unsigned char*)&buff[i*tmpsize], tmpsize );
	};
	
	buffSize = i* tmpsize + t + 1;
	buff[buffSize-1] = '\0';
	gzclose( gz );
	 remove ( "rarScene.gz" );

}
int DecodeMP4Streams::DecodeTextureStream( char* textbuff, int textbuffSize )
{
	jas_stream_t *in, *out;

	if (jas_init()) {
		return -1;
	}

	in = jas_stream_memopen( (char*)textbuff, textbuffSize );
	
	jas_image_t *image;

	int infmt =jas_image_getfmt(in);
	char *inopts = NULL;
	image = jas_image_decode(in, infmt, inopts);
	jas_stream_close(in);

	int numcmpts = jas_image_numcmpts(image);

	{
		jas_image_t *newimage;
		jas_cmprof_t *outprof;
		outprof = jas_cmprof_createfromclrspc(JAS_CLRSPC_SRGB);
		newimage = jas_image_chclrspc(image, outprof, JAS_CMXFORM_INTENT_PER);
		jas_image_destroy(image);
		jas_cmprof_destroy(outprof);
		image = newimage;
	}

	out = jas_stream_memopen( NULL, 0 );

	int outfmt = jas_image_strtofmt( "bmp" );
	char outopts = '\0';
	jas_image_encode( image, out, outfmt, &outopts );
	jas_stream_flush(out);

	m_textBuffSize = jas_stream_length( out );
	m_ptextBuff = ( unsigned char* ) malloc( m_textBuffSize );
	jas_stream_rewind( out );
	jas_stream_read( out, m_ptextBuff, m_textBuffSize );

	jas_stream_close(out);
	

	jas_image_destroy(image);
	jas_cleanup();


}


void DecodeMP4Streams::GetDecodedTexture( unsigned char* &buff, int &buffSize, bool alloc )
{
	if (alloc)
		buff = (unsigned char*) malloc (m_textBuffSize);
	memcpy( buff, m_ptextBuff, m_textBuffSize );
	buffSize = m_textBuffSize;

}