#ifndef _ENCODEMP4STREAMS_H_
#define _ENCODEMP4STREAMS_H_

/******************************************************************************\
* Includes
\******************************************************************************/


#include <stdio.h>
#include <tchar.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <vector>


#include "MP43DMC.h"
#include "MP4Animation.h"
#include "CMDArguments.h"


/******************************************************************************\
* Defines
\******************************************************************************/
// maximum size of BBA buffer
#define	BBABUFERSIZE 10000000


/******************************************************************************\
* Class EncodeMP4Streams
\******************************************************************************/
class EncodeMP4Streams
{
public:

	EncodeMP4Streams( CMDArguments* cmd );
	~EncodeMP4Streams(void);

	//methods
	
	//encode geometry in 3DMC format
	//DMC is object from MP43DMC class that should be encoded
	void Encode3DMCStream( MP43DMC* DMC );
	//return encoded 3DMC buffer and it's size
	// If alloc is true, the buffer is allocated inside the function
	// If alloc is false, the buffer should be allocated before calling the function
	void GetEncoded3DMC(  char* &DMCbuff, int &DMCbuffSize, bool alloc );
	//return the id (order number) of the 3dmc buffer
	int Get3DMCID() {return m_p3DMC->Get3DMCID();}
	//return the dae id for this mesh
	const char* Get3DMCFileID() { return m_p3DMC->Get3DMCFileID();}
	//encode animation in BBA format
	//animation is object from MP4animation class that should be encoded
	void EncodeBBAStream( MP4AnimationBuff * animation );
	//return encoded BBA buffer and it's size
	// If alloc is true, the buffer is allocated inside the function
	// If alloc is false, the buffer should be allocated before calling the function
	void GetEncodedBBA( unsigned char* &AnimbuffOUT, int &AnimbuffSizeOUT, bool alloc );
	//return the id (order number) of the bba buffer
	int GetAnimID() {return m_panimation->GetAnimationID();}
	//return the dae id for this animation ( it's the id of the last entity  )
	const char* GetAnimFileID() { return m_panimation->GetAnimationFileID();}
	//encode file with image in JPEG2000 format
	//file_name is the name of the file should be encoded
	int EncodeTextureStream( char* file_name);
	//return encoded JPEG2000 buffer and it's size
	// If alloc is true, the buffer is allocated inside the function
	// If alloc is false, the buffer should be allocated before calling the function
	bool GetEncodedTexture( char* &buff, int &size, bool alloc );
	//encode xml data in gzip format
	void EncodeXMLData( char* buff, int size );
	//return encoded XML buffer and it's size
	// If alloc is true, the buffer is allocated inside the function
	// If alloc is false, the buffer should be allocated before calling the function
	bool GetEncodedXml( char* &buff, int &size, bool alloc );

	//encode animation in FAMC format
	//dmcMesh is list of object from MP43DMC class that should be encoded
	//the size of the dmcMesh is the  number of frames
	void EncodeFAMCStream( std::vector<MeshFrame> &dmcMesh );
	//return encoded FAMC buffer and it's size
	// If alloc is true, the buffer is allocated inside the function
	// If alloc is false, the buffer should be allocated before calling the function
	void GetEncodedFAMC(  char* &FAMCbuff, int &FAMCbuffSize, bool alloc );
	//return the id (order number) of the FAMC buffer
	int GetFAMCID() {return (*m_pFAMC)[0].mesh->Get3DMCID();}
	//return the dae id for this mesh
	//for all the frames, there is the same dae id for one mesh
	const char* GetFAMCFileID() { return (*m_pFAMC)[0].mesh->Get3DMCFileID();}
		//char* tmp = (char*) malloc ( strlen(((*m_pFAMC)[0].mesh->Get3DMCFileID())+ 5 ));
		//sprintf((char*)tmp,"%sfamc",(*m_pFAMC)[0].mesh->Get3DMCFileID());
		//return ( tmp );}


	//functions for dump in files
	void Get3DMCFile ( const char* fileName );
	void GetBBAFile ( const char* fileName );
	void GetFAMCFile ( const char* fileName );

	void EncodeCIStream( MP4AnimationBuff * animation, int key_qp, int keyvalue_qp); 
	void EncodePIStream( MP4AnimationBuff * animation, int key_qp, int keyvalue_qp); 
	void EncodeOIStream( MP4AnimationBuff * animation, int key_qp, int keyvalue_qp); 
	void GetEncodedCI( unsigned char* &AnimbuffOUT, int &AnimbuffSizeOUT, bool alloc ); 
	void GetEncodedPI( unsigned char* &AnimbuffOUT, int &AnimbuffSizeOUT, bool alloc ); 
	void GetEncodedOI( unsigned char* &AnimbuffOUT, int &AnimbuffSizeOUT, bool alloc ); 


private:

	//variables coding parametars
	CMDArguments* m_params;

	/*int m_dmc_bitsPerCoord ;
	int m_dmc_bitsPerNormal  ;
	int m_dmc_bitsPerColor ;
	int m_dmc_bitsPerTexCoord   ;
	int m_dmc_img_flag;
	int m_dmc_img_u ;
	int m_dmc_img_v ;
	int m_dmc_coordPrediction  ;
	int m_dmc_normalPrediction ;
	int m_dmc_colorPrediction ;
	int m_dmc_texCoordPrediction ;
	int m_dmc_erMode;   
	int m_dmc_packetSize;
	int m_dmc_boundaryPrediction;
	bool m_dmc_hierarchicalMode ;
	bool m_dmc_saveClsFile    ;
	bool m_dmc_saveWrlLevels  ;
	bool m_dmc_vertex_order_flag ;
	bool m_dmc_face_order_flag  ;
	bool m_dmc_face_order_per_CC_flag ;
	bool m_dmc_vertex_order_per_CC_flag ;

	int m_bba_automatic;
	int m_bba_P_frames;
	int m_bba_coding_type;
	int m_bba_p2q;
	int m_bba_qs;
	
	int m_famc_sizeAnimSeg;
	int m_famc_transformType;
	double m_famc_globalRMSE;
	double m_famc_ratioWeights;
	int m_famc_qAT;
	int m_famc_qCoord;
	int m_famc_qNormal;
	int m_famc_qColor;
	int m_famc_qOther;
	int m_famc_LD_auto;
	int m_famc_LD_L;
	int m_famc_LD_l;
	int m_famc_LD_P;
	int m_famc_LD_B;
	int m_famc_LD_Pred;
	int m_famc_SNR_L;
	int m_famc_SNR_l;
	int m_famc_Normal_Pred;*/
	
	//variables
	bool Encode_OK;
	MP4AnimationBuff * m_panimation;
	MP43DMC* m_p3DMC;
	std::vector<MeshFrame>* m_pFAMC;
	char* m_pAnimbuff;
	int m_AnimbuffSize;
	unsigned char* m_pAnimbuffOUT;
	int m_AnimbuffSizeOUT;
	int m_AnimID;
	const char* m_pAnimFileId;
	char* m_p3DMCbuff;
	int m_3DMCbuffSize;
	int m_3DMCID;
	const char* m_p3DMCFileId;
	char* m_pBinBuff;
	int m_BinBuffSize;
	char* m_pjpgBuff;
	int m_jpgBuffSize;
	char* m_pFAMCbuff;
	int m_FAMCbuffSize;
	int m_FAMCID;
	const char* m_pFAMCFileId;

	//methods
	void MakeTextualBBAHeader();
	void MakeTextualBBAFrame( MP4Frames* frame);
	bool MakeTextualBBA();
	bool encodeBBA();
	void read3DMCData();

	//CI
	bool encodeCI(int key_qp, int keyvalue_qp); 
	bool encodePI(int key_qp, int keyvalue_qp); 
	bool encodeOI(int key_qp, int keyvalue_qp); 
	//CI
};




#endif //_ENCODEMP4STREAMS_H_


