/******************************************************************************\
* Includes.
\******************************************************************************/

#include "MP4Mux.h"
/******************************************************************************\
 MP4Mux.cpp: implementation of the MP4Mux class:
\******************************************************************************/



/******************************************************************************\
* Class MP4Mux
\******************************************************************************/

MP4Mux::MP4Mux()
{
	m_TrackList = gf_list_new();
	m_trackID	= 0;
	m_hasBinXml = false;
}

MP4Mux::~MP4Mux()
{
	for ( unsigned int i = 0; i < gf_list_count( m_TrackList ); i++ )
	{
		TrackData* data = (TrackData*) gf_list_get( m_TrackList, i );
		delete data;
	}
	gf_list_del( m_TrackList );
}

int MP4Mux::Set3DMC(  char *buff, int buffSize, int ID, const char* fileId )
{
	TrackData* data;
	data = new TrackData();
	data->trackID				= ++m_trackID;
	data->trackNo				= 0;
	data->buff					=( char* ) malloc ( buffSize * sizeof (char));
	memcpy( data->buff, buff , buffSize * sizeof (char));
	data->size					= buffSize;
	data->samples				= gf_list_new();
	data->ID					= ID;
	data->fileID				= strdup( fileId );
	data->mime					= strdup("visual/scene");
	data->ODID					= BASIC_DMC_ODID+m_trackID;
	data->ESID					= DMC_ESID+m_trackID;
	data->streamFormat			= NULL;
	data->streamType			= GF_ISOM_PL_VISUAL;
	data->objectTypeIndication	= DMC_OBJECT_TYPE_INDICATION;
	data->bufferSizeDB			= DMC_BUFFER_SIZE_DB ; // buffSize ?????
	data->mediaType				= GF_ISOM_MEDIA_VISUAL;
	data->trackType				= BT_3DMC;
	gf_list_add( m_TrackList ,  data ); 
	
	return 0;
}

int MP4Mux::SetFAMC(  char *buff, int buffSize, int ID, const char* fileId )
{
	TrackData* data;
	data = new TrackData();
	data->trackID				= ++m_trackID;
	data->trackNo				= 0;
	data->buff					=( char* ) malloc ( buffSize * sizeof (char));
	memcpy( data->buff, buff , buffSize * sizeof (char));
	data->size					= buffSize;
	data->samples				= gf_list_new();
	data->ID					= ID;
	data->fileID				= strdup( fileId );
	data->mime					= strdup("visual/scene");
	data->ODID					= BASIC_FAMC_ODID+m_trackID;
	data->ESID					= FAMC_ESID+m_trackID;
	data->streamFormat			= NULL;
	data->streamType			= GF_ISOM_PL_VISUAL;
	data->objectTypeIndication	= FAMC_OBJECT_TYPE_INDICATION;
	data->bufferSizeDB			= FAMC_BUFFER_SIZE_DB ; // buffSize ?????
	data->mediaType				= GF_ISOM_MEDIA_VISUAL;
	data->trackType				= BT_FAMC;
	gf_list_add( m_TrackList ,  data ); 
	SetFAMCSamples( data );
	
	return 0;
}


int MP4Mux::SetTexture( char *buff, int buffSize, int ID , const char* fileId)
{
	TrackData* data;
	data = new TrackData();
	data->trackID				= ++m_trackID;
	data->trackNo				= 0;
	data->buff					=( char* ) malloc ( buffSize * sizeof (char));
	memcpy( data->buff, buff , buffSize * sizeof (char));
	data->size					= buffSize;
	data->samples				=  gf_list_new();
	data->ID					= ID;
	data->fileID				= strdup(fileId);
	data->mime					= strdup("texture/jpeg");
	data->ODID					= BASIC_JPG2000_ODID+m_trackID;
	data->ESID					= JPG2000_ESID+m_trackID;
	data->streamFormat			= NULL;
	data->streamType			= GF_ISOM_PL_VISUAL;
	data->objectTypeIndication	= JPG2000_OBJECT_TYPE_INDICATION;
	data->bufferSizeDB			= JPG2000_BUFFER_SIZE_DB ; // buffSize ?????
	data->mediaType				= GF_ISOM_MEDIA_VISUAL;
	data->trackType				= BT_TEX;
	gf_list_add( m_TrackList , data ); 

	return 0;	
}

int MP4Mux::SetBBA(  char *buff, int buffSize, int ID , const char* fileId)
{
	TrackData* data;
	data = new TrackData();
	data->trackID				= ++m_trackID;
	data->trackNo				= 0;
	data->buff					=( char* ) malloc ( buffSize * sizeof (char));
	memcpy( data->buff, buff , buffSize * sizeof (char));
	data->size					= buffSize;
	data->samples				=  gf_list_new();
	data->ID					= ID;
	data->fileID				= strdup(fileId);
	data->mime					= strdup("visual/bba");
	data->ODID					= BASIC_BBA_ODID+m_trackID;
	data->ESID					= BBA_ESID+m_trackID;
	data->streamFormat			= NULL;
	data->streamType			= GF_ISOM_PL_SCENE;
	data->objectTypeIndication	= BBA_OBJECT_TYPE_INDICATION;
	data->bufferSizeDB			= BBA_BUFFER_SIZE_DB ; // buffSize ?????
	data->mediaType				= GF_ISOM_MEDIA_SCENE;
	data->trackType				= BT_BBA;
	gf_list_add( m_TrackList , data ); 
	SetSamples( data );

	return 0;
}

int MP4Mux::ReadBeginSample(char* data_buffer,int current_pos,int fileSize)
{
	unsigned int begin=0;
	while ((begin!=490)&&(current_pos<fileSize-4))
	{
		begin=(unsigned char)data_buffer[current_pos+0];

		begin<<=8;
		begin|=(unsigned char)(data_buffer[current_pos+1]);

		begin<<=8;
		begin|=(unsigned char)(data_buffer[current_pos+2]);

		begin<<=8;
		begin|=(unsigned char)(data_buffer[current_pos+3]);


		current_pos++;
	}
	return current_pos+3;
}


bool MP4Mux::ReadNextSample(char* data_buffer, char* frame,int &frame_size,int &current_pos,int fileSize )
{
	bool hasMoreFrames=true;
	unsigned int beginframe=0;
	unsigned int endframe=0;
	frame_size=0;

	while ((beginframe!=492)&&(beginframe!=491)&&(current_pos<fileSize-3))
	{
		beginframe=(unsigned char)(data_buffer[current_pos+0]);

		beginframe<<=8;
		beginframe|=(unsigned char)(data_buffer[current_pos+1]);

		beginframe<<=8;
		beginframe|=(unsigned char)(data_buffer[current_pos+2]);

		beginframe<<=8;
		beginframe|=(unsigned char)(data_buffer[current_pos+3]);

		current_pos++;
	}
	
	//frame[frame_size]=data_buffer[current_pos-2];
	//frame_size++;

	frame[frame_size]=data_buffer[current_pos-1];
	frame_size++;

	while ((endframe!=492)&&(endframe!=491)&&(current_pos<(fileSize-3)))
	{
		frame[frame_size]=data_buffer[current_pos];
		frame_size++;
		endframe=(unsigned char)(data_buffer[current_pos+0]);

		endframe<<=8;
		endframe|=(unsigned char)(data_buffer[current_pos+1]);

		endframe<<=8;
		endframe|=(unsigned char)(data_buffer[current_pos+2]);

		endframe<<=8;
		endframe|=(unsigned char)(data_buffer[current_pos+3]);
		current_pos++;
//		FILE * fh = fopen("D:\\pos.txt", "a");
//		char* str;
//		int t = current_pos;
		
//		fprintf(fh,"%d\n",t);
		//fwrite(str,1, sizeof(str)+10,fh);
//		fclose(fh);

	}

	current_pos=current_pos-1;	

	if ((beginframe==491)||(endframe==491)) hasMoreFrames=false;
	if ((current_pos>=fileSize-4)) {hasMoreFrames=false;}

	//FILE* f;
	//f = fopen("C://testbba2","wb");
	//
	//fwrite(frame,frame_size,1,f);
	//fclose ( f );


	return hasMoreFrames;
}


bool MP4Mux::SetSamples( TrackData* data )
{
	char* b;
	int bs = 0;
	int begin_pos=0;
	int current_pos=0;
	bool isFirstFrame = true;
	current_pos=ReadBeginSample((char *) data->buff,current_pos,data->size );
	bs= current_pos+1;
	b = (char*) malloc(bs);
	memcpy(b,data->buff,bs);
	begin_pos=current_pos;
	char frame_data[4096*16];
	int frame_size=0;
	char firstFrame[4096*16];
	int firstFrameSize = 0;
	bool hasFrames = true;
	
	int i = 1;
	int broj =0;
	while (hasFrames)
	{
		hasFrames = ReadNextSample(( char *)data->buff,frame_data,frame_size,current_pos,data->size);
		//from the first frame create the track, so put in data->buff values from the first frame
		if (isFirstFrame)
		{
			memcpy(firstFrame,frame_data,frame_size);
			firstFrameSize = frame_size;
			isFirstFrame = false;
		}
		//add samples in the track
		SampleData* smpl;
		smpl = new SampleData();
		smpl->buff =( char* ) malloc ( frame_size * sizeof (char));
		memcpy(smpl->buff, frame_data, frame_size);
		smpl->size					= frame_size;
		gf_list_add( data->samples , smpl ); 
		broj+=frame_size;
	}

	SampleData* tmp = (SampleData*)gf_list_get( data->samples, gf_list_count(data->samples) - 1 );
	char* tmpb = (char*) malloc(tmp->size+4);
	memcpy(tmpb, tmp->buff, tmp->size);
	tmpb[tmp->size] = 0x00;
	tmpb[tmp->size+1] = 0x00;
	tmpb[tmp->size+2] = 0x01;
	tmpb[tmp->size+3] = 0xEB;
	free(tmp->buff);
	tmp->buff = tmpb;
	tmp->size+=4;

	free( data->buff );
	data->size = bs + firstFrameSize;
	data->buff = (char*)malloc( data->size );
	memcpy( data->buff,b,bs);
	memcpy( data->buff+bs, firstFrame ,firstFrameSize );

	return true;
}


bool MP4Mux::ReadNextFAMCSample(char* data_buffer, char* frame,int &frame_size,int &current_pos,int fileSize )
{
	bool hasMoreFrames=true;
	unsigned int beginframe=0;
	unsigned int endframe=0;
	frame_size=0;

	while ((beginframe!=4026597376)&&(current_pos<fileSize-3))
	{
		beginframe=(unsigned char)(data_buffer[current_pos+0]);

		beginframe<<=8;
		beginframe|=(unsigned char)(data_buffer[current_pos+1]);

		beginframe<<=8;
		beginframe|=(unsigned char)(data_buffer[current_pos+2]);

		beginframe<<=8;
		beginframe|=(unsigned char)(data_buffer[current_pos+3]);

		current_pos++;
	}
	
	frame[frame_size]=data_buffer[current_pos-1];
	frame_size++;

	while ((endframe!=4026597376)&&(current_pos<(fileSize-3)))
	{
		endframe=(unsigned char)(data_buffer[current_pos+0]);

		endframe<<=8;
		endframe|=(unsigned char)(data_buffer[current_pos+1]);

		endframe<<=8;
		endframe|=(unsigned char)(data_buffer[current_pos+2]);

		endframe<<=8;
		endframe|=(unsigned char)(data_buffer[current_pos+3]);

		if ( endframe!=4026597376 )
		{
			frame[frame_size]=data_buffer[current_pos];
			frame_size++;
		}

		current_pos++;
		//FILE * fh = fopen("D:\\pos.txt", "a");
		//char* str;
		//int t = current_pos;
		//
		//fprintf(fh,"%d\n",t);
		////fwrite(str,1, sizeof(str)+10,fh);
		//fclose(fh);

	}

	current_pos=current_pos-1;	
	//frame_size--;

	if ((current_pos>=fileSize-4)) 
	{
		current_pos++;
		frame[frame_size++]=data_buffer[current_pos++];
		frame[frame_size++]=data_buffer[current_pos++];
		frame[frame_size++]=data_buffer[current_pos++];
		hasMoreFrames=false;
	}

	//FILE* f;
	//f = fopen("C://testbba2","wb");
	//
	//fwrite(frame,frame_size,1,f);
	//fclose ( f );


	return hasMoreFrames;
}


bool MP4Mux::SetFAMCSamples( TrackData* data )
{
	char* b;
	int bs = 0;
	int current_pos=0;
	bool isFirstFrame = true;
	char frame_data[4096*50];
	int frame_size=0;
	char firstFrame[4096*50];
	int firstFrameSize = 0;
	bool hasFrames = true;
	
	int i = 1;
	int broj =0;
	while (hasFrames)
	{
		hasFrames = ReadNextFAMCSample(( char *)data->buff,frame_data,frame_size,current_pos,data->size);
		//from the first frame create the track, so put in data->buff values from the first frame
		if (isFirstFrame)
		{
			memcpy(firstFrame,frame_data,frame_size);
			firstFrameSize = frame_size;
			isFirstFrame = false;
		}
		//add samples in the track
		SampleData* smpl;
		smpl = new SampleData();
		smpl->buff =( char* ) malloc ( frame_size * sizeof (char));
		memcpy(smpl->buff, frame_data, frame_size);
		smpl->size					= frame_size;
		gf_list_add( data->samples , smpl ); 
		broj+=frame_size;
	}

	//put in data->buff only first frame
	free( data->buff );
	data->size = bs + firstFrameSize;
	data->buff = (char*)malloc( data->size );
	memcpy( data->buff,b,bs);
	memcpy( data->buff+bs, firstFrame ,firstFrameSize );

	return true;
}



int MP4Mux::SetXMLinMETA(  char *buff, int buffSize )
{
	//create temporary file
	m_hasBinXml = true;
	FILE *f;
	f = fopen( "rarScene.gz" , "wb");
	fwrite(buff, buffSize,1,f);
	fclose( f );

	return 0;
}


int MP4Mux::CreateMP4 (const char* file_name )
{
	m_mp4FileName = file_name;

	// Create OD Track
	
	MakeOD *OD = new MakeOD();
	OD->InitOD ();
	TrackData* data = NULL;
	for (int i = 0; i< gf_list_count(m_TrackList); i++)
	{
		data = ( TrackData* )gf_list_get(m_TrackList, i );
		if (!(data->streamType == GF_ISOM_PL_SCENE 
			&& data->objectTypeIndication == BIFS_OBJECT_TYPE_INDICATION)) 
			OD->AddODSample (data->ODID, data->ESID, data->streamFormat, GF_ISOM_PL_OD/*bese constantno 1 *//*OD_STREAM_TYPE??GF_ISOM_PL_OD == 1 ???*/, data->objectTypeIndication, data->bufferSizeDB );	
	}

	OD->CreateODBuffer ();
	data = new TrackData();
	data->trackID				= ++m_trackID;
	data->trackNo				= 0;
	data->buff					=( char* ) malloc ( OD->au_length * sizeof (char));
	memcpy( data->buff, OD->outAU, OD->au_length * sizeof (char));
	data->size					= OD->au_length;
	data->samples				= NULL;
	data->ID					= 0;
	data->fileID				= strdup("od");
	data->mime					= strdup("od/od");
	data->ODID					= BASIC_OD_ODID;
	data->ESID					= OD_ESID;
	data->streamFormat			= NULL;
	data->streamType			= GF_ISOM_PL_OD;
	data->objectTypeIndication	= OD_OBJECT_TYPE_INDICATION;
	data->bufferSizeDB			= OD_BUFFER_SIZE_DB ; // buffSize ?????
	data->mediaType				= GF_ISOM_MEDIA_OD;
	data->trackType				= BT_OD;
	gf_list_add( m_TrackList , data ); 

	// Create OD in tracks in mp4 container
	OD->InitOD ();
	for (int i = 0; i< gf_list_count(m_TrackList); i++)
	{
		data = (TrackData*)gf_list_get( m_TrackList , i );
		OD->AddODSample (data->ODID, data->ESID, data->streamFormat,data->streamType, data->objectTypeIndication, data->bufferSizeDB );
	}
	// create empty mp4 file 
	GF_ISOFile *mm  =  gf_isom_create_movie(m_mp4FileName, GF_ISOM_OPEN_WRITE, "");
	
	//Add iods
	GF_Descriptor* dec = gf_odf_desc_new(GF_ODF_IOD_TAG/*GF_ODF_IPMP_TL_TAG*/);
	
	GF_Err e = gf_isom_add_desc_to_root_od( mm, dec ); 

	//Add tracks to file
	for (int i = 0; i< gf_list_count(m_TrackList); i++)
	{
		data = (TrackData*)gf_list_get( m_TrackList , i );

		u32 trakNo = gf_isom_new_track( mm, data->ESID ,data->mediaType, 1000); 
		data->trackNo = trakNo;
		GF_ISOSample *firstSample = gf_isom_sample_new();
		firstSample->data = data->buff;
		data->buff = NULL;
		firstSample->dataLength = data->size;
		data->size = 0;
		
		
		u32 index;
		GF_ObjectDescriptor *od = (GF_ObjectDescriptor*) gf_list_get(OD->odUpdate->objectDescriptors,i);
		GF_ESD* pESD = (GF_ESD*)gf_list_get( od->ESDescriptors, 0 );  //for each OD  'i-1' entry
		e = gf_isom_new_mpeg4_description(mm, trakNo, pESD, NULL, NULL, &index);  
		e = gf_isom_add_sample( mm, trakNo, index, firstSample );
		SampleData* smpl;
		GF_ISOSample* sample;
		int uou=0;
		for (int j=1; j< gf_list_count(data->samples); j++)
		{
			
			smpl = (SampleData*)gf_list_get( data->samples , j );
			GF_ISOSample* sample =  gf_isom_sample_new();
			sample->data = smpl->buff;
			smpl->buff = NULL;
			sample->dataLength = smpl->size;
			smpl->size = 0;
			uou +=smpl->size;
			//to do::get the real frame rate
			//sample->DTS = j * 40 - 40;
			e = gf_isom_add_sample(mm, trakNo, index,sample); //// Ok -tracknumber 
		}
		
		if ((data->streamType == GF_ISOM_PL_OD) 
			|| (data->streamType == GF_ISOM_PL_SCENE && data->objectTypeIndication == BIFS_OBJECT_TYPE_INDICATION))
			e =gf_isom_add_track_to_root_od( mm, trakNo ); 

				
		if (data->streamType == GF_ISOM_PL_SCENE && data->objectTypeIndication == BIFS_OBJECT_TYPE_INDICATION)
		{
			//add bifsDesc
			GF_TrackBox* track = gf_isom_get_track(mm->moov, trakNo);
			GF_ESD* esd;
			u32 StreamDescriptionIndex = 1;
			Media_GetESD(track->Media, StreamDescriptionIndex, &esd, 1);
			char *data = (char*)malloc(3);
			data[0] = 0x14;
			data[1] = 0xA5;
			data[2] = 0x40;
			GF_DefaultDescriptor* desc = (GF_DefaultDescriptor*)gf_odf_desc_new( GF_ODF_DSI_TAG );
			desc->data = data;
			desc->dataLength = 3;
			esd->decoderConfig->decoderSpecificInfo = desc;
			//end add BifsDesc
		}
	}

	//now file has tracks 
	// add meta-data to  mp4 file
	gf_isom_set_meta_type( mm, true, 0, GF_4CC('m','p','g','4'));
	// add xml to meta data
	if (m_hasBinXml)
	{
		
		e = gf_isom_set_meta_xml( mm, true, 0, "rarScene.gz", true );
		//remove temporary file
		remove( "rarScene.gz" );
	}
	//add items to meta
	
	
	for (int i = 0; i< gf_list_count(m_TrackList); i++)
	{
		data = ( TrackData* )gf_list_get(m_TrackList, i );
		char data_ref[50];
		sprintf(data_ref, "#ffp(trackID=%d)",data->trackID);
		//@URL here is used for location of the track (#ffp(track_ID=%d))
		//data refefence is created from this URL
		e = gf_isom_add_meta_item( mm, true,0, false , NULL ,data->fileID, data->mime ,"",data_ref,NULL);
	}

	gf_isom_close(mm);
	OD->DeleteODData(); 

	return 0;	
}
