#ifndef _MP4MUX_H_
#define _MP4MUX_H_

/******************************************************************************\
* Includes
\******************************************************************************/

#include "MakeOD.h"

#include "gpac/nodes_mpeg4.h"
#include "gpac/internal/scenegraph_dev.h"
#include "gpac/scenegraph.h"

/******************************************************************************\
* Defines
\******************************************************************************/
	 //for OD stream
#define	BASIC_OD_ODID					0
#define	OD_ESID							101
#define	OD_STREAM_TYPE					2
#define	OD_OBJECT_TYPE_INDICATION		1
#define	OD_BUFFER_SIZE_DB				500000		
	//for Bifs stream
#define	BASIC_BIFS_ODID					0
#define	BIFS_ESID						201
#define	BIFS_STREAM_TYPE				3
#define	BIFS_OBJECT_TYPE_INDICATION		2
#define	BIFS_BUFFER_SIZE_DB				4000000		
	//for JPG2000 stream  (JPEG2000)
#define	BASIC_JPG2000_ODID				300
#define	JPG2000_ESID					600
#define	JPG2000_STREAM_TYPE				4
#define	JPG2000_OBJECT_TYPE_INDICATION	0x6e
#define	JPG2000_BUFFER_SIZE_DB			9000000
	//for BBA stream
#define	BASIC_BBA_ODID					500
#define	BBA_ESID						301
#define	BBA_STREAM_TYPE					0
#define	BBA_OBJECT_TYPE_INDICATION		5
#define	BBA_BUFFER_SIZE_DB				0

	//for 3DMC stream
#define	BASIC_DMC_ODID					800
#define	DMC_ESID						401
#define	DMC_STREAM_TYPE					0
#define	DMC_OBJECT_TYPE_INDICATION		55
#define	DMC_BUFFER_SIZE_DB				4000000	

	//for FAMC stream
#define	BASIC_FAMC_ODID					900
#define	FAMC_ESID						501
#define	FAMC_STREAM_TYPE				0
#define	FAMC_OBJECT_TYPE_INDICATION		100
#define	FAMC_BUFFER_SIZE_DB				4000000	
/******************************************************************************\
* Types
\******************************************************************************/

// defines types of tracks in mp4 file
enum BufferType
{
	BT_NONE		= 0,
	BT_OD		= 1,
	BT_BIFS		= 2,
	BT_TEX		= 3,
	BT_BBA		= 4,
	BT_3DMC		= 5,
	BT_FAMC		= 5
	
};

/******************************************************************************\
* Structures
\******************************************************************************/
class SampleData
{
public:


	SampleData()
	{
		//buff is deleted when the sample is deleted
		buff = NULL;
		size = 0;
		DTS = -1;
	}

	~SampleData()
	{
		if ( buff )
			free( buff );
	}

	char*		buff;
	int			size;
	int			DTS;

};

class TrackData
{
public:

	TrackData()
	{
		//buff is deleted when the sample is deleted
		buff	= NULL;
		size	= 0;
		fileID	= NULL;
		mime	= NULL;
		streamFormat = NULL;
	}

	~TrackData()
	{
		if( fileID )
			free( fileID );
		if( mime )
			free( mime );
		if( streamFormat )
			free( streamFormat );
		if( buff )
			free( buff );
		for ( unsigned int i = 0; i < gf_list_count( samples ); i++ )
		{
			SampleData* ps = ( SampleData* ) gf_list_get( samples, i );
			delete ps;
		}
		gf_list_del( samples );
	}

	int			trackID;
	int			trackNo;
	char*		buff;
	int			size;
	int			ID;
	char*		fileID;
	char*		mime;
	GF_List*	samples;
	int			ODID;
	int			ESID;
	char*		streamFormat;
	int			streamType;
	int			objectTypeIndication;
	int			bufferSizeDB;
	u32			mediaType;
	BufferType	trackType;

};


/******************************************************************************\
* Class MP4Mux
\******************************************************************************/
class MP4Mux
{
public:

	//de/constructor
	MP4Mux();
	~MP4Mux();
	
	//methods
	
	//Add one track with 3DMC in MP4 file
	// ID = the order number of the 3DMC buffer
	// fileId = the name of the resource from which is created 3DMC buffer 
	// ( fileID  for collada files is  daeID of geometry buffer )
	int Set3DMC(  char *buff, int buffSize, int ID, const char* fileId );
	//Add one track with FAMC in MP4 file
	// ID = the order number of the FAMC buffer
	// fileId = the name of the resource from which is created FAMC buffer 
	// ( fileID  for collada files is  daeID of geometry buffer )
	int SetFAMC(  char *buff, int buffSize, int ID, const char* fileId );
	//Add one track with JPG2000 in MP4 file
	// ID = the order number of the Texture buffer
	// fileId = the name of the resource from which is created Texture buffer
	// ( fileID  for collada files is  daeID of image buffer )
	int SetTexture( char *buff, int buffSize, int ID , const char* fileId);
	//Add one track with BBA in MP4 file
	// ID = the order number of the Animation buffer
	// fileId = the name of the resource from which is created Animation buffer
	// ( fileID  for collada files is  daeID of image buffer )
	int SetBBA(  char *buff, int buffSize, int ID , const char* fileId);
	//Add encoded XML in MP4 file
	int SetXMLinMETA(  char *buff, int buffSize );
	// Create OD track and then MP4 file
	//file_name == the name of the mp4 file
	int CreateMP4 (const char* file_name );
	

private:
	
	//variables

	GF_List* m_TrackList;
	const char* m_mp4FileName;
	int m_trackID;
	bool m_hasBinXml;

	//functions

	int		ReadBeginSample(char* data_buffer,int current_pos,int fileSize);
	bool	ReadNextSample(char* data_buffer, char* frame,int &frame_size,int &current_pos,int fileSize );
	bool	SetSamples( TrackData* data  );
	bool	ReadNextFAMCSample(char* data_buffer, char* frame,int &frame_size,int &current_pos,int fileSize );
	bool	SetFAMCSamples( TrackData* data  );
	

};


#endif //_MP4MUX_H_
