/******************************************************************************\
* Includes
\******************************************************************************/
#include "MP4toColladaDecode.h"

#include "DecodeMP4Streams.h"
#include "MP4Animation.h"
#include "MP43DMC.h"
#include "MP4toVBLib.h"

#include "Euler.h" 

#define _USE_MATH_DEFINES 
#include <math.h> 

#include "FCDocument/FCDAnimated.h"
//#include "MPG_IfsQ_C_Ifs.h"
//#include "MPG_IfsQ_E_Ifs.h"
#include "matrix.h"

#include "Decoder/CompressedPositionInterpolatorD.h" 
#include "Decoder/CompressedCoordinateInterpolatorD.h" 
#include "Decoder/CompressedOrientationInterpolatorD.h"

/******************************************************************************\
 MP4toColladaDecode.cpp: implementation of the MP4toColladaDecode class.
\******************************************************************************/

/******************************************************************************\
* Class MP4toColladaDecode
\******************************************************************************/
MP4toColladaDecode::MP4toColladaDecode(void)
{
}


MP4toColladaDecode::~MP4toColladaDecode(void)
{

}

bool MP4toColladaDecode::RestoreXML( const char * colladaFileName )
{
	unsigned char * restBuff = NULL;
	int				restBuffSize = 0;
	MP4toVBLib_GetXMLfromMETA( restBuff ,restBuffSize,true);

	DecodeMP4Streams *ds= new DecodeMP4Streams();
	MP43DMC * dmc = NULL;
	ds->DecodeXMLStream( restBuff, restBuffSize);
	if (restBuff)
		free(restBuff);
	restBuff = NULL;
	restBuffSize =0;
	ds->GetDecodedXML( restBuff, restBuffSize,true);
	delete ds;

	FILE *dae;
	dae = fopen( colladaFileName , "wb");
	fwrite(restBuff, restBuffSize,1,dae);
	fclose( dae );
	if (restBuff)
		free( restBuff );
	
	return true;
}

bool MP4toColladaDecode::RestoreGeometry( FCDocument* file )
{
	unsigned char* buff= NULL;
	int buffSize = 0;
	FCDGeometryLibrary *gl = file->GetGeometryLibrary();
	
	FCDGeometrySource	*gmsC	= NULL;
	FCDGeometrySource	*gmsT	= NULL;
	FCDGeometrySource	*gmsN	= NULL;
	FCDGeometrySource	*gmsRGB	= NULL;

	FloatList	verticesC;
	FloatList	verticesT;
	FloatList	verticesN;
	FloatList	verticesRGB;
	
	FCDGeometryPolygonsInput *indC;
	FCDGeometryPolygonsInput *indT;
	FCDGeometryPolygonsInput *indN;
	FCDGeometryPolygonsInput *indRGB;

	for (int i=0; i<gl->GetEntityCount(); i++)
	{
		FCDGeometry *ge = gl->GetEntity(i);
		MP4toVBLib_Get3DMCbyRef(  ge->GetDaeId().c_str() , buff, buffSize, true);
		
		DecodeMP4Streams *ds= new DecodeMP4Streams();
		MP43DMC * dmc = NULL;
		ds->Decode3DMCStream((char*)buff, buffSize);
		free( buff );
		buff = NULL;
		buffSize = 0;
		dmc = ds->GetDecoded3DMC();
		
		std::vector<MeshFrame> meshVec;
		MeshFrame mesh;
		mesh.mesh = dmc;
		mesh.time=0;
		meshVec.push_back( mesh );

		//if ( MP4toVBLib_GetFAMCbyRef(  ge->GetDaeId().c_str() , buff, buffSize, true) )
		//{
		//	ds->DecodeFAMCStream( (char*) buff, buffSize, meshVec );
		//	RestoreCoordInterpolator( file, ge, meshVec );
		//}	
	
		FCDGeometryMesh *gm = ge->GetMesh();
		//add vertices
		gmsC = gm->FindSourceByType(FUDaeGeometryInput::POSITION);
		if ( gmsC )
		{
			verticesC.clear();
			for( int j = 0; j < dmc->GetCoordinateCount();j++ )
				verticesC.push_back( dmc->GetCoordinate()[j]);
				
			gmsC->SetStride( 3 );
			gmsC->SetData( verticesC,gmsC->GetStride(),0,0);
		}
		
		//add normals
		gmsN = gm->FindSourceByType(FUDaeGeometryInput::NORMAL);
		if ( gmsN )
		{
			verticesN.clear();
			for( int j = 0; j < dmc->GetNormalCount();j++ )
				verticesN.push_back( dmc->GetNormal()[j]);
				
			gmsN->SetStride( 3 );
			gmsN->SetData( verticesN,gmsN->GetStride(),0,0);
		}

		//add text coord
		gmsT = gm->FindSourceByType(FUDaeGeometryInput::TEXCOORD);
		if ( gmsT )
		{
			verticesT.clear();
			for( int j = 0; j < dmc->GetTextureCoordinateCount();j++ )
				verticesT.push_back( dmc->GetTextureCoordinate()[j]);
			
			gmsT->SetStride( 2 );
			gmsT->SetData( verticesT,gmsT->GetStride(),0,0);
		}
		//add color
		gmsRGB = gm->FindSourceByType(FUDaeGeometryInput::COLOR);
		if ( gmsRGB )
		{
			verticesRGB.clear();
			for( int j = 0; j < dmc->GetColorCount();j++ )
				verticesRGB.push_back( dmc->GetColor()[j]);
			
			gmsRGB->SetStride( 3 );
			gmsRGB->SetData( verticesRGB,gmsRGB->GetStride(),0,0);
		}
	
		//indices
		int idx = 0;
		for (int tmp = 0; tmp < gm->GetPolygonsCount(); tmp++ )
		{
			FCDGeometryPolygons *gmp	= gm->GetPolygons ( tmp );
			indC = gmp->FindInput(FUDaeGeometryInput::POSITION);
			indN = gmp->FindInput(FUDaeGeometryInput::NORMAL);
			indT = gmp->FindInput(FUDaeGeometryInput::TEXCOORD);
			indRGB = gmp->FindInput(FUDaeGeometryInput::COLOR);
			
			int textureNo = 0;
			for (int temp=0; temp< gmp->GetInputCount(); temp++)
			{
				if ( gmp->GetInput( temp )->GetSemantic() == FUDaeGeometryInput::TEXCOORD)
					textureNo++ ;
			}
			if ( textureNo == 1 )
			{
				int cnt = indC->GetIndices()[0];
				gmp->RemoveFace(0);
				for (int k=0; k<cnt; k++)
				{
					gmp->AddFace(3);				
					for ( int j = 0; j < 3; j++ )
					{					
						indC->GetIndices()[k*3+j] = dmc->GetCoordinateIndex()[idx];
						if ( indN )
							indN->GetIndices()[k*3+j] = dmc->GetNormalIndex()[idx];
						if (indT)
							indT->GetIndices()[k*3+j] = dmc->GetTextureCoordinateIndex()[idx];
						if (indRGB)
							indRGB->GetIndices()[k*3+j] =dmc->GetColorIndex()[idx];
						idx++;
					}
					idx++;
				}
			}
			else
			{
				int cnt = indC->GetIndices()[0];
				for (int k=0; k<cnt; k++)
				{
					for ( int j = 0; j < 3; j++ )
					{					
						indC->GetIndices()[k*3+j] = dmc->GetCoordinateIndex()[idx];
						if ( indN )
							indN->GetIndices()[k*3+j] = dmc->GetNormalIndex()[idx];
						if (indT)
							indT->GetIndices()[k*3+j] = dmc->GetTextureCoordinateIndex()[idx];
						if (indRGB)
							indRGB->GetIndices()[k*3+j] =dmc->GetColorIndex()[idx];
						idx++;
					}
					idx++;
				}
			}
		}

		if ( MP4toVBLib_GetFAMCbyRef(  ge->GetDaeId().c_str() , buff, buffSize, true) )
		{
			ds->DecodeFAMCStream( (char*) buff, buffSize, meshVec );
			RestoreCoordInterpolator( file, ge, meshVec );
		}	

		delete ds;
	}
	return true;
}



//restore animation without CI
//bool MP4toColladaDecode::RestoreAnimation( FCDocument* file )
//{
//	FCDAnimationLibrary *anlib  = NULL;
//	FCDAnimation		*an		= NULL;
//	FCDAnimation		*anim	= NULL;
//	FCDAnimationChannel *anchn	= NULL;
//	FCDAnimationCurve	*ancur	= NULL;
//	int cnt = 0;
//	
//	anlib = file->GetAnimationLibrary();
//	if (! anlib || (anlib->GetEntityCount()==0) )
//		return true;
//
//	//ova e kod1 za files bez bez bezier, koga nema entity, direktno pod library se site deca
//	
//	an = anlib->GetEntity ( anlib->GetEntityCount()-1 );
//	unsigned char* buff= NULL;
//	int buffSize = 0;
//	MP4toVBLib_GetBBAbyRef( an->GetDaeId().c_str(), buff, buffSize, true );
//
//	if ( !buff || !buffSize )
//		return false;
//
//	DecodeMP4Streams *ds= new DecodeMP4Streams();
//	MP4AnimationBuff *ab= NULL;
//	ds->DecodeBBAStream(buff, buffSize);
//	free ( buff );
//	ab = ds->GetDecodedBBA();
//
//	for ( int i=0; i<anlib->GetEntityCount(); i++)
//	{		
//			an = anlib->GetEntity ( i );
//
//			anim	= an ; 
//			anchn	= anim->GetChannel(0);
//
//			fm::string &tp = anchn->GetTargetPointer();//= data.targetPointer;
//			fm::string &tq = anchn->GetTargetQualifier();//data.targetQualifier;
//
//			if ( !tp.size() )
//				continue;
//
//			if ( tp.find( "rotation" ))
//			{
//
//			}
//			
//			if ( tp.find( "transform" ) )
//			{
//				if ( !tq.size() )
//				{
//					ancur	= anchn->GetCurve( 0 );
//					FCDAnimationKey** keys = ancur->GetKeys();
//					cnt =  ancur->GetKeyCount();
//					
//					for ( int k =0; k<16; k++ )
//					{
//						ancur	= anchn->GetCurve( k );
//						keys	= ancur->GetKeys();
//
//						for (int l=0; l<ancur->GetKeyCount(); l++)
//						{
//							int tx = k/4;
//							int ty = k%4;
//							keys[l]->output = ab->GetMP4Frame(l)->GetMP4BoneDataMatrix(i)[ty*4 + tx];//k e kelija vo matricata
//						}
//					}
//				}	
//				else
//				{
//					size_t first  = tq.find_first_of( "(" );
//					size_t second = tq.find_last_of( ")" );
//					char x = tq.at( first + 1 );
//					char y = tq.at( second - 1 );
//
//					int ix = x-'0';
//					int iy = y-'0';
//
//					ancur	= anchn->GetCurve( 0 );
//					FCDAnimationKey** keys = ancur->GetKeys();
//					
//					for (int l=0; l<ancur->GetKeyCount(); l++)
//						keys[l]->output=ab->GetMP4Frame(l)->GetMP4BoneDataMatrix(i)[iy*4+ix];
//				}
//			}
//	}
//	delete ds;
//
//	return true;
//}
//
//
FCDTransform::Type MP4toColladaDecode::GetTransformNameSub(FCDocument* file, fm::string &tp, fm::string &tq, FCDSceneNode* node) const  
{ 
	//printf("%d(%s),", node->GetType(), node->GetDaeId().c_str()); 
	if(node->GetDaeId() == tp) { 
		for(int i=0; i<node->GetTransformCount(); i++) { 
			if(node->GetTransform(i)->GetSubId() == tq)  
				return node->GetTransform(i)->GetType(); 
		} 
		return FCDTransform::Type::TYPE_COUNT;
	} 

	for(int i=0; i<node->GetChildrenCount(); i++) { 
		FCDTransform::Type ret = GetTransformNameSub(file, tp, tq, node->GetChild(i)); 
		if(ret != FCDTransform::Type::TYPE_COUNT) return ret; 
	} 
	return FCDTransform::Type::TYPE_COUNT; 
} 

FCDTransform::Type MP4toColladaDecode::GetTransformName(FCDocument* file, fm::string &tp, fm::string &tq) const  
{ 

	for(int i=0; i<file->GetVisualSceneLibrary()->GetEntityCount(); i++) { 
		for(int j=0; j<file->GetVisualSceneLibrary()->GetEntity(i)->GetChildrenCount(); j++) { 
			FCDTransform::Type ret = GetTransformNameSub(file, tp, tq, file->GetVisualSceneLibrary()->GetEntity(i)->GetChild(j)); 
			if(ret != FCDTransform::Type::TYPE_COUNT) return ret; 
		} 
	} 
	return FCDTransform::Type::TYPE_COUNT; 
} 




bool MP4toColladaDecode::RestoreAnimation( FCDocument* file )
{
	FCDAnimationLibrary *anlib  = NULL;
	FCDAnimation		*an		= NULL;
	FCDAnimation		*anim	= NULL;
	FCDAnimationChannel *anchn	= NULL;
	FCDAnimationCurve	*ancur	= NULL;
	int cnt = 0;

	anlib = file->GetAnimationLibrary();
	if (! anlib || (anlib->GetEntityCount()==0) )
		return true;

	//ova e kod1 za files bez bez bezier, koga nema entity, direktno pod library se site deca

	an = anlib->GetEntity ( anlib->GetEntityCount()-1 );
	unsigned char* buff= NULL;
	int buffSize = 0;
	MP4toVBLib_GetBBAbyRef( an->GetDaeId().c_str(), buff, buffSize, true );
 
	if ( buffSize < 1 )
	 return true;

	switch(buff[0]) { 
	case 'p': 
		{ 
			ofstream of("PositionInterpolation.tmp", ios::out | ios::binary); 
			for(int i=1; i<buffSize; i++) 
				of.put(buff[i]); 
			of.close(); 
			CompressedPositionInterpolatorD compress; 
			compress.decode("PositionInterpolation.tmp"); 

			for ( int ai=0; ai<anlib->GetEntityCount(); ai++) 
			{                

				an = anlib->GetEntity ( ai ); 

				anim    = an ;  
				anchn   = anim->GetChannel(0);

				ancur	= anchn->GetCurve( 0 );
				for ( int j = 0; j < ancur->GetKeyCount(); j++ ) { 
					ancur->GetKey(j)->input = compress.key[j]; 
					printf("%f ", ancur->GetKey(j)->input); 
				} 
				printf("\n");
				for(int i=0; i<anchn->GetCurveCount(); i++) { 
					ancur   = anchn->GetCurve( i ); 
					for ( int j = 0; j < ancur->GetKeyCount(); j++ ) { 
						if(i==0) ancur->GetKey(j)->output = compress.keyValue[j].x; 
						if(i==1) ancur->GetKey(j)->output = compress.keyValue[j].y; 
						if(i==2) ancur->GetKey(j)->output = compress.keyValue[j].z; 
						printf("%f ", ancur->GetKey(j)->output); 
					}
				}
			} 

			break; 
		} 
	case 'o': 
		{ 

			ofstream of("OrientationInterpolation.tmp", ios::out | ios::binary); 
			for(int i=1; i<buffSize; i++) 
				of.put(buff[i]); 
			of.close(); 
			CompressedOrientationInterpolatorD compress; 
			compress.decode("OrientationInterpolation.tmp"); 

			double *tempKey = new double[compress.dKey.GetSize()]; 
			double *tempKVx = new double[compress.dKey.GetSize()]; 
			double *tempKVy = new double[compress.dKey.GetSize()]; 
			double *tempKVz = new double[compress.dKey.GetSize()]; 

			for ( int j = 0; j < compress.dKey.GetSize(); j++ ) { 
				//printf("%f ", ancur->GetKey(j)->input); 
				tempKey[j] = compress.dKey[j]; 
				printf("\\DECODE : %f %f %f %f\n", compress.dKeyValue[j].x,compress.dKeyValue[j].y,compress.dKeyValue[j].z,compress.dKeyValue[j].theta); 
				double *temp = toEuler(compress.dKeyValue[j].x, compress.dKeyValue[j].y, compress.dKeyValue[j].z,compress.dKeyValue[j].theta); 
				tempKVx[j] = temp[0]/M_PI*180.0; 
				tempKVy[j] = temp[1]/M_PI*180.0; 
				tempKVz[j] = temp[2]/M_PI*180.0; 
				delete[] temp; 
			} 


			for ( int ai=0; ai<anlib->GetEntityCount(); ai++) 
			{                


				an = anlib->GetEntity ( ai ); 

				anim    = an ;  
				anchn   = anim->GetChannel(0);


				ancur	= anchn->GetCurve( 0 );
				for ( int j = 0; j < ancur->GetKeyCount(); j++ ) { 
					ancur->GetKey(j)->input = tempKey[j]; 
				} 

				for ( int j = 0; j < ancur->GetKeyCount(); j++ ) { 
					if(ai==0) ancur->GetKey(j)->output = tempKVz[j]; 
					if(ai==1) ancur->GetKey(j)->output = tempKVy[j]; 
					if(ai==2) ancur->GetKey(j)->output = tempKVx[j]; 
				} 
			} 

			delete []tempKey; 
			delete []tempKVx; 
			delete []tempKVy; 
			delete []tempKVz; 
			break; 
		} 
	case 'c': 
		{ 
			FILE *f;
			f = fopen("logAFTER.txt", "w");
			ofstream of("CoordinateInterpolation.tmp", ios::out | ios::binary); 
			for(int i=1; i<buffSize; i++) 
				of.put(buff[i]); 
			of.close(); 
			CompressedCoordinateInterpolatorD compress; 
			compress.decode("CoordinateInterpolation.tmp"); 
//			compress.decode("C:\\asdf.acc"); 

			for(int i=0; i<3; i++) {
				printf("%f ", compress.k.dKey[i]);
			}printf("\n");
			for(int i=0; i<24*3; i++) {
				if(i%3==0) printf("\n");
				printf("%d %f " , i%24, compress.coordIKeyValue.dKeyValue[i]);
			}

			fm::vector<float> gmlist;
			FCDGeometryLibrary      *gmlib = file->GetGeometryLibrary(); 
			FCDGeometrySource       *gmsC = NULL;

			for(int j=0; j<gmlib->GetEntityCount(); j++) { 
				FCDGeometry             *ge = gmlib->GetEntity( j ); 
				FCDGeometryMesh         *gm =  ge->GetMesh(); 
				gmsC   = gm->FindSourceByType(FUDaeGeometryInput::POSITION); 
				FloatList verticesC = gmsC->GetSourceData().GetDataList(); 
				int strideC = gmsC->GetStride(); 

				bool sw = false; 
				for(int k=0; k<verticesC.size(); k++ ) { 
					gmlist.push_back(verticesC[k]); 
					sw = true; 
				} 
				if(sw) break; 
			} 


			struct temp2 { float key; float key2; };
			fm::map<int, fm::vector<struct temp2> > temp;
			for(int i=0; i<compress.k.nNumberOfRemainingKey*gmlist.size(); i+=gmlist.size()) {
				for(int j=0; j<gmlist.size(); j++) {
					printf("Compare : %f, %f\n", gmlist[j], compress.coordIKeyValue.dKeyValue[i+j]);
					if(abs(gmlist[j] - compress.coordIKeyValue.dKeyValue[i+j]) > 0.01) {
						struct temp2 t;
						t.key = compress.k.dKey[i/gmlist.size()];
						t.key2 = compress.coordIKeyValue.dKeyValue[i+j];
						printf("ADD : %f %f, \n", t.key, t.key2);
						temp[j].push_back(t);
					}
				}
			}
			if (temp.empty()) break;
			
			for ( int ai=0; ai<anlib->GetEntityCount(); ai++) 
			{                

				an = anlib->GetEntity ( ai ); 

				anim    = an ;  
				anchn   = anim->GetChannel(0); 

				//FCDAnimationChannelData& data = FArchiveXML::GetChannelData(*anchn);
				//fm::string &tp = data.targetPointer;
				//fm::string &tq = data.targetQualifier;
				FCDExtra* extra = an->GetExtra();
				FCDEType* type = extra->FindType("AnimTargets");
				FCDETechnique* tech = type->FindTechnique("TEMP");
				fm::string tp = tech->GetChildNode(0)->GetContent();
				fm::string tq = tech->GetChildNode(1)->GetContent();

				//blagica
				size_t begin = tq.find( '(' );
				size_t end = tq.find( ')' );
				fstring tmp = tq.substr ( begin +1, end - begin  - 1 );
				int pos = atoi( tmp.c_str() );
				tmp = tq.substr(end+2,1);
				int coord = -1;
				if (tmp.length())
				{
					coord = tmp.c_str()[0] - 'X';
					if ( coord < 0 )
						coord = tmp.c_str()[0] - 'x';
					if (coord < 0 || coord > 2)
						coord = -1;
				}

				char** qual = new char*[anchn->GetCurveCount()];
				float** value = new float*[anchn->GetCurveCount()];
				for ( int curve = 0; curve < anchn->GetCurveCount(); curve++ )
				{
					if( coord >= 0 )
						if ( coord == 0 )
							qual[curve] = strdup(".X");
						else
							if ( coord == 1 )
								qual[curve] = strdup(".Y");
							else
								qual[curve] = strdup(".Z");
					else
						qual[curve] = NULL;
					value[curve] = new float[1];
					value[curve][0] = 0.f;
				}

				FCDAnimated* trans = NULL;

				trans = new FCDAnimated(anchn,anchn->GetCurveCount(),(const char**)qual, value);
				trans->SetArrayElement(pos);
				delete [] qual;
				delete [] value;

				int d; 
				sscanf(tq.c_str(), "(%d)", &d);
				if ( coord >=0 )
					d = d * 3 + coord;
				ancur   = anchn->GetCurve( 0 ); 

				//blagica
				trans->AddCurve(0, ancur);
				//blagica
				
				for ( int j = 0; j < ancur->GetKeyCount(); j++ ) { 
					//printf("%f ", ancur->GetKey(j)->input); 
					ancur->GetKey(j)->input = temp[d][j].key; 
				} 
				int k=0; 
				for(int i=0; i<ancur->GetKeyCount(); i++) { 
					for ( int j = 0; j < anchn->GetCurveCount(); j++ ) { 
						ancur   = anchn->GetCurve( j ); 
						//printf("%f ", ancur->GetKey(i)->output); 
						ancur->GetKey(i)->output = temp[d][k++].key2; 
						printf( "%f\n", ancur->GetKey(i)->output );
						//fprintf(f,"%f\n", ancur->GetKey(j)->output); 
					} 
				} 

				//blagica
				FUObjectContainer<FCDAnimated> &list = gmsC->GetAnimatedValues();
				list.push_back(trans);
				//blagica
			} 
			for (int tmp=0; tmp<36; tmp++)
				fprintf(f,"%f\n", compress.coordIKeyValue.dKeyValue[tmp]); 
			fclose(f);
			break; 
		} 
	default: 
		{ 
			DecodeMP4Streams *ds= new DecodeMP4Streams(); 
			MP4AnimationBuff *ab= NULL; 
			ds->DecodeBBAStream(buff, buffSize); 
			free ( buff ); 
			ab = ds->GetDecodedBBA(); 

			for ( int i=0; i<anlib->GetEntityCount(); i++) 
			{                

				an = anlib->GetEntity ( i ); 

				anim    = an ;  
				anchn   = anim->GetChannel(0); 


				/*FCDAnimationChannelData& data = FArchiveXML::GetChannelData(*anchn);
				fm::string &tp = data.targetPointer; 
				fm::string &tq = data.targetQualifier; */
				
				FCDExtra* extra = an->GetExtra();
				FCDEType* type = extra->FindType("AnimTargets");
				FCDETechnique* tech = type->FindTechnique("TEMP");
				fm::string tp = tech->GetChildNode(0)->GetContent();
				fm::string tq = tech->GetChildNode(1)->GetContent();
				

				if ( !tp.size() ) 
					continue; 
				fm::string tp1, tp2; 
				size_t splitIndex = tp.find_first_of("/"); 

				if(splitIndex != fm::string::npos) { 
					tp1 = tp.substr(0, splitIndex); 
					tp2 = tp.substr (splitIndex+1); 
				} else { 
					tp1 = tp; 
					tp2.clear(); 
				} 

				switch (GetTransformName(file, tp1, tp2)) 
				{ 
				case FCDTransform::Type::ROTATION:  
				case FCDTransform::Type::TRANSLATION:  
					//                                      case FCDTransform::Type::CI:  
					{ 


					} 
				case FCDTransform::Type::MATRIX: 
					{ 

						if ( !tq.size() ) 
						{ 
							ancur   = anchn->GetCurve( 0 ); 
							FCDAnimationKey** keys = ancur->GetKeys(); 
							cnt =  ancur->GetKeyCount(); 

							for ( int k =0; k<16; k++ ) 
							{ 
								ancur   = anchn->GetCurve( k ); 
								keys    = ancur->GetKeys(); 

								for (int l=0; l<ancur->GetKeyCount(); l++) 
								{ 
									int tx = k/4; 
									int ty = k%4; 
									keys[l]->output = ab->GetMP4Frame(l)->GetMP4BoneDataMatrix(i)[ty*4 + tx];//k e kelija vo matricata 
								} 
							} 

						}        
						else 
						{ 
							size_t first  = tq.find_first_of( "(" ); 
							size_t second = tq.find_last_of( ")" ); 
							char x = tq.at( first + 1 ); 
							char y = tq.at( second - 1 ); 

							int ix = x-'0'; 
							int iy = y-'0'; 

							ancur   = anchn->GetCurve( 0 ); 
							FCDAnimationKey** keys = ancur->GetKeys(); 


							for (int l=0; l<ancur->GetKeyCount(); l++) 
								keys[l]->output=ab->GetMP4Frame(l)->GetMP4BoneDataMatrix(i)[ix*4+iy]; 
						} 
					} 
				} 

			} 
			
			delete ds; 
		} 
		break; 
	} 

	return true;
}




bool MP4toColladaDecode::RestoreImages( FCDocument* file )
{
	unsigned char* buff= NULL;
	int size = 0;
	FCDImageLibrary *il = file->GetImageLibrary();
	
	if (! il || (il->GetEntityCount()==0) )
		return true;

	for (int i=0; i<il->GetEntityCount(); i++)
	{
		FCDImage *ie = il->GetEntity(i);
		MP4toVBLib_GetTexturebyRef(  ie->GetDaeId().c_str() , buff, size, true);
		
		DecodeMP4Streams *ds= new DecodeMP4Streams();
		
		ds->DecodeTextureStream((char*)buff, size);
		if (buff)
			free( buff );
		size = 0;
		
		ds->GetDecodedTexture( buff, size, true );
		
		const char * name = strrchr( ie->GetFilename().c_str(), '/' );
		if ( !name )
		{
			name = strrchr( ie->GetFilename().c_str(), '\\' );				
		}
		
		FILE *f;
		if ( name ) 
		{
			name ++;
			fstring t = name;
			ie->SetFilename( t );			
		}
		
		f = fopen( ie->GetFilename().c_str(), "wb" );
		fwrite( buff, 1,  size, f );
		fclose( f );

		if (buff)
			free( buff );
		size = 0;
		
		delete ds;
	}
	return true;
}



bool MP4toColladaDecode::RestoreColladaFile( const char * mp4FileName, const char* colladaFileName )
{
	MP4toVBLib_Init();
	MP4toVBLib_LoadMP4File( mp4FileName, NOTHING);
	
	printf( "Restoring XML stream\n" );
	RestoreXML( colladaFileName );

	FCDocument	*file	= NULL;
	FCollada::Initialize();

	printf( "Loading XML file\n" );
	file = FCollada::LoadDocument( (const fchar*) colladaFileName );

	if ( !file )
	{
		MP4toVBLib_Free();
		printf("Cannot open COLLADA rest file\n");
		return false;
	}
	
	printf( "Restoring Geometry\n" );
	RestoreGeometry( file );

	printf( "Restoring Animation\n" );
	RestoreAnimation( file );
	RestoreImages( file );
	
	MP4toVBLib_Free();

	printf( "Saving COLLADA file\n" );
	FCollada::SaveDocument( file, (const fchar*) colladaFileName );

	delete file;
	FCollada::Release();

	return true;
}

bool MP4toColladaDecode::RestoreCoordInterpolator( FCDocument* file, FCDGeometry *ge, std::vector<MeshFrame> &MeshVec )
{
	FCDAnimationLibrary *anlib  = NULL;
	FCDAnimation		*an		= NULL;
	FCDAnimationChannel *anchn	= NULL;
	FCDAnimationCurve	*ancur	= NULL;
	FCDAnimationKey		**keys  = NULL;
	int cnt = 0;

	anlib = file->GetAnimationLibrary();
	if (! anlib || (anlib->GetEntityCount()==0) )
		return false;

	FCDGeometryMesh *gm = ge->GetMesh();
	FCDGeometrySource *gs = NULL;
	for (int tm=0; tm<gm->GetSourceCount(); tm++ )
	{
		gs = gm->GetSource( tm );
		if (gs->GetType() == FUDaeGeometryInput::POSITION)
			break;
		gs = NULL;
	}
	if ( gs )
	{
		for ( int i = 0; i < anlib->GetEntityCount(); i++ )
		{
			an = anlib->GetEntity(i);

			anchn	= an->GetChannel(0);
			//FCDAnimationChannelData& data = FArchiveXML::GetChannelData(*anchn);
			fm::string tp;// = data.targetPointer;
			fm::string tq;// = data.targetQualifier;
			//fstring name = tp.substr(0, tp.find('/'));
			//fstring namefm = name + "_FAMC_";

			//add extra node
			FCDExtra * fe = NULL;
			FCDEType *ty = NULL;
			FCDETechnique *th = NULL;
			FCDENode *tn= NULL;
			fe = an->GetExtra();

			int tyc = fe->GetTypeCount();
			for ( int j = 0; j < tyc; ++j )
			{
				ty = fe->GetType(j);
				if ( ty->GetName() == "" )
					break;
				ty = NULL;
			}

			if ( !ty )
				continue;
			
			int tc = ty->GetTechniqueCount();
			for ( int j = 0; j < tc; ++j )
			{
				th = ty->GetTechnique(j);
				if ( !stricmp( th->GetProfile(),  "famc" ) )
					break;
				th = NULL;
			}
			
			if ( !th )
				continue;

			FCDENode* en = NULL;
			int ec = th->GetChildNodeCount();
			for ( int j = 0; j < ec; ++j )
			{
				en = th->GetChildNode(j);
				if ( !stricmp( en->GetName(),  "anim_target" ) )
					break;
				en = NULL;
			}

			if ( !en )
				continue;

			

			fstring daeid = en->GetContent();
			
			FUStringConversion::SplitTarget(daeid, tp, tq);

			FCDExtra* extra = an->GetExtra();
			FCDEType* type = extra->FindType("AnimTargets");
			FCDETechnique* tech = type->FindTechnique("TEMP");

			tech->GetChildNode(0)->SetContent(TO_FSTRING(tp));//data.targetPointer = tp;
			//anchn->SetTargetPointer(tp);
			tech->GetChildNode(1)->SetContent(tq);//data.targetQualifier = tq;
			//anchn->SetTargetQualifier(tq);

			if ( tp == gs->GetDaeId() )// || namefm == gs->GetDaeId() )
			{
				//if ( namefm == gs->GetDaeId() )
				//	gs->SetDaeId(gs->GetDaeId().substr(0,gs->GetDaeId().find("*FAMC*")));
				size_t begin = tq.find( '(' );
				size_t end = tq.find( ')' );
				fstring tmp = tq.substr ( begin +1, end - begin  - 1 );
				int pos = atoi( tmp.c_str() );
				tmp = tq.substr(end+2,1);
				int coord = -1;
				if (tmp.length())
				{
					coord = tmp.c_str()[0] - 'X';
					if ( coord < 0 )
						coord = tmp.c_str()[0] - 'x';
					if (coord < 0 || coord > 2)
						coord = -1;
				}

				char** qual = new char*[anchn->GetCurveCount()];
				float** value = new float*[anchn->GetCurveCount()];

				for ( int curve = 0; curve < anchn->GetCurveCount(); curve++ )
				{
					if( coord >= 0 )
					
						if ( coord == 0 )
							qual[curve] = strdup(".X");
						else
							if ( coord == 1 )
								qual[curve] = strdup(".Y");
							else
								qual[curve] = strdup(".Z");
					
					else
						qual[curve] = NULL;
					value[curve] = new float[1];
					value[curve][0] = 0.f;
				}

				FCDAnimated* trans = NULL;

				trans = new FCDAnimated(anchn,anchn->GetCurveCount(),(const char**)qual, value);
				trans->SetArrayElement(pos);
				delete [] qual;
				delete [] value;
				
				//FCDAnimated anim;
				//anim.SetTargetPointer(tp);
				//anim.SetArrayElement(pos);
				//gs->GetAnimatedValues().push_back(anim);
				int mult = 1;
				if ( coord >=0 )
					mult = 3;
				else
					coord = 0;

				for ( int j = 0; j < anchn->GetCurveCount(); j++ )
				{
					ancur = anchn->GetCurve(j);
					trans->AddCurve(j, ancur);
					keys	= ancur->GetKeys();
					for ( int k = 0; k < ancur->GetKeyCount(); k++ )
						ancur->GetKey(k)->output = MeshVec[k].mesh->GetCoordinate()[pos * mult + coord] - MeshVec[0].mesh->GetCoordinate()[pos * mult + coord];
				}
				FUObjectContainer<FCDAnimated> &list = gs->GetAnimatedValues();
				list.push_back(trans);
			}
		}
	}
	return true;
}