#include "X3DParser.h"


#include "gpac/nodes_x3d.h"

X3DParser::X3DParser(void)
{
	m_pX3DFileName		= NULL;
	m_pRestX3DFileName	= NULL;
	m_p3dmc				= NULL;
	m_3dmcCount			= 0;
	
}

X3DParser::~X3DParser(void)
{
	if ( m_pRestX3DFileName )
		free(m_pRestX3DFileName);
	if ( m_pX3DFileName )
		free(m_pX3DFileName);
}

bool X3DParser::LoadX3DFile( const char * fileName )
{
	if ( m_pX3DFileName )
		free(m_pX3DFileName);
	m_pX3DFileName = strdup(fileName);
	ParseX3DFile();
	return true;
}


bool X3DParser::ParseX3DFile()
{
	LoadFile();
	RemoveGeometry();
	
	return true;
}


bool X3DParser::LoadFile()
{
	memset(&m_lc,0, sizeof(m_lc));
	m_lc.scene_graph = gf_sg_new();
	m_lc.type = GF_SM_LOAD_X3D;
	m_lc.ctx = gf_sm_new( m_lc.scene_graph );

	FILE *f;
	f = fopen( m_pX3DFileName , "r" );
	fseek( f, 0, SEEK_END );
	int size = ftell( f );
	fseek( f, 0, SEEK_SET );
	char *str = (char*) malloc( size+1 );
	fread( str, 1, size, f );
	fclose( f );
	str[size]=0;
	
	//parse file in a string
	
	gf_sm_load_string( &m_lc, str, true );

	free( str );

	int cnt = 0;

	for ( unsigned int i = 0; i < gf_list_count(m_lc.scene_graph->all_nodes); i++ )
	{
		GF_Node* pNode = (GF_Node*)gf_list_get( m_lc.scene_graph->all_nodes, i );
		if ( pNode->sgprivate->tag == TAG_X3D_IndexedFaceSet )
			cnt++;
	}

	return true;
}

bool X3DParser::RemoveGeometry()
{
	int cnt = 0;

	for ( unsigned int i = 0; i < gf_list_count(m_lc.scene_graph->all_nodes); i++ )
	{
		GF_Node* pNode = (GF_Node*)gf_list_get( m_lc.scene_graph->all_nodes, i );
		if ( pNode->sgprivate->tag == TAG_X3D_IndexedFaceSet )
			cnt++;
	}
	
	m_p3dmc = new MP43DMC*[cnt];

	int cur = 0;
	for ( unsigned int i = 0; i < gf_list_count(m_lc.scene_graph->all_nodes); i++ )
	{
		GF_Node* pNode = (GF_Node*)gf_list_get( m_lc.scene_graph->all_nodes, i );
		if ( pNode->sgprivate->tag == TAG_X3D_IndexedFaceSet )
		{
			
			X_IndexedFaceSet* pIfs = (X_IndexedFaceSet*)pNode;
			X_Coordinate* pCoord = NULL;
			X_Normal* pNormal = NULL;
			X_TextureCoordinate* pTexCoord = NULL;
			X_Color* pColor = NULL;
			MFInt32* pCoordIndex = NULL;
			MFInt32* pNormalIndex = NULL;
			MFInt32* pTexCoordIndex = NULL;
			MFInt32* pColorIndex = NULL;
			
			int countN = 0;
			int countT = 0;
			int countC = 0;
			int countRGB = 0;
			int indexN = 0;
			int indexT = 0;
			int indexC = 0;
			int indexRGB = 0;

			if (pIfs->coord && pIfs->coord->sgprivate->tag == TAG_X3D_Coordinate)
			{
				pCoord = ( X_Coordinate* ) pIfs->coord;	
				countC = pCoord->point.count;
			}
			
			if ( !countC )
				continue;

			if (pIfs->normal && pIfs->normal->sgprivate->tag == TAG_X3D_Normal)
			{
				pNormal = ( X_Normal* ) pIfs->normal;	
				countN = pNormal->vector.count;
			}
		
			if (pIfs->texCoord && pIfs->texCoord->sgprivate->tag == TAG_X3D_TextureCoordinate)
			{
				pTexCoord = ( X_TextureCoordinate* ) pIfs->texCoord;	
				countC = pTexCoord->point.count;
			}
		
			if (pIfs->color && pIfs->color->sgprivate->tag == TAG_X3D_Color)
			{
				pColor = ( X_Color* ) pIfs->color;	
				countRGB = pColor->color.count;
			}
		
			u32 parCount = gf_node_get_parent_count( (GF_Node*) pCoord );

			for ( u32 j = 0; j < parCount; j++ )
			{
				X_IndexedFaceSet* par = (X_IndexedFaceSet*)gf_node_get_parent((GF_Node*)pCoord, j);
				if (par->coordIndex.count != 0 )
				{
					indexC += par->coordIndex.count;
					if ( pNormal )
					{
						indexN += par->coordIndex.count;
					}
					if ( pTexCoord )
					{
						indexT += par->coordIndex.count;
					}
					if ( pColor )
					{
						indexRGB += par->coordIndex.count;
					}
				}

				if (par->normalIndex.count != 0)
				{
					indexN += par->normalIndex.count;
				}

				if (par->texCoordIndex.count != 0)
				{
					indexT += par->texCoordIndex.count;
				}

				if (par->colorIndex.count != 0)
				{
					indexRGB += par->colorIndex.count;
				}
			}
			
			m_3dmcCount ++;
			u32 ifsid = gf_node_get_id( (GF_Node*) pIfs );
			if( !ifsid )
			{
				ifsid = gf_sg_get_next_available_node_id(m_lc.scene_graph);
				gf_node_set_id( (GF_Node*) pIfs, ifsid, NULL );
			}
			char ref[10] ; 
			itoa( gf_node_get_id( (GF_Node*) pIfs ), ref,10);
			m_p3dmc[cur] = new MP43DMC(countC, countN, countT, countRGB ,indexC/4, indexN/4, indexT/4, indexRGB/4, cur ,  ref );
	
			// Add coordinates
			if ( pCoord )
			{
				for( int j = 0; j <pCoord->point.count; j++ )
					m_p3dmc[cur]->AddCoordinate( pCoord->point.vals[j].x, pCoord->point.vals[j].y, pCoord->point.vals[j].z  );
				// remove coordinates 
				free( pCoord->point.vals );
				pCoord->point.vals = NULL;
				pCoord->point.count = 0;
			}

			// Add normals
			if ( pNormal )
			{
				for( int j = 0; j <pNormal->vector.count; j++ )
					m_p3dmc[cur]->AddNormal( pNormal->vector.vals[j].x, pNormal->vector.vals[j].y, pNormal->vector.vals[j].z  );
				// remove normals 
				free( pNormal->vector.vals );
				pNormal->vector.vals = NULL;
				pNormal->vector.count = 0;
			}

			// Add textures
			if (pTexCoord)
			{
				for( int j = 0; j <pTexCoord->point.count; j++ )
					m_p3dmc[cur]->AddTextureCoordinate( pTexCoord->point.vals[j].x, pTexCoord->point.vals[j].y);
			// remove textures
				free( pTexCoord->point.vals );
				pTexCoord->point.vals = NULL;
				pTexCoord->point.count = 0;
			}

			// Add color
			if ( pColor )
			{
				for( int j = 0; j <pColor->color.count; j++ )
					m_p3dmc[cur]->AddColor( pColor->color.vals[j].red, pColor->color.vals[j].green,  pColor->color.vals[j].blue  );
				// remove color
				free( pColor->color.vals );
				pColor->color.vals = NULL;
				pColor->color.count = 0;
			}

			for ( u32 k = 0; k < parCount; k++ )
			{
				//Add indecies
				X_IndexedFaceSet* par = (X_IndexedFaceSet*)gf_node_get_parent((GF_Node*)pCoord, k);
				if (par->coordIndex.count != 0 )
				{
					pCoordIndex = &par->coordIndex;
					indexC = par->coordIndex.count;
					if ( pNormal && !par->normalIndex.count )
					{
						indexN = par->coordIndex.count;
						pNormalIndex = &par->coordIndex;
					}
					if ( pTexCoord && !par->texCoordIndex.count )
					{
						indexT = par->coordIndex.count;
						pTexCoordIndex = &par->coordIndex;
					}
					if ( pColor && !par->colorIndex.count )
					{
						indexRGB = par->coordIndex.count;
						pColorIndex = &par->coordIndex;
					}
				}

				if (par->normalIndex.count != 0)
				{
					indexN = par->normalIndex.count;
					pNormalIndex = &par->normalIndex;
				}

				if (par->texCoordIndex.count != 0)
				{
					indexT = par->texCoordIndex.count;
					pTexCoordIndex = &par->texCoordIndex;
				}

				if (par->colorIndex.count != 0)
				{
					indexRGB = par->colorIndex.count;
					pColorIndex = &par->colorIndex;
				}
				//Add indecies
				if (indexC != 0 )
				{
					for( int j = 0; j <indexC; j+=4 )
						m_p3dmc[cur]->AddCoordinateIndex( pCoordIndex->vals[j+0], pCoordIndex->vals[j+1], pCoordIndex->vals[j+2] );
					//remove indices
					free( pCoordIndex->vals );
					pCoordIndex->vals = (s32*)malloc( 1 * sizeof(s32) );
					pCoordIndex->vals[0] = indexC;
					pCoordIndex->count = 1;
				}
				if (indexN != 0 )	
				{
					for( int j = 0; j <indexN; j+=4 )
						m_p3dmc[cur]->AddNormalIndex( pNormalIndex->vals[j+0], pNormalIndex->vals[j+1], pNormalIndex->vals[j+2] );
					//remove indices
					free( pNormalIndex->vals );
					pNormalIndex->vals = (s32*)malloc( 1 * sizeof(s32) );
					pNormalIndex->vals[0] = indexN;
					pNormalIndex->count = 1;
				}
				if (indexT != 0 )	
				{
					for( int j = 0; j <indexT; j+=4 )
						m_p3dmc[cur]->AddTextureCoordinateIndex( pTexCoordIndex->vals[j+0], pTexCoordIndex->vals[j+1], pTexCoordIndex->vals[j+2] );
					//remove indices
					free( pTexCoordIndex->vals );
					pTexCoordIndex->vals = (s32*)malloc( 1 * sizeof(s32) );
					pTexCoordIndex->vals[0] = indexT;
					pTexCoordIndex->count = 1;
				}
				if (indexRGB != 0 )	
				{
					for( int j = 0; j <indexRGB; j+=4 )
						m_p3dmc[cur]->AddColorIndex( pColorIndex->vals[j+0], pColorIndex->vals[j+1], pColorIndex->vals[j+2] );
					//remove indices
					free( pColorIndex->vals );
					pColorIndex->vals = (s32*)malloc( 1 * sizeof(s32) );
					pColorIndex->vals[0] = indexRGB;
					pColorIndex->count = 1;
				}
			}
			cur++;
		}
		
	}
	//dump new cene in file m_pRestX3DFileName
	if ( m_pRestX3DFileName )
		free(m_pRestX3DFileName);
	m_pRestX3DFileName = (char*) malloc ( strlen(m_pX3DFileName)+8);
	sprintf(m_pRestX3DFileName,"%stmp",m_pX3DFileName);
	gf_sm_dump( m_lc.ctx, m_pRestX3DFileName , GF_SM_DUMP_X3D_XML );
	//gf_sm_dump( m_lc.ctx, m_pRestX3DFileName , GF_SM_DUMP_X3D_XML);

	return true;
}


int X3DParser::Get3DMCStreamsCount()
{
	return m_3dmcCount;
}


MP43DMC* X3DParser::Get3DMCStream( int id )
{
	if ((id<0)||(id>=m_3dmcCount))
		return NULL;
	return m_p3dmc[id];
}

bool X3DParser::GetXMLrest ( char* &buff, int &size, bool alloc )
{
	FILE* f = fopen ( m_pRestX3DFileName, "rb" );
	if ( !f )
		return false;
	fseek(f,0, SEEK_END);
	size = ftell(f);
	fseek(f,0,0);
	if ( alloc )
		buff = (char*) malloc ( size );
	fread( buff, 1, size, f );
	fclose ( f );
	remove ( m_pRestX3DFileName );
	return true;
}
