using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Diagnostics;



namespace MP25
{
    public partial class MainForm : Form
    {
        EncoderParameters par;
        private string EncXmlFIle;
        private string EncOutDir;
        private string EncMP4FIle;

        private string DecXmlFIle;
        private string DecOutDir;
        private string DecMP4FIle;

        private string workingDir;

        public MainForm()
        {
            InitializeComponent();

            par = new EncoderParameters();
            EncXmlFIle = "";
            EncOutDir = "";
            EncMP4FIle = "";
        }

        private void btnEncAdvanced_Click(object sender, EventArgs e)
        {
            AdvancedEnc frm = new AdvancedEnc(par);
            frm.ShowDialog(this);
        }

        private void btnEncOpenXmlFile_Click(object sender, EventArgs e)
        {
            openDlg.Filter = "3D XML Files (*.dae,*.xmt,*.x3d)|*.dae;*.xmt;*.x3d|Collada Files (*.dae)|*.dae|XMT Files(*.xmt)|*.xmt|X3D FIles (*.x3d)|*.x3d|All Files (*.*)|*.*";
            if( openDlg.ShowDialog(this) == DialogResult.OK )
            {
                txtEncXmlFile.Text = openDlg.FileName;
            }
        }

        private void btnOpenOutDir_Click(object sender, EventArgs e)
        {
            if ( folderDlg.ShowDialog(this) == DialogResult.OK )
            {
                txtEncOutDir.Text = folderDlg.SelectedPath;
            }
        }

        private void btnDecOpenMp4File_Click(object sender, EventArgs e)
        {
            openDlg.Filter = "MP4 Files (*.mp4)|*.mp4|All Files (*.*)|*.*";
            if ( openDlg.ShowDialog(this) == DialogResult.OK )
            {
                txtDecMp4File.Text = openDlg.FileName;
            }
        }

        private void btnDecOpenOutDir_Click(object sender, EventArgs e)
        {
            if ( folderDlg.ShowDialog(this) == DialogResult.OK )
            {
                txtDecOutDir.Text = folderDlg.SelectedPath;
            }
        }

        private void btnEncStart_Click(object sender, EventArgs e)
        {
            txtEncXmlFile.Text = txtEncXmlFile.Text.TrimEnd(' ');
            if (txtEncXmlFile.Text == "")
            {
                MessageBox.Show(this, "Input file name empty", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                txtEncXmlFile.Focus();
                return;
            }
            txtEncOutDir.Text = txtEncOutDir.Text.TrimEnd(' ');
            if (txtEncOutDir.Text == "")
            {
                MessageBox.Show(this, "Output directory empty", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                txtEncOutDir.Focus();
                return;
            }
            txtEncMp4File.Text = txtEncMp4File.Text.TrimEnd(' ');
            if (txtEncMp4File.Text == "")
            {
                MessageBox.Show(this, "Output file name empty", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                txtEncMp4File.Focus();
                return;
            }
            if ( !File.Exists(txtEncXmlFile.Text))
            {
                MessageBox.Show(this, "Input file doesn't exsist", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                txtEncXmlFile.Focus();
                return;
            }

            EncXmlFIle = txtEncXmlFile.Text;
            EncOutDir = txtEncOutDir.Text;
            EncMP4FIle = txtEncMp4File.Text;

            Directory.SetCurrentDirectory(workingDir);

            ThreadStart trs = new ThreadStart(Encode);
            Thread tr = new Thread(trs);
            tr.Start();
            //Encode();
        }

        private void Encode()
        {
            Process exec = new Process();
            string name = "Test.exe";
            string args = "-e";

            args += " -input " + EncXmlFIle;
            args += " -output " + EncOutDir +"\\" +EncMP4FIle;

            args += " -dmc_bitsPerCoord " + par.m_3dmc_bits_coord.ToString();
            args += " -dmc_bitsPerNormal " + par.m_3dmc_bits_normal.ToString();
            args += " -dmc_bitsPerColor " + par.m_3dmc_bits_color.ToString();
            args += " -dmc_bitsPerTexCoord " + par.m_3dmc_bits_texcoord.ToString();

            if (!par.m_3dmc_order_vertex)
                args += " -dmc_vertex_order_flag";
            if (!par.m_3dmc_order_face)
                args += " -dmc_face_order_flag";

            args += " -bba_coding_type " + par.m_bba_coding.ToString();
            args += " -bba_p2q " + par.m_bba_p2qs.ToString();
            args += " -bba_qs " + par.m_bba_qs.ToString();

            args += " -famc_qCoord " + par.m_famc_bits_coord.ToString();
            args += " -famc_qNormal " + par.m_famc_bits_normal.ToString();
            args += " -famc_qColor " + par.m_famc_bits_color.ToString();
            args += " -famc_qOther " + par.m_famc_bits_texcoord.ToString();

            if (par.m_inter_coding == 0)
                args += " -FAMC";

            exec.StartInfo = new ProcessStartInfo(name, args);
            exec.Start();

            while (!exec.HasExited)
                Thread.Sleep(20);
        }

        private void button5_Click(object sender, EventArgs e)
        {
            txtDecMp4File.Text = txtDecMp4File.Text.TrimEnd(' ');
            if (txtDecMp4File.Text == "")
            {
                MessageBox.Show(this, "Input file name is empty", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                txtDecMp4File.Focus();
                return;
            }
            txtDecOutDir.Text = txtDecOutDir.Text.TrimEnd(' ');
            if (txtDecOutDir.Text == "")
            {
                MessageBox.Show(this, "Output directory is empty", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                txtDecOutDir.Focus();
                return;
            }
            txtDecXmlFile.Text = txtDecXmlFile.Text.TrimEnd(' ');
            if (txtDecXmlFile.Text == "")
            {
                MessageBox.Show(this, "Output file name is empty", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                txtDecXmlFile.Focus();
                return;
            }
            if (!File.Exists(txtDecMp4File.Text))
            {
                MessageBox.Show(this, "Input file doesn't exsist", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                txtDecMp4File.Focus();
                return;
            }

            DecXmlFIle = txtDecXmlFile.Text;
            DecOutDir = txtDecOutDir.Text;
            DecMP4FIle = txtDecMp4File.Text;

            Directory.SetCurrentDirectory(workingDir );

            ThreadStart trs = new ThreadStart(Decode);
            Thread tr = new Thread(trs);
            
            tr.Start();
            //Decode();
        }

        private void Decode()
        {
            Process exec = new Process();
            string name = "Test.exe";
            string args = "-d";

            args += " -input " +  DecMP4FIle;
            args += " -output " + DecOutDir + "\\" + DecXmlFIle;

            exec.StartInfo = new ProcessStartInfo(name, args);
            exec.Start();

            while (!exec.HasExited)
                Thread.Sleep(20);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            workingDir = Directory.GetCurrentDirectory();
        }

    }
}